//
//  ZBeeLib.h
//  Part of Z-Bee.C library
//
//  Created by Kirill Rogozhkin on 21/9/22.
//  Based on Z-Way source code written by Alex Skalozub and Serguei Poltorak
//
//  Copyright (c) 2022 Z-Wave.Me
//  All rights reserved
//  info@z-wave.me
//
//  This source file is subject to the terms and conditions of the
//  Z-Wave.Me Software License Agreement which restricts the manner
//  in which it may be used.
//

#ifndef zbee_h
#define zbee_h

#include "ZErrors.h"
#include "ZLog.h"
#include "ZDefsPublic.h"
#include "ZEzspTypes.h"
#include "ZDataExt.h"

// Allocate and initialize a ZBee object
//
// @param pzbee:     
// pointer that gets a ZBee object
// Note! ZBee structure must be filled with zeros first!
//
// @param port:     
// name of port to connect to.
// either serial port (COM1 or /dev/ttyUSB0),
//
// @param speed:
// UART speed in kbps
// 115200 by default
// 
// @param config_folder:
// path to config files
// NULL for current folder
// 
// @param translations_folder:
// path to translation files
// NULL for current folder
// 
// @param zddx_folder:
// path to zddx files
// NULL for current folder
//
// @param name
// Instance name to distinguish messages in log
//
// @param logger
// Logger to write messages to
//
ZWEXPORT ZWError zbee_init(ZBee *pzbee, ZWCSTR port, speed_t speed, ZWCSTR config_folder, ZWCSTR translations_folder, ZWCSTR zddx_folder, ZWCSTR name, ZWLog logger);

// Save state, close all handles and deallocate a ZBee object
//
// @param pzbee:
// pointer to ZBee object
// set to NULL on return
//
ZWEXPORT void zbee_terminate(ZBee *pzbee);

// Start worker thread and open port
//
// @param zbee:
// ZBee object instance
//
// @param termination_callback:
// function called on termination of Z-Bee worker thread (underlying device has gone)
// Syntax is (void) termination_callback(ZBee zbee)
// NULL if callback is not needed
// Note, that the callback is called in Z-Bee worker thread.
// Jobs are not processed anymore, but Data Holders are still available.
//
// @param arg:
// optional arg passed to termination callback
//
ZWEXPORT ZWError zbee_start(ZBee zbee, ZBTerminationCallback termination_callback, void* arg);

// Stop processing of commands and terminate worker thread
//
// @param zbee:
// ZBee object instance
//
ZWEXPORT ZWError zbee_stop(ZBee zbee);

// Returns logger instance used by zbee object
//
// @param zbee:
// ZBee object instance
//
ZWEXPORT ZWLog zbee_get_logger(ZBee zbee);

// Returns instance name for zbee object
//
// @param zbee:
// ZBee object instance
//
ZWEXPORT ZWCSTR zbee_get_name(ZBee zbee);

// Returns device id by Eui64 or -1 in case of invalid arguments or -2 if device is not found
//
// @param zbee:
// ZBee object instance
//
// @param eui64
// 8-byte Eui64 in little endian order
//
ZWEXPORT int zbee_get_device_id_by_eui64(ZBee zbee, ZWBYTE *eui64);

// Discover PAN ID, get network topology, version, capabilities etc.
// Function exits only then discovery finished (successfully or unsuccessfully).
//
// @param zbee:
// ZBee object instance
//
ZWEXPORT ZWError zbee_discover(ZBee zbee);

// Check if queue is empty (or has only jobs in state 'Done')
//
// @param zbee:
// ZBee object instance
//
ZWEXPORT ZWBOOL zbee_is_idle(ZBee zbee);

// Check that Z-Bee is still running (Z-Bee working thread still works).
// termination_callback parameter of zbee_start() can be used as alternative to this function.
//
// @param zbee:
// ZBee object instance
//
ZWEXPORT ZWBOOL zbee_is_running(ZBee zbee);

// Returns the time of last change in the list of devices
//
// @param zbee:
// ZBee object instance
//
ZWEXPORT time_t zbee_get_devices_update_time(const ZBee zbee);

// Returns the time of last change in the list endpoints of a device
//
// @param zbee:
// ZBee object instance
//
// @param node_id:
// NodeId
//
ZWEXPORT time_t zbee_get_endpoints_update_time(const ZBee zbee, ZBNWK node_id);

// Returns the time of last change in the list clusters of an endpoint of a device
//
// @param zbee:
// ZBee object instance
//
// @param node_id:
// NodeId
//
// @param endpoint_id:
// Endpoint Id
//
ZWEXPORT time_t zbee_get_clusters_update_time(const ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id);

// Send NoOperation to a node and wake up it's queue.
// Used to explicitelly check if mains powered is reachable or the battery device is currently awake and mark it as failed if not reachable.
//
// @param zbee:
// ZBee object instance
//
// @param node_id:
// NodeId
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
// Even on non-delivered packet successCallback is called after IsFailed call
// To check device reachability look on isFailed flag
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function success or failure
//
ZWEXPORT ZWError zbee_device_send_nop(ZBee zbee, ZBNWK node_id, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Force queue wakeup for a device
// Try to deliver jobs in the queue for the specified device.
// Note that jobs in the queue might be marked again as W (wait for wakeup) after first unsuccessful delivery.
//
// @param zbee:
// ZBee object instance
//
// @param node_id:
// NodeId
//
ZWEXPORT void zbee_device_awake_queue(const ZBee zbee, ZBNWK node_id);

// Attach callback function from Device change event.
// Duplicate address add of function callback is suppressed (one callback can be binded only once).
//
// @param zbee:
// Z-Bee object instance
//
// @param mask:
// Mask of which events to watch
//
// @param callback:
// Function to be attached to change event
//
// @param arg:
// Custom argument for callback function
//
ZWEXPORT ZWError zbee_device_add_callback(const ZBee zbee, ZBDeviceChangeType mask, ZBDeviceCallback callback, void *arg);

// Detach callback function from Device change event
// Delete all callbacks with same address of function callback
//
// @param zbee:
// Z-Bee object instance
//
// @param callback:
// Function to be detached from change event
//
ZWEXPORT ZWError zbee_device_remove_callback(const ZBee zbee, ZBDeviceCallback callback);

// Attach callback function from Device change event
// Allows many callbacks with same address of callback function (but supress duplicate callback/arg pair)
//
// @param zbee:
// Z-Bee object instance
//
// @param mask:
// Mask of which events to watch
//
// @param callback:
// Function to be attached to change event
//
// @param arg:
// Custom argument for callback function
//
ZWEXPORT ZWError zbee_device_add_callback_ex(const ZBee zbee, ZBDeviceChangeType mask, ZBDeviceCallback callback, void *arg);

// Detach callback function from Device change event
// Remove only once callback referred as function address and argument pair
//
// @param zbee:
// Z-Bee object instance
//
// @param callback:
// Function to be detached from change event
//
ZWEXPORT ZWError zbee_device_remove_callback_ex(const ZBee zbee, ZBDeviceCallback callback, void *arg);

// Run Cluster Interview.
//
// @param zbee:
// Z-Bee object instance
//
// @param device_id:
// Node Id
//
// @param endpoint_id:
// Endpoint Id
//
// @param cc_id:
// Cluster Id
//
ZWEXPORT ZWError zbee_cluster_interview(const ZBee zbee, ZBNWK device_id, ZWBYTE endpoint_id, ZBClusterId cc_id);

// Force re-interview of the device.
// Deletes Clusters and Endpoint structures,
// renders them from scratch based on the device interview.
//
// @param zbee:
// Z-Bee object instance
//
// @param device_id:
// Node Id
//
ZWEXPORT ZWError zbee_device_interview_force(const ZBee zbee, ZBNWK device_id);

// Check if device interview is done (all Clusters interview are done).
//
// @param zbee:
// Z-Bee object instance
//
// @param device_id:
// Node Id
//
ZWEXPORT ZWBOOL zbee_device_is_interview_done(const ZBee zbee, ZBNWK device_id);

// Start/stop Inclusion of a new node
//
// @param zbee:
// Z-Bee object instance
//
// @param startStop:
// Start inclusion mode if True, stop if False
//
ZWEXPORT ZWError zbee_controller_add_node_to_network(ZBee zbee, ZWBOOL startStop);

// Start/stop exclusion of a node
//
// @param: zbee
// Z-Bee object instance
//
// @param: node_id
// Node ID to remove
//
ZWEXPORT ZWError zbee_controller_remove_node_from_network(ZBee zbee, ZBNWK node_id);

// Reset the controller
// Note: this function will delete ALL data from the Zigbee chip and restore it to factory default !
//
// @param: zbee
// Z-Bee object instance
//
ZWEXPORT ZWError zbee_controller_set_default(ZBee zbee);

// Saves controller configuration, defaults and other needed files as tgz archive
//
// @param: zbee
// Z-Bee object instance
//
// @param: data
// Pointer to a variable that will receive an allocated data buffer
// It should be freed by caller when it is no longer needed
//
// @param: length
// Pointer to a variable that will receive data length
//
ZWEXPORT ZWError zbee_controller_config_save(ZBee zbee, ZWBYTE **data, size_t *length);

// Restores controller configuration, defaults and other needed files from tgz archive
// Note: This function restores network PAN ID, nodeId and information about nodes in network.
// 
// It has a lot of side effects! After this function consider:
// - run routing healing
//
// @param: zbee
// Z-Bee object instance
//
// @param: data
// Data buffer with archive contents
//
// @param: length
// Data length
//
// @param: full
// Perform full restore (including home id and network topology)
//
ZWEXPORT ZWError zbee_controller_config_restore(ZBee zbee, const ZWBYTE *data, size_t length, ZWBOOL full);

// Save all Z-Bee data to the disc
// This function runs at Z-Bee library termination
//
// Run this function from time to time to have current data for persistance.
//
// The file is named config/zddx/<panId>-ZigbeeData.xml (see config_folder parameter in zbee_init) and is read only on Z-Bee library startup.
//
// @param zbee:
// Z-Bee object instance
//
ZWEXPORT ZWError zbddx_save_to_xml(const ZBee zbee);

// Returns 0 terminated list of registered devices Node Id.
// Note that device 0 is not included in the list (since it is zero terminated), but is present via explicit addressing (if exists)
// NULL is returned if incorrect Z-Bee object is given.
//
// @param zbee:
// Z-Bee object instance
//
ZWEXPORT ZBDevicesList zbee_devices_list(const ZBee zbee);

// Free list of registered devices Node Id previously allocated by zbee_devices_list.
//
// @param list:
// List to be freed
//
ZWEXPORT void zbee_devices_list_free(ZBDevicesList list);

// Returns 0 terminated list of registered endpoints Id for specified device.
// Note, that endpoint 0 is not included in the list (since it is zero terminated), but is ALWAYS present via excplicit addressing..
// NULL is returned if incorrect Z-Bee object is given or device does not exist.
//
// @param zbee:
// Z-Bee object instance
//
// @param deviceId:
// Device in question
//
ZWEXPORT ZBEndPointsList zbee_endpoints_list(const ZBee zbee, ZBNWK deviceId);

// Free list of registered endpoints Id for specified device previously allocated by zbee_endpoints_list.
//
// @param list:
// List to be freed
//
ZWEXPORT void zbee_endpoints_list_free(ZBEndPointsList list);

// Returns 0 terminated list of registered Clusters Id for specified endpoint of device
// Note that cluster 0 is not included in the list (since it is zero terminated), but is present via explicit addressing (if exists)
// NULL is returned if incorrect Z-Bee object is given or device does not exist or endpoint does not exist.
//
// @param zbee:
// Z-Bee object instance
//
// @param deviceId:
// Device in question
//
// @param endpointId:
// Endpoint in question
//
ZWEXPORT ZBClustersList zbee_clusters_list(const ZBee zbee, ZBNWK deviceId, ZWBYTE endpointId, ZWBOOL inOut);

// Free list of registered Clusters Id for specified endpoint of device previously allocated by zbee_clusters_list.
//
// @param list:
// List to be freed
//
ZWEXPORT void zbee_clusters_list_free(ZBClustersList list);

// Returns if cluster both exists for specified endpoint of device, and is rendered as supported
//
// @param zbee:
// Z-Bee object instance
//
// @param node_id:
// Device in question
//
// @param node_id:
// Endpoint in question
//
// @param cluster_id:
// Cluster in question
//
ZWEXPORT ZWBOOL zbee_cluster_is_supported(const ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZBClusterId cluster_id);

// Returns the cluster name by id or NULL if not supported by the library
// 
// @param cl_id:
// Cluster in question
//
// @param profile_id:
// Zigbee Profile
//
ZWEXPORT ZWCSTR zbee_get_cluster_name(ZWBYTE cl_id, uint16_t profile_id);

// Prints into provided stream debug information about queue content.
// Queue includes pending jobs, processing jobs as well as done jobs for some period to help analyse outgoing queue.
// The output contains one job per line with few fields describing the job:
//  Send count
//  Status
//  Wait for Serial API ACK
//  Wait for Serial API Response
//  Wait for Serial API Callback
//  Wait for Device Reply
//  Timeout in ms
//  Destination node ID (or 0 if not to node)
//  Frame type
//  Serial API Function ID
//  Payload
//  Serial API Callback ID
//  Job description
// The format is %hhi %c %c %c %c %c %3.2f %3hhi ( %02hhX %02hhX %s %02hhX) %s\n
// Status:
//  D - done
//  W - waiting for wakeup of the device
//  S - waiting for security session
//  (space) - job to be processed
// Wait for Serial API ... and Wait for Reply:
//  (space) - no need to wait for message of that type
//  - - wait for message
//  + - message received
//
// @param zbee:
// Z-Bee object instance
//
// @param stream:
// Stream with text representation of the queue
//
ZWEXPORT void zbee_queue_inspect(const ZBee zbee, FILE * stream);

// Inject EZSP Packet to core
//
// @param zbee:
// Z-Bee object instance
//
// @param buffer:
// Raw packet
//
// @param stream:
// Length in bytes of raw packet
//
ZWEXPORT ZWError zbee_inject_ezsp_packet_raw(ZBee zbee, ZWBYTE *buffer, size_t size);

#endif