//
//  ClustersPublic.h
//  Part of Z-Matter.C library
//
//  Created by Kirill Rogozhkin on 11/11/22.
//  Based on Z-Way source code written by Alex Skalozub and
//
//  Copyright (c) 2022 Z-Wave.Me
//  All rights reserved
//  info@z-wave.me
//
//  This source file is subject to the terms and conditions of the
//  Z-Wave.Me Software License Agreement which restricts the manner
//  in which it may be used.
//

#ifndef zmatter_cluster_public_h
#define zmatter_cluster_public_h

#include "ZPlatform.h"
#include "ZDefsPublic.h"

// Cluster Class OnOff (0x0006/6) //
// Profile Zigbee Device Profile (0x0104/260) //
// Attributes and commands for switching devices between ‘On’ and ‘Off’ states.
//
// @DH	onOff			Current state
// @DH	globalSceneControl	Global Scene state
// @DH	onTime			Time the “on” state SHALL be maintained before automatically transitioning to the “off” state when using the On with timed off command. 
// @DH	offWaitTime		Time the “off” state SHALL be guarded to prevent an on command turning the device back to its “on” state

// Send On (0x00/0) or Off (0x01/1)
//
// @param: zmatter 
// ZMatter object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: value
// New onOff state:
//  TRUE  - send On command
//  FALSE - send Off command
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zmatter_cc_on_off_set(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZWBOOL value, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Send Toggle (0x02/2)
//
// @param: zmatter 
// ZMatter object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zmatter_cc_on_off_toggle(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Send Off with Effect (0x40/64)
//
// @param: zmatter 
// ZMatter object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: effect_id
// Effect Id:
//  0x00        - Delayed All Off
//  0x01        - Dying Light
//  0x02...0xFF - Reserved
//
// @param: effect_var
// Effect Variant:
//  if effect_id 0x00:
//  @default: 0x00
//   0x00        - Fade to off in 0.8 seconds
//   0x01        - No fade
//   0x02        - 50% dim down in 0.8 seconds then fade to off in 12 seconds
//   0x03...0xFF - Reserved
//  if effect_id 0x01:
//  @default: 0x00
//   0x00        - 20% dim up in 0.5s then fade to off in 1 second
//   0x01...0xFF - Reserved
//  if effect_id 0x02...0xFF:
//   0x00...0xFF - Reserved
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zmatter_cc_on_off_off_with_effect(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZWBYTE effect_id, ZWBYTE effect_var, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Send On with Recall Global Scene (0x41/65)
//
// @param: zmatter 
// ZMatter object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zmatter_cc_on_off_on_with_recall_global_scene(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Send On with Time Off (0x42/66)
//
// @param: zmatter 
// ZMatter object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: control
// Contains information on how the device is to be operated.
//  0x00        -  The On With Timed Off command SHALL be processed unconditionally.
//  0x01...0x7F - Reserved
//  0x80        - The On With Timed Off command SHALL only be accepted if the OnOff attribute is equal to 0x01.
//  0x81...0xFF - Reserved
//
// @param: on_time
// The length of time (in 1/10ths second) that the device is to remain “on”.
//  0x0000...0xFFFE - time in 1/10th second
//  0xFFFF          - Reserved
//
// @param: off_wait_time
// The length of time (in 1/10ths second) that the device SHALL remain “off”.
//  0x0000...0xFFFE - time in 1/10th second
//  0xFFFF          - Reserved
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zmatter_cc_on_off_on_with_timed_off(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZWBOOL control, ZWWORD on_time, ZWWORD off_wait_time, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Set Time of On Time 
//
// @param: zmatter 
// ZMatter object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: on_time
// The length of time (in 1/10ths second) that the device SHALL remain “off”.
//  0x0000...0xFFFE - time in 1/10th second
//  0xFFFF          - Reserved
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zmatter_cc_on_off_set_on_time(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZWWORD on_time, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Set Time of On with Time Off
//
// @param: zmatter 
// ZMatter object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: off_wait_time
// The length of time (in 1/10ths second) that the device SHALL remain “off”.
//  0x0000...0xFFFE - time in 1/10th second
//  0xFFFF          - Reserved
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zmatter_cc_on_off_set_off_wait_time(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZWWORD off_wait_time, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// On Off Get
//
// @param: zmatter 
// ZMatter object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_on_off_get(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// On Off Configuration Get
//
// @param: zmatter 
// ZMatter object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_on_off_configuration_get(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Set On Off Reporting
//
// @param: zmatter
// ZMatter object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: direction
// TRUE  - receiver will be configured to receiver reports
// FALSE - receiver will be configured to send reports
//
// @param: min_interval
// Minimal Reporting Interval
//
// @param: max_interval
// Maximum Reporting Interval
//
// @param: timeout
// Timeout
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_on_off_set_on_off_reporting(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZWBOOL direction, ZWWORD min_interval, ZWWORD max_interval, ZWWORD timeout, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Set On Off Reporting
//
// @param: zmatter
// ZMatter object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: direction
// TRUE  - receiver will be configured to receiver reports
// FALSE - receiver will be configured to send reports
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_on_off_reset_on_off_reporting(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZWBOOL direction, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

//-- Cluster Class PollControl (0x0020/32) //
// Profile Zigbee Device Profile (0x0104/260) //
// This cluster provides a mechanism for the management of an end device’s MAC Data Request rate. For the purposes of this cluster, the term “poll” always refers to the sending of a MAC Data Request from the end device to the end device’s parent.
//
// @DH	checkInInterval			The Check-inInterval represents the default amount of time between check-ins by the poll control server with the poll control client. The Check-inInterval is measured in quarterseconds. A value of 0 indicates that the Poll Control Server is turned off and the poll control server will not check-in with the poll control client.
// @DH	longPollInterval    	The LongPollInterval defines the frequency of polling that an end device does when it is NOT in fast poll mode. The LongPollInterval SHOULD be longer than the ShortPollInterval attribute but shorter than the Check-inInterval attribute.
// @DH	shortPollInterval		An end device that implements the Poll Control server MAY optionally expose the ShortPollInterval attribute. The ShortPollInterval represents the number of quarterseconds that an end device waits between MAC Data Requests to its parent when it is expecting data (i.e., in fast poll mode
// @DH	fastPollTimeout		    The FastPollTimeout attribute represents the number of quarterseconds that an end device will stay in fast poll mode by default. It is suggested that the FastPollTimeout attribute value be greater than 7.68 seconds.
// @DH	checkInIntervalMin		The Poll Control Server MAY optionally provide its own minimum value for the Check-inInterval to protect against the Check-inInterval being set too low and draining the battery on the end device implementing the Poll Control Server
// @DH	shortPollIntervalMin	The Poll Control Server MAY optionally provide its own minimum value for the LongPollInterval to protect against another device setting the value to too short a time resulting in an inadvertent power drain on the device.
// @DH	fastPollTimeoutMax		The Poll Control Server MAY optionally provide its own maximum value for the FastPollTimeout to avoid it being set to too high a value resulting in an inadvertent power drain on the device

// Fast Poll Stop
//
// @param: zmatter 
// ZMatter object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_poll_control_fast_poll_stop(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Set Long Poll Interval
//
// @param: zmatter 
// ZMatter object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: new_long_poll_interval
// New Long Poll Interval
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_poll_control_set_long_poll_interval(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, int new_long_poll_interval, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Set Short Poll Interval
//
// @param: zmatter 
// ZMatter object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: new_short_poll_interval
// New Short Poll Interval
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_poll_control_set_short_poll_interval(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZWWORD new_short_poll_interval, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Poll Control Configuration Get
//
// @param: zmatter 
// ZMatter object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_poll_control_configuration_get(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

//-- Cluster Class Basic (0x0000/0) //
// Profile ZigMatter Device Profile (0x0104/260) //
// This cluster supports an interface to the node or physical device. 
// It provides attributes and commands for determining basic information, 
// setting user information such as location, and resetting to factory defaults.
//
// @DH	zclVersion		The ZCLVersion attribute represents a published set of foundation items, such as global commands and functional descriptions.
// @DH	applicationVersion	The ApplicationVersion attribute is 8 bits in length and specifies the version number of the application software contained in the device. The usage of this attribute is manufacturer dependent
// @DH	stackVersion		The StackVersion attribute is 8 bits in length and specifies the version number of the implementation of the ZigMatter stack contained in the device. The usage of this attribute is manufacturer dependent.
// @DH	hwVersion		The HWVersion attribute is 8 bits in length and specifies the version number of the hardware of the device. The usage of this attribute is manufacturer dependent
// @DH	manufacturerName	The ManufacturerName attribute is a maximum of 32 bytes in length and specifies the name of the manufacturer as a ZigMatter character string
// @DH	modelIdentifier		The ModelIdentifier attribute is a maximum of 32 bytes in length and specifies the model number (or other identifier) assigned by the manufacturer as a ZigMatter character string.
// @DH	dateCode		The DateCode attribute is a ZigMatter character string with a maximum length of 16 bytes. The first 8 characters specify the date of manufacturer of the device in international date notation according to ISO 8601, i.e., YYYYMMDD, e.g., 20060814. The final 8 characters MAY include country, factory, line, shift or other related information at the option of the manufacturer. The format of this information is manufacturer dependent
// @DH	powerSource		The PowerSource attribute is 8 bits in length and specifies the source(s) of power available to the device. Bits b0–b6 of this attribute represent the primary power source of the device and bit b7 indicates whether the device has a secondary power source in the form of a battery backup.
// 				0x00 - Unknown
// 				0x01 - Mains (single phase)
// 				0x02 - Mains (3 phase)
// 				0x03 - Battery
// 				0x04 - DC Source
// 				0x05 - Emergency mains constrants
// 				0x06 - Emergency mains and transfer switch
// @DH	locationDescription	The LocationDescription attribute is a maximum of 16 bytes in length and describes the physical location of the device as a ZigMatter character string. This location description MAY be added into the device during commissioning
// @DH	physicalEnvironment	The PhysicalEnvironment attribute is 8 bits in length and specifies the type of physical environment in which the device will operate. This attribute SHALL be set to one of the non-reserved values. All values are valid for endpoints supporting all profiles except when noted
// 				0x00 - Unspecified environment
// 				0x01 - Mirror (ZSE Profile)
// 				0x01 - Atrium
// 				0x02 - Bar
// 				0x03 - Courtyard
// 				0x04 - Bathroom
// 				0x05 - Bedroom
// 				0x06 - Billiard Room
// 				0x07 - Utility Room
// 				0x08 - Cellar
// 				0x09 - Storage Closet
// 				0x0a - Theater
// 				0x0b - Office
// 				0x0c - Deck
// 				0x0d - Den
// 				0x0e - Dining Room
// 				0x0f - Electrical Room
// 				0x10 - Elevator
// 				0x11 - Entry
// 				0x12 - Family Room
// 				0x13 - Main Floor
// 				0x14 - Upstairs
// 				0x15 - Downstairs
// 				0x16 - Basement/Lower Level
// 				0x17 - Gallery
// 				0x18 - Game Room
// 				0x19 - Garage
// 				0x1a - Gym
// 				0x1b - Hallway
// 				0x1c - House
// 				0x1d - Kitchen
// 				0x1e - Laundry Room
// 				0x1f - Library
// 				0x20 - Master Bedroom
// 				0x21 - Mud Room (small room for coats and boots)
// 				0x22 - Nursery
// 				0x23 - Pantry
// 				0x24 - Office
// 				0x25 - Outside
// 				0x26 - Pool
// 				0x27 - Porch
// 				0x28 - Sewing Room
// 				0x29 - Sitting Room
// 				0x2a - Stairway
// 				0x2b - Yard
// 				0x2c - Attic
// 				0x2d - Hot Tub
// 				0x2e - Living Room
// 				0x2f - Sauna
// 				0x30 - Shop/Workshop
// 				0x31 - Guest Bedroom
// 				0x32 - Guest Bath
// 				0x33 - Powder Room (1/2 bath)
// 				0x34 - Back Yard
// 				0x35 - Front Yard
// 				0x36 - Patio
// 				0x37 - Driveway
// 				0x38 - Sun Room
// 				0x39 - Living Room
// 				0x3a - Spa
// 				0x3b - Whirlpool
// 				0x3c - Shed
// 				0x3d - Equipment Storage
// 				0x3e - Hobby/Craft Room
// 				0x3f - Fountain
// 				0x40 - Pond
// 				0x41 - Reception Room
// 				0x42 - Breakfast Room
// 				0x43 - Nook
// 				0x44 - Garden
// 				0x45 - Balcony
// 				0x46 - Panic Room
// 				0x47 - Terrace
// 				0x48 - Roof
// 				0x49 - Toilet
// 				0x4a - Toilet Main
// 				0x4b - Outside Toilet
// 				0x4c - Shower room
// 				0x4d - Study
// 				0x4e - Front Garden
// 				0x4f - Back Garden
// 				0x50 - Kettle
// 				0x51 - Television
// 				0x52 - Stove
// 				0x53 - Microwave
// 				0x54 - Toaster
// 				0x55 - Vacuum
// 				0x56 - Appliance
// 				0x57 - Front Door
// 				0x58 - Back Door
// 				0x59 - Fridge Door
// 				0x60 - Medication Cabinet Door
// 				0x61 - Wardrobe Door
// 				0x62 - Front Cupboard Door
// 				0x63 - Other Door
// 				0x64 - Waiting Room
// 				0x65 - Triage Room
// 				0x66 - Doctor’s Office
// 				0x67 - Patient’s Private Room
// 				0x68 - Consultation Room
// 				0x69 - Nurse Station
// 				0x6a - Ward
// 				0x6b - Corridor
// 				0x6c - Operating Theatre
// 				0x6d - Dental Surgery Room
// 				0x6e - Medical Imaging Room
// 				0x6f - Decontamination Room
// 				0xff - Unknown environment
// @DH	deviceEnabled		The DeviceEnabled attribute is a Boolean and specifies whether the device is enabled or disabled. This attribute SHALL be set to one of the non-reserved value 'Disabled' means that the device does not send or respond to application level commands, other than commands to read or write attributes. Values of attributes which depend on the operation of the application MAY be invalid, and any functionality triggered by writing to such attributes MAY be disabled. ZigMatter networking functionality remains operational. If implemented, the identify cluster cannot be disabled, i.e., it remains functional regardless of this setting
// 				0x00 - Disable
// 				0x01 - Enable
// @DH	alarmMask		The AlarmMask attribute is 8 bits in length and specifies which of a number of general alarms MAY be generated. These alarms are provided as basic alarms that a device MAY use even if no other clusters with alarms are present on the device
// 				0x00 - General hardware fault
// 				0x01 - General software fault
// @DH	disableLocalConfig	The DisableLocalConfig attribute allows a number of local device configuration functions to be disable. The intention of this attribute is to allow disabling of any local configuration user interface, for example to prevent reset or binding buttons being activated by non-authorized persons in a public building. Bit 0 of the DisableLocalConfig attribute disables any factory reset button (or equivalent) on the device. Bit 1 disables any device configuration button(s) (or equivalent)—for example, a bind button.
// 				0th bit
// 				 0 - Reset (to factory defaults) enabled
// 				 1 - Reset (to factory defaults) disabled
// 				1st bit
// 				 0 - Device configuration enabled
// 				 1 - Device configuration disabled
// @DH	swBuildId		The SWBuildID attribute represents a detailed, manufacturer-specific reference to the version of the software

// Basic Reset
//
// @param: zmatter 
// ZMatter object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_basic_reset(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Configuration Get
//
// @param: zmatter 
// ZMatter object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_basic_configuration_get(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id,ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

//-- Cluster Class Identify (0x0003/3) //
// Profile ZigMatter Device Profile (0x0104/260) //
// Attributes and commands to put a device into an Identification mode (e.g., flashing a light), that indicates to an observer – e.g., an installer - which of several devices it is, also to request any device that is identifying itself to respond to the initiator.
//
// @DH	identifyTime		The IdentifyTime attribute specifies the remaining length of time, in seconds, that the device will continue to identify itself.

// Identify Identify
//
// @param: zmatter 
// ZMatter object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: identify_time
// Identify Time
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_identify_identify(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZWWORD identify_time, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Identify Identify Query
//
// @param: zmatter 
// ZMatter object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_identify_identify_query(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Identify Trigger Effect
//
// @param: zmatter 
// ZMatter object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: effect_id
// The Effect Identifier field specifies the identify effect to use:
//  0x00 - Blink. e.g., Light is turned on/off once.
//  0x01 - Breathe. e.g., Light turned on/off over 1 second and repeated 15 times.
//  0x02 - Okay. e.g., Colored light turns green for 1 second; noncolored light flashes twice.
//  0x0b - Channel change. e.g., Colored light turns orange for 8 seconds; noncolored light switches to maximum brightness for 0.5s and then minimum brightness for 7.5s.
//  0xfe - Finish effect. Complete the current effect sequence before terminating. e.g., if in the middle of a breathe effect (as above), first complete the current 1s breathe effect and then terminate the effect.
//  0xff - Stop effect. Terminate the effect as soon as possible
// 
// @param: effect_var
// @default: 0x00
// Effect Variant:
//   0x00 - Default
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_identify_trigger_effect(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZWBYTE effect_id, ZWBYTE effect_var, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Identify Set Identify Time
//
// @param: zmatter 
// ZMatter object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: identify_time
// It specifies the remaining length of time, in seconds, that the device will continue to identify itself
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_identify_set_identify_time(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZWWORD identify_time, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Identify Get
//
// @param: zmatter 
// ZMatter object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_identify_get(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

//-- Cluster Class Ias Zone (0x0500/1280) //
// Profile ZigMatter Device Profile (0x0104/260) //
// The IAS Zone cluster defines an interface to the functionality of an IAS security zone device. IAS Zone supports up to two alarm types per zone, low battery reports and supervision of the IAS network
//
// @DH	zoneState	The ZoneState attribute contains the values summarized in Table.
// 			0x00 - NotEnroll
//			0x01 - Enroll   
// @DH	zoneType	The Zone Type dictates the meaning of Alarm1 and Alarm2 bits of the ZoneStatus attribute, as also indicated in this table.
//			0x0000 - StandardCEI
//			0x000d - MotionSensor
//			0x0015 - ContactSwitch
//			0x0028 - FireSensor
//			0x002a - WaterSensor
//			0x002b - COSensor
//			0x002c - PersonalEmergencyDevice
//			0x002d - VibrationMovementSensor
//			0x010f - RemoteControl
//			0x0115 - KeyFob
//			0x021d - Keypad
//			0x0225 - StandardWarningDevice
//			0x0226 - GlassBreakSensor
//			0x0229 - SecurityRepeater
//			0xffff - InvalidZoneType
// @DH	zoneStatus	The ZoneStatus attribute is a bit map.
//			0th bit (Alarm1)
//			 1 – opened or alarmed
//			 0 – closed or not alarmed
//			1st bit (Alarm2)
//			 1 – opened or alarmed
//			 0 – closed or not alarmed
//			2nd bit (Tamper)
//			 1 – Tampered
//			 0 – Not tampered
//			3rd bit (Battery)
//			 1 – Low battery
//			 0 – Battery OK
//			4th bit (Supervision reports (Note 1) )
//			 1 – Reports
//			 0 – Does not report
//			5th bit (Restore reports (Note 2) )
//			 1 – Reports restore
//			 0 – Does not report restore
//			6th bit (Trouble )
//			 1 – Trouble/Failure
//			 0 – OK
//			7th bit (AC (mains))
//			 1 – AC/Mains fault
//			 0 – AC/Mains OK
//			8th bit (Test )
//			 1 – Sensor is in test mode
//			 0 – Sensor is in operation mode
//			9th bit (Battery Defect)
//			 1 – Sensor detects a defective battery
//			 0 – Sensor battery is functioning normally
// @DH	iasCieAddress	The IAS_CIE_Address attribute specifies the address that commands generated by the server SHALL be sent to. All commands received by the server must also come from this address. It is up to the zone's specific implementation to permit or deny change (write) of this attribute at specific times
// @DH	zoneId		A unique reference number allocated by the CIE at zone enrollment time.
// 			0x00...0xFE - The unique identifier of zone
// 			0xFF - Invalid zone identifier
// @DH	numberOfZoneSensitivityLevelsSupported	Provides the total number of sensitivity levels supported by the IAS Zone server. 
// @DH	CurrentZoneSensitivityLevel	Allows an IAS Zone client to query and configure the IAS Zone server’s sensitivity level.
// @DH	delay		The Delay field is defined as the amount of time, in quarter-seconds, from the moment when a change takes place in one or more bits of the Zone Status attribute and the successful transmission of the Zone Status Change Notification. This is designed to help congested networks or offline servers quantify the amount of time from when an event was detected and when it could be reported to the client.

// IAS Zone Initiate Normal Operation Mode
//
// @param: zmatter 
// ZMatter object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_ias_zone_initiate_normal_operation_mode(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// IAS Zone Initiate Test Mode
//
// @param: zmatter 
// ZMatter object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: test_mode_duration
// Specifies the duration, in seconds, for which the IAS Zone server SHALL operate in its test mode
//
// @param: current_zone_sensivity_level
// Specifies the sensitivity level the IAS Zone server SHALL use for the duration of the Test Mode and with which it must update its CurrentZoneSensitivityLevel attribute.
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_ias_zone_initiate_test_mode(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZWBYTE test_mode_duration, ZWBYTE current_zone_sensivity_level, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Enroll Device
//
// @param: zmatter 
// ZMatter object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: zone_id
// Zone Identifier
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_ias_zone_enroll(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZWBYTE zone_id, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Ias Zone Get
//
// @param: zmatter 
// ZMatter object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_ias_zone_get(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Ias Zone Configuration Get
//
// @param: zmatter 
// ZMatter object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_ias_zone_configuration_get(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

//-- Cluster Class Color Control (0x0300/768) //
// Profile ZigMatter Device Profile (0x0104/260) //
// This cluster provides an interface for changing the color of a light. Color is specified according to the Commission Internationale de l'Éclairage (CIE) specification CIE 1931 Color Space. Color control is carried out in terms of x,y values, as defined by this specification.
//
// @DH	currentHue		It contains the current hue value of the light. It is updated as fast as practical during commands that change the hue. The hue in degrees SHALL be related to the CurrentHue attribute by the relationship Hue = CurrentHue x 360 / 254 (CurrentHue in the range 0 - 254 inclusive)
//				0x00...0xFE - valid
//				0xFF - invalid
// @DH	currentSaturation	It holds the current saturation value of the light. It is updated as fast as practical during commands that change the saturation. The saturation SHALL be related to the CurrentSaturation attribute by the relationship Saturation = CurrentSaturation/254 (CurrentSaturation in the range 0 - 254 inclusive)
//				0x00...0xFE - valid
//				0xFF - invalid
// @DH	remainingTime  		It holds the time remaining, in 1/10ths of a second, until the currently active command will be complete.
// @DH	currentX		The currentX attribute contains the current value of the normalized chromaticity value x, as defined in the CIE xyY Color Space. It is updated as fast as practical during commands that change the color. The value of x SHALL be related to the CurrentX attribute by the relationship x = CurrentX / 65536 (CurrentX in the range 0 to 65279 inclusive)
// @DH	currentY		The currentY attribute contains the current value of the normalized chromaticity value y, as defined in the CIE xyY Color Space. It is updated as fast as practical during commands that change the color. The value of y SHALL be related to the CurrentY attribute by the relationship y = CurrentY / 65536 (CurrentY in the range 0 to 65279 inclusive)
// @DH	driftCompensation	It indicates what mechanism, if any, is in use for compensation for color/intensity drift over time
//				0x00 - None
//				0x01 - Other / Unknown
//				0x02 - Temperature monitoring
//				0x03 - Optical luminance monitoring and feedback
//				0x04 - Optical color monitoring and feedback
// @DH	compensationText	It holds a textual indication of what mechanism, if any, is in use to compensate for color/intensity drift over time.
// @DH	colorTemperatureMireds	The ColorTemperatureMireds attribute contains a scaled inverse of the current value of the color temperature. The unit of ColorTemperatureMireds is the mired (micro reciprocal degree), AKA mirek (micro reciprocal kelvin). It is updated as fast as practical during commands that change the color. Color temperature in kelvins = 1,000,000 / ColorTemperatureMireds, where ColorTemperatureMireds is in the range 1 to 65279 mireds inclusive, giving a color temperature range from 1,000,000 kelvins to 15.32 kelvins.
// @DH	colorMode		It indicates which attributes are currently determining the color of the device. 
//				0x00 - CurrentHue and CurrentSaturation
//				0x01 - CurrentX and CurrentY
//				0x02 - ColorTemperatureMired
// @DH	enhancedCurrentHue	It represents non-equidistant steps along the CIE 1931 color triangle, and it provides 16-bits precision. The upper 8 bits of this attribute SHALL be used as an index in the implementation specific XY lookup table to provide the non-equidistance steps (see the ZLL test specification for an example). The lower 8 bits SHALL be used to interpolate between these steps in a linear way in order to provide color zoom for the user. To provide compatibility with standard ZCL, the CurrentHue attribute SHALL contain a hue value in the range 0 to 254, calculated from the EnhancedCurrentHue attribute.
// @DH	enhancedColorMode	It specifies which attributes are currently determining the color of the device.
//				0x00 - CurrentHue and CurrentSaturation
//				0x01 - CurrentX and CurrentY
//				0x02 - ColorTemperatureMireds
//				0x03 - EnhancedCurrentHue and CurrentSaturation
// @DH	colorLoopActive		It specifies the current active status of the color loop. 
//				0x00 - the color loop SHALL not be active
//				0x01 - the color loop SHALL be active
//				0x02...0xff - are reserved.
// @DH	colorLoopDirection	It specifies the current direction of the color loop. 
//				0x00	  - the EnhancedCurrentHue attribute SHALL be decremented
//				0x01	  - the EnhancedCurrentHue attribute SHALL be incremented
//				0x02...0xff - are reserved
// @DH	colorLoopTime		It specifies the number of seconds it SHALL take to perform a full color loop, i.e., to cycle all values of the EnhancedCurrentHue attribute (between 0x0000 and 0xffff)
// @DH	colorLoopStartEnhancedHue	It specifies the value of the EnhancedCurrentHue attribute from which the color loop SHALL be started
// @DH	colorLoopStoredEnhancedHue	It pecifies the value of the EnhancedCurrentHue attribute before the color loop was started. Once the color loop is complete, the EnhancedCurrentHue attribute SHALL be restored to this value.
// @DH	colorCapabilities	It specifies the color capabilities of the device supporting the color control cluster. If a bit is set to 1, the corresponding attributes and commands SHALL become mandatory. If a bit is set to 0, the corresponding attributes and commands need not be implemented
//				0th bit (Hue/saturation supported)
//				 1 - implemented attributes: CurrentHue, CurrentSaturation; commands: Move to hue, Move hue, Step hue, Move to saturation, Move saturation, Step saturation, Move to hue and saturation, Stop move step
//				 0 - not implemented
//				1st bit (Enhanced hue supported)
//				 1 - implemented attributes: EnhancedCurrentHue; commands: Enhanced move to hue, Enhanced move hue, Enhanced step hue, Enhanced move to hue and saturation, Stop move step
//				 0 - not implemented
//				2nd bit (Color loop supported)
//				 1 - implemented attributes: ColorLoopActive, ColorLoopDirection, ColorLoopTime, ColorLoopStartEnhancedHue, ColorLoopStoredEnhancedHue; commands: Color loop set
//				 0 - not implemented
//				3rd bit (XY attributes supported)
//				 1 - implemented attributes: CurrentX, CurrentY; commands: Move to color, Move color, Step color, Stop move step
//				 0 - not implemented
//				4th bit (Color temperature supported)
//				 1 - implemented attributes: ColorTemperatureMireds, ColorTempPhysicalMinMireds, ColorTempPhysicalMaxMireds; commands: Move to color temperature, Move color temperature, Step color temperature, Stop move step
//				 0 - not implemented 
// @DH	colorTempPhysicalMinMireds	It indicates the minimum mired value supported by the hardware. ColorTempPhysicalMinMireds corresponds to the maximum color temperature in kelvins supported by the hardware. ColorTempPhysicalMinMireds <= ColorTemperatureMireds.
// @DH	colorTempPhysicalMaxMireds	It indicates the maximum mired value supported by the hardware. ColorTempPhysicalMaxMireds corresponds to the minimum color temperature in kelvins supported by the hardware. ColorTemperatureMireds <= ColorTempPhysicalMaxMireds.
// @DH	numberOfPrimaries	It contains the number of color primaries implemented on this device. A value of 0xff SHALL indicate that the number of primaries is unknown Where this attribute is implemented, the attributes below for indicating the “x” and “y” color values of the primaries SHALL also be implemented for each of the primaries from 1 to NumberOfPrimaries, without leaving gaps. Implementation of the Primary1Intensity attribute and subsequent intensity attributes is optional.
//				0x00...0xFE - number of color primaries implemented on this device
//				0xFF - unknown
// @DH	primary1X		It contains the normalized chromaticity value x for this primary, as defined in the CIE xyY Color Space. The value of x SHALL be related to the Primary1X attribute by the relationship x = Primary1X / 65536 (Primary1X in the range 0 to 65279 inclusive)
//				0x0000...0xFEFF - valid
//				0xFFFF - invalid
// @DH	primary1Y		It contains the normalized chromaticity value y for this primary, as defined in the CIE xyY Color Space. The value of y SHALL be related to the Primary1Y attribute by the relationship y = Primary1Y / 65536 (Primary1Y in the range 0 to 65279 inclusive)
//				0x0000...0xFEFF - valid
//				0xFFFF - invalid
// @DH	primary1Intensity	It contains a representation of the maximum intensity of this primary as defined in the Dimming Light Curve in the Ballast Configuration cluster, normalized such that the primary with the highest maximum intensity contains the value 0xfe. A value of 0xff SHALL indicate that this primary is not available.
//				0x00...0xFE - valid
//				0xFF - invalid
// @DH	primary2X		It contains the normalized chromaticity value x for this primary, as defined in the CIE xyY Color Space. The value of x SHALL be related to the Primary2X attribute by the relationship x = Primary2X / 65536 (Primary2X in the range 0 to 65279 inclusive)
//				0x0000...0xFEFF - valid
//				0xFFFF - invalid
// @DH	primary2Y		It contains the normalized chromaticity value y for this primary, as defined in the CIE xyY Color Space. The value of y SHALL be related to the Primary2Y attribute by the relationship y = Primary2Y / 65536 (Primary2Y in the range 0 to 65279 inclusive)
//				0x0000...0xFEFF - valid
//				0xFFFF - invalid
// @DH	primary2Intensity	It contains a representation of the maximum intensity of this primary as defined in the Dimming Light Curve in the Ballast Configuration cluster, normalized such that the primary with the highest maximum intensity contains the value 0xfe. A value of 0xff SHALL indicate that this primary is not available.
//				0x00...0xFE - valid
//				0xFF - invalid
// @DH	primary3X		It contains the normalized chromaticity value x for this primary, as defined in the CIE xyY Color Space. The value of x SHALL be related to the Primary3X attribute by the relationship x = Primary3X / 65536 (Primary3X in the range 0 to 65279 inclusive)
//				0x0000...0xFEFF - valid
//				0xFFFF - invalid
// @DH	primary3Y		It contains the normalized chromaticity value y for this primary, as defined in the CIE xyY Color Space. The value of y SHALL be related to the Primary3Y attribute by the relationship y = Primary3Y / 65536 (Primary3Y in the range 0 to 65279 inclusive)
//				0x0000...0xFEFF - valid
//				0xFFFF - invalid
// @DH	primary3Intensity	It contains a representation of the maximum intensity of this primary as defined in the Dimming Light Curve in the Ballast Configuration cluster, normalized such that the primary with the highest maximum intensity contains the value 0xfe. A value of 0xff SHALL indicate that this primary is not available.
//				0x00...0xFE - valid
//				0xFF - invalid
// @DH	primary4X		It contains the normalized chromaticity value x for this primary, as defined in the CIE xyY Color Space. The value of x SHALL be related to the Primary4X attribute by the relationship x = Primary4X / 65536 (Primary4X in the range 0 to 65279 inclusive)
//				0x0000...0xFEFF - valid
//				0xFFFF - invalid
// @DH	primary4Y		It contains the normalized chromaticity value y for this primary, as defined in the CIE xyY Color Space. The value of y SHALL be related to the Primary4Y attribute by the relationship y = Primary4Y / 65536 (Primary4Y in the range 0 to 65279 inclusive)
//				0x0000...0xFEFF - valid
//				0xFFFF - invalid
// @DH	primary4Intensity	It contains a representation of the maximum intensity of this primary as defined in the Dimming Light Curve in the Ballast Configuration cluster, normalized such that the primary with the highest maximum intensity contains the value 0xfe. A value of 0xff SHALL indicate that this primary is not available.
//				0x00...0xFE - valid
//				0xFF - invalid
// @DH	primary5X		It contains the normalized chromaticity value x for this primary, as defined in the CIE xyY Color Space. The value of x SHALL be related to the Primary5X attribute by the relationship x = Primary5X / 65536 (Primary5X in the range 0 to 65279 inclusive)
//				0x0000...0xFEFF - valid
//				0xFFFF - invalid
// @DH	primary5Y		It contains the normalized chromaticity value y for this primary, as defined in the CIE xyY Color Space. The value of y SHALL be related to the Primary5Y attribute by the relationship y = Primary5Y / 65536 (Primary5Y in the range 0 to 65279 inclusive)
//				0x0000...0xFEFF - valid
//				0xFFFF - invalid
// @DH	primary5Intensity	It contains a representation of the maximum intensity of this primary as defined in the Dimming Light Curve in the Ballast Configuration cluster, normalized such that the primary with the highest maximum intensity contains the value 0xfe. A value of 0xff SHALL indicate that this primary is not available.
//				0x00...0xFE - valid
//				0xFF - invalid
// @DH	primary6X		It contains the normalized chromaticity value x for this primary, as defined in the CIE xyY Color Space. The value of x SHALL be related to the Primary6X attribute by the relationship x = Primary6X / 65536 (Primary6X in the range 0 to 65279 inclusive)
//				0x0000...0xFEFF - valid
//				0xFFFF - invalid
// @DH	primary6Y		It contains the normalized chromaticity value y for this primary, as defined in the CIE xyY Color Space. The value of y SHALL be related to the Primary6Y attribute by the relationship y = Primary6Y / 65536 (Primary6Y in the range 0 to 65279 inclusive)
//				0x0000...0xFEFF - valid
//				0xFFFF - invalid
// @DH	primary6Intensity	It contains a representation of the maximum intensity of this primary as defined in the Dimming Light Curve in the Ballast Configuration cluster, normalized such that the primary with the highest maximum intensity contains the value 0xfe. A value of 0xff SHALL indicate that this primary is not available.
//				0x00...0xFE - valid
//				0xFF	  - invalid
// @DH	whitePointX		It contains the normalized chromaticity value x, as defined in the CIE xyY Color Space, of the current white point of the device. The value of x SHALL be related to the WhitePointX attribute by the relationship x = WhitePointX / 65536 (WhitePointX in the range 0 to 65279 inclusive)
//				0x0000...0xFEFF - valid
//				0xFFFF - invalid
// @DH	whitePointY		It contains the normalized chromaticity value y, as defined in the CIE xyY Color Space, of the current white point of the device. The value of y SHALL be related to the WhitePointY attribute by the relationship y = WhitePointY / 65536 (WhitePointY in the range 0 to 65279 inclusive)
//				0x0000...0xFEFF - valid
//				0xFFFF - invalid
// @DH	colorPointRX		It contains the normalized chromaticity value x, as defined in the CIE xyY Color Space, of the red color point of the device. The value of x SHALL be related to the ColorPointRX attribute by the relationship x = ColorPointRX / 65536 (ColorPointRX in the range 0 to 65279 inclusive)
//				0x0000...0xFEFF - valid
//				0xFFFF - invalid
// @DH	colorPointRY		It contains the normalized chromaticity value y, as defined in the CIE xyY Color Space, of the red color point of the device. The value of y SHALL be related to the ColorPointRY attribute by the relationship y = ColorPointRY / 65536 (ColorPointRY in the range 0 to 65279 inclusive)
//				0x0000...0xFEFF - valid
//				0xFFFF - invalid
// @DH	colorPointRIntensity	It contains a representation of the relative intensity of the red color point as defined in the Dimming Light Curve in the Ballast Configuration cluster, normalized such that the color point with the highest relative intensity contains the value 0xfe. A value of 0xff SHALL indicate an invalid value.
//				0x00...0xFE - valid
//				0xFF - invalid
// @DH	colorPointGX		It contains the normalized chromaticity value x, as defined in the CIE xyY Color Space, of the green color point of the device. The value of x SHALL be related to the ColorPointGX attribute by the relationship x = ColorPointRX / 65536 (ColorPointRX in the range 0 to 65279 inclusive)
//				0x0000...0xFEFF - valid
//				0xFFFF - invalid
// @DH	colorPointGY		It contains the normalized chromaticity value y, as defined in the CIE xyY Color Space, of the green color point of the device. The value of y SHALL be related to the ColorPointGY attribute by the relationship y = ColorPointRY / 65536 (ColorPointRY in the range 0 to 65279 inclusive)
//				0x0000...0xFEFF - valid
//				0xFFFF - invalid
// @DH	colorPointGIntensity	It contains a representation of the relative intensity of the red color point as defined in the Dimming Light Curve in the Ballast Configuration cluster, normalized such that the color point with the highest relative intensity contains the value 0xfe. A value of 0xff SHALL indicate an invalid value.
//				0x00...0xFE - valid
//				0xFF - invalid
// @DH	colorPointBX		It contains the normalized chromaticity value x, as defined in the CIE xyY Color Space, of the blue color point of the device. The value of x SHALL be related to the ColorPointBX attribute by the relationship x = ColorPointRX / 65536 (ColorPointRX in the range 0 to 65279 inclusive)
//				0x0000...0xFEFF - valid
//				0xFFFF - invalid
// @DH	colorPointBY		It contains the normalized chromaticity value y, as defined in the CIE xyY Color Space, of the blue color point of the device. The value of y SHALL be related to the ColorPointBY attribute by the relationship y = ColorPointRY / 65536 (ColorPointRY in the range 0 to 65279 inclusive)
//				0x0000...0xFEFF - valid
//				0xFFFF - invalid
// @DH	colorPointBIntensity	It contains a representation of the relative intensity of the red color point as defined in the Dimming Light Curve in the Ballast Configuration cluster, normalized such that the color point with the highest relative intensity contains the value 0xfe. A value of 0xff SHALL indicate an invalid value.
//				0x00...0xFE - valid
//				0xFF - invalid

// Move to Hue (0x00/0)
//
// @param: zmatter 
// ZMatter object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: hue
// Hue
//
// @param: direction
// Direction:
//  0x00 - Shortest distance
//  0x01 - Longest distance
//  0x02 - Up
//  0x03 - Down
//  
// @param: transition_time
// The Transition Time field specifies, in 1/10ths of a second, the time that SHALL be taken to move to the new hue.
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_color_control_move_to_hue(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZWBYTE hue, ZWBYTE direction, ZWWORD transition_time, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Move Hue (0x01/1)
//
// @param: zmatter 
// ZMatter object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: move_mode
// Move mode
//  0x00 - Stop
//  0x01 - Up
//  0x02 - Reserved
//  0x03 - Down
//
// @param: rate
// The Rate field specifies the rate of movement in steps per second. A step is a change in the device’s hue of one unit. If the Rate field has a value of zero, the command has no effect and a default response command is sent in response, with the status code set to INVALID_FIELD
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_color_control_move_hue(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZWBYTE move_mode, ZWBYTE rate, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Step Hue (0x02/2)
//
// @param: zmatter 
// ZMatter object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: step_mode
// Move mode
//  0x00 - Reserved
//  0x01 - Up
//  0x02 - Reserved
//  0x03 - Down
//
// @param: step_size
// The change to be added to (or subtracted from) the current value of the device’s hue.
//
// @param: transition_time
// The Transition Time field specifies, in 1/10ths of a second, the time that SHALL be taken to perform the step. A step is a change in the device’s hue of ‘Step size’ units.
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_color_control_step_hue(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZWBYTE step_mode, ZWBYTE step_size, ZWBYTE transition_time, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Move to Saturation (0x03/3)
//
// @param: zmatter 
// ZMatter object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: saturation
// Desired Saturation
//
// @param: transition_time
// The Transition Time field specifies, in 1/10ths of a second, the time that SHALL be taken to perform the step.
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_color_control_move_to_saturation(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZWBYTE saturation, ZWWORD transition_time, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Move Saturation (0x04/4)
//
// @param: zmatter 
// ZMatter object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: move_mode
// Move Mode
//  0x00 - Stop
//  0x01 - Up
//  0x02 - Reserved
//  0x03 - Down
//
// @param: rate
// The Rate field specifies the rate of movement in steps per second. A step is a change in the device’s hue of one unit. If the Rate field has a value of zero, the command has no effect and a default response command is sent in response, with the status code set to INVALID_FIELD
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_color_control_move_saturation(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZWBYTE move_mode, ZWBYTE rate, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Step Saturation (0x05/5)
//
// @param: zmatter 
// ZMatter object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: step_mode
// Move mode
//  0x00 - Reserved
//  0x01 - Up
//  0x02 - Reserved
//  0x03 - Down
//
// @param: step_size
// The change to be added to (or subtracted from) the current value of the device’s saturation.
//
// @param: transition_time
// The Transition Time field specifies, in 1/10ths of a second, the time that SHALL be taken to perform the step. A step is a change in the device’s hue of ‘Step size’ units.
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_color_control_step_saturation(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZWBYTE step_mode, ZWBYTE step_size, ZWBYTE transition_time, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Move to Hue and Saturation (0x06/6)
//
// @param: zmatter 
// ZMatter object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: hue
// Desired hue
//
// @param: saturation
// Desired Saturation
//
// @param: transition_time
// The Transition Time field specifies, in 1/10ths of a second, the time that SHALL be taken to perform the step.
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_color_control_move_to_hue_and_saturation(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZWBYTE hue, ZWBYTE saturation, ZWWORD transition_time, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Move to Color (0x07/7)
//
// @param: zmatter 
// ZMatter object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: color_x
// Desired Color X
//
// @param: color_y
// Desired Color Y
//
// @param: transition_time
// The Transition Time field specifies, in 1/10ths of a second, the time that SHALL be taken to perform the step.
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_color_control_move_to_color(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZWWORD color_x, ZWWORD color_y, ZWWORD transition_time, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Move Color (0x08/8)
//
// @param: zmatter 
// ZMatter object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: rate_x
// The RateX field specifies the rate of movement in steps per second. A step is a change in the device's CurrentX attribute of one unit
//
// @param: rate_y
// The RateY field specifies the rate of movement in steps per second. A step is a change in the device's CurrentY attribute of one unit
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_color_control_move_color(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZWWORD rate_x, ZWWORD rate_y, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Step Color (0x09/9)
//
// @param: zmatter 
// ZMatter object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: step_x
// The StepX fields specify the change to be added to the device's CurrentX attribute attribute.
//
// @param: step_y
// The StepY fields specify the change to be added to the device's CurrentY attribute attribute.
//
// @param: transition_time
// The Transition Time field specifies, in 1/10ths of a second, the time that SHALL be taken to perform the step.
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_color_control_step_color(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZWWORD step_x, ZWWORD step_y, ZWWORD transition_time, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Step Color (0x0A/10)
//
// @param: zmatter 
// ZMatter object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: color_temp_mireds
// Move from its current color to the color given by the Color Temperature Mireds field.
//
// @param: transition_time
// The Transition Time field specifies, in 1/10ths of a second, the time that SHALL be taken to perform the step.
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_color_control_move_to_color_temperature(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZWWORD color_temp_mireds, ZWWORD transition_time, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Enhanced Move to Hue (0x40/64)
//
// @param: zmatter 
// ZMatter object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: enhanced_hue
// Enhanced Hue
//
// @param: direction
// Direction:
//  0x00 - Shortest distance
//  0x01 - Longest distance
//  0x02 - Up
//  0x03 - Down
//  
// @param: transition_time
// The Transition Time field specifies, in 1/10ths of a second, the time that SHALL be taken to move to the new hue.
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_color_control_enhance_move_to_hue(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZWWORD enhanced_hue, ZWBYTE direction, ZWWORD transition_time, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Enhanced Move Hue (0x41/65)
//
// @param: zmatter 
// ZMatter object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: move_mode
// Move mode
//  0x00 - Stop
//  0x01 - Up
//  0x02 - Reserved
//  0x03 - Down
//
// @param: rate
// The Rate field specifies the rate of movement in steps per second. A step is a change in the device’s hue of one unit. If the Rate field has a value of zero, the command has no effect and a default response command is sent in response, with the status code set to INVALID_FIELD
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_color_control_enhance_move_hue(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZWBYTE move_mode, ZWWORD rate, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Enhanced Step Hue (0x42/66)
//
// @param: zmatter 
// ZMatter object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: step_mode
// Move mode
//  0x00 - Reserved
//  0x01 - Up
//  0x02 - Reserved
//  0x03 - Down
//
// @param: step_size
// The change to be added to (or subtracted from) the current value of the device’s hue.
//
// @param: transition_time
// The Transition Time field specifies, in 1/10ths of a second, the time that SHALL be taken to perform the step. A step is a change in the device’s hue of ‘Step size’ units.
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_color_control_enhance_step_hue(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZWBYTE step_mode, ZWWORD step_size, ZWWORD transition_time, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Enhanced Move to Hue and Saturation (0x43/67)
//
// @param: zmatter 
// ZMatter object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: enhanced_hue
// Desired Enhanced Hue
//
// @param: saturation
// Desired Saturation
//
// @param: transition_time
// The Transition Time field specifies, in 1/10ths of a second, the time that SHALL be taken to perform the step.
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_color_control_enhance_move_to_hue_and_saturation(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZWWORD enhanced_hue, ZWBYTE saturation, ZWWORD transition_time, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Color Loop Set (0x44/68)
//
// @param: zmatter 
// ZMatter object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: update_flags
// The Update Flags field is 8 bits in length and specifies which color loop attributes to update before the color loop is started
//
// @param: action
// The Action field is 8 bits in length and specifies the action to take for the color loop if the Update Action sub-field of the Update Flags field is set to 1
//  0x00 - De-activate the color loop.
//  0x01 - Activate the color loop from the value in the ColorLoopStartEnhancedHue field.
//  0x02 - Activate the color loop from the value of the EnhancedCurrentHue attribute.
//
// @param: direction
// The Direction field is 8 bits in length and specifies the direction for the color loop if the Update Direction field of the Update Flags field is set to 1.
//  0x00 - Decrement the hue in the color loop.
//  0x01 - Increment the hue in the color loop.
//
// @param: time
// The Time field is 16 bits in length and specifies the number of seconds over which to perform a full color loop if the Update Time field of the Update Flags field is set to 1
//
// @param: start_hue
// The Start Hue field is 16 bits in length and specifies the starting hue to use for the color loop if the Update Start Hue field of the Update Flags field is set to 1
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_color_control_color_loop_set(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZWBYTE update_flags, ZWBYTE action, ZWBYTE direction, ZWWORD time, ZWWORD start_hue, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Stop Move Step (0x47/71)
//
// @param: zmatter 
// ZMatter object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: move_mode
// Move mode:
//  0x00 - Stop
//  0x01 - Up
//  0x02 - Reserved
//  0x03 - Down
//
// @param: rate
// The Rate field is 16-bits in length and specifies the rate of movement in steps per second. A step is a change in the color temperature of a device by one unit. If the Rate field has a value of zero, the command has no effect and a ZCL Default Response command SHALL be sent in response, with the status code set to INVALID_FIELD.
//
// @param: color_temp_min_mireds
// The Color Temperature Minimum Mireds field is 16-bits in length and specifies a lower bound on the ColorTemperatureMireds attribute (an upper bound on the color temperature in kelvins) for the current move operation such that: ColorTempPhysicalMinMireds <= Color Temperature Minimum Mireds field <= ColorTemperatureMireds As such if the move operation takes the ColorTemperatureMireds attribute towards the value of the Color Temperature Minimum Mireds field it SHALL be clipped so that the above invariant is satisfied. If the Color Temperature Minimum Mireds field is set to 0x0000, ColorTempPhysicalMinMireds SHALL be used as the lower bound for the ColorTemperatureMireds attribute.
//
// @param: color_temp_max_mireds
// The Color Temperature Maximum Mireds field is 16-bits in length and specifies an upper bound on the ColorTemperatureMireds attribute (a lower bound on the color temperature in kelvins) for the current move operation such that: ColorTemperatureMireds <= Color Temperature Maximum Mireds field <= ColorTempPhysicalMaxMireds As such if the move operation takes the ColorTemperatureMireds attribute towards the value of the Color Temperature Maximum Mireds field it SHALL be clipped so that the above invariant is satisfied. If the Color Temperature Maximum Mireds field is set to 0x0000, ColorTempPhysicalMaxMireds SHALL be used as the upper bound for the ColorTemperatureMireds attribute.
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_color_control_stop_move_step(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZWBYTE move_mode, ZWWORD rate, ZWWORD color_temp_min_mireds, ZWWORD color_temp_max_mireds, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Move Color Temperature (0x4b/75)
//
// @param: zmatter 
// ZMatter object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: move_mode
// Move mode:
//  0x00 - Stop
//  0x01 - Up
//  0x02 - Reserved
//  0x03 - Down
//
// @param: rate
// The Rate field is 16-bits in length and specifies the rate of movement in steps per second. A step is a change in the color temperature of a device by one unit. If the Rate field has a value of zero, the command has no effect and a ZCL Default Response command SHALL be sent in response, with the status code set to INVALID_FIELD.
//
// @param: color_temp_min_mireds
// The Color Temperature Minimum Mireds field is 16-bits in length and specifies a lower bound on the ColorTemperatureMireds attribute (an upper bound on the color temperature in kelvins) for the current move operation such that: ColorTempPhysicalMinMireds <= Color Temperature Minimum Mireds field <= ColorTemperatureMireds As such if the move operation takes the ColorTemperatureMireds attribute towards the value of the Color Temperature Minimum Mireds field it SHALL be clipped so that the above invariant is satisfied. If the Color Temperature Minimum Mireds field is set to 0x0000, ColorTempPhysicalMinMireds SHALL be used as the lower bound for the ColorTemperatureMireds attribute.
//
// @param: color_temp_max_mireds
// The Color Temperature Maximum Mireds field is 16-bits in length and specifies an upper bound on the ColorTemperatureMireds attribute (a lower bound on the color temperature in kelvins) for the current move operation such that: ColorTemperatureMireds <= Color Temperature Maximum Mireds field <= ColorTempPhysicalMaxMireds As such if the move operation takes the ColorTemperatureMireds attribute towards the value of the Color Temperature Maximum Mireds field it SHALL be clipped so that the above invariant is satisfied. If the Color Temperature Maximum Mireds field is set to 0x0000, ColorTempPhysicalMaxMireds SHALL be used as the upper bound for the ColorTemperatureMireds attribute.
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_color_control_move_color_temperature(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZWBYTE move_mode, ZWWORD rate, ZWWORD color_temp_min_mireds, ZWWORD color_temp_max_mireds, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Move Color Temperature (0x4c/76)
//
// @param: zmatter 
// ZMatter object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: step_mode
// Move mode:
//  0x00 - Reserved
//  0x01 - Up
//  0x02 - Reserved
//  0x03 - Down
//
// @param: step_size
// The Step Size field is 16-bits in length and specifies the change to be added to (or subtracted from) the current value of the device’s color temperature.
//
// @param: transition_time
// The Transition Time field is 16-bits in length and specifies, in units of 1/10ths of a second, the time that SHALL be taken to perform the step. A step is a change to the device’s color temperature of a magnitude corresponding to the Step Size field
//
// @param: color_temp_min_mireds
// The Color Temperature Minimum Mireds field is 16-bits in length and specifies a lower bound on the ColorTemperatureMireds attribute (an upper bound on the color temperature in kelvins) for the current move operation such that: ColorTempPhysicalMinMireds <= Color Temperature Minimum Mireds field <= ColorTemperatureMireds As such if the move operation takes the ColorTemperatureMireds attribute towards the value of the Color Temperature Minimum Mireds field it SHALL be clipped so that the above invariant is satisfied. If the Color Temperature Minimum Mireds field is set to 0x0000, ColorTempPhysicalMinMireds SHALL be used as the lower bound for the ColorTemperatureMireds attribute.
//
// @param: color_temp_max_mireds
// The Color Temperature Maximum Mireds field is 16-bits in length and specifies an upper bound on the ColorTemperatureMireds attribute (a lower bound on the color temperature in kelvins) for the current move operation such that: ColorTemperatureMireds <= Color Temperature Maximum Mireds field <= ColorTempPhysicalMaxMireds As such if the move operation takes the ColorTemperatureMireds attribute towards the value of the Color Temperature Maximum Mireds field it SHALL be clipped so that the above invariant is satisfied. If the Color Temperature Maximum Mireds field is set to 0x0000, ColorTempPhysicalMaxMireds SHALL be used as the upper bound for the ColorTemperatureMireds attribute.
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_color_control_step_color_temperature(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZWBYTE step_mode, ZWWORD step_size, ZWWORD transition_time, ZWWORD color_temp_min_mireds, ZWWORD color_temp_max_mireds, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Color Control Get Hue, Saturation
//
// @param: zmatter 
// ZMatter object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_color_control_get_hue_saturation(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Color Control Get Color
//
// @param: zmatter 
// ZMatter object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_color_control_get_color(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Color Control Get Remaining Time
//
// @param: zmatter 
// ZMatter object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_color_control_get_remaining_time(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Set Current Hue Reporting
//
// @param: zmatter
// ZMatter object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: direction
// TRUE  - receiver will be configured to receiver reports
// FALSE - receiver will be configured to send reports
//
// @param: min_interval
// Minimal Reporting Interval
//
// @param: max_interval
// Maximum Reporting Interval
//
// @param: timeout
// Timeout
//
// @param: threshold
// Threshold
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_color_control_set_current_hue_reporting(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZWBOOL direction, ZWWORD min_interval, ZWWORD max_interval, ZWWORD timeout, ZWBYTE threshold, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Reset Current Hue Reporting
//
// @param: zmatter
// ZMatter object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: direction
// TRUE  - receiver will be configured to receiver reports
// FALSE - receiver will be configured to send reports
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_color_control_reset_current_hue_reporting(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZWBOOL direction, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Set Current Saturation Reporting
//
// @param: zmatter
// ZMatter object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: direction
// TRUE  - receiver will be configured to receiver reports
// FALSE - receiver will be configured to send reports
//
// @param: min_interval
// Minimal Reporting Interval
//
// @param: max_interval
// Maximum Reporting Interval
//
// @param: timeout
// Timeout
//
// @param: threshold
// Threshold
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_color_control_set_current_saturation_reporting(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZWBOOL direction, ZWWORD min_interval, ZWWORD max_interval, ZWWORD timeout, ZWBYTE threshold, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Reset Current Saturation Reporting
//
// @param: zmatter
// ZMatter object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: direction
// TRUE  - receiver will be configured to receiver reports
// FALSE - receiver will be configured to send reports
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_color_control_reset_current_saturation_reporting(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZWBOOL direction, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Set Current X Reporting
//
// @param: zmatter
// ZMatter object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: direction
// TRUE  - receiver will be configured to receiver reports
// FALSE - receiver will be configured to send reports
//
// @param: min_interval
// Minimal Reporting Interval
//
// @param: max_interval
// Maximum Reporting Interval
//
// @param: timeout
// Timeout
//
// @param: threshold
// Threshold
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_color_control_set_current_x_reporting(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZWBOOL direction, ZWWORD min_interval, ZWWORD max_interval, ZWWORD timeout, ZWWORD threshold, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Reset Current X Reporting
//
// @param: zmatter
// ZMatter object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: direction
// TRUE  - receiver will be configured to receiver reports
// FALSE - receiver will be configured to send reports
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_color_control_reset_current_x_reporting(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZWBOOL direction, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Set Current Y Reporting
//
// @param: zmatter
// ZMatter object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: direction
// TRUE  - receiver will be configured to receiver reports
// FALSE - receiver will be configured to send reports
//
// @param: min_interval
// Minimal Reporting Interval
//
// @param: max_interval
// Maximum Reporting Interval
//
// @param: timeout
// Timeout
//
// @param: threshold
// Threshold
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_color_control_set_current_y_reporting(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZWBOOL direction, ZWWORD min_interval, ZWWORD max_interval, ZWWORD timeout, ZWWORD threshold, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Reset Current Y Reporting
//
// @param: zmatter
// ZMatter object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: direction
// TRUE  - receiver will be configured to receiver reports
// FALSE - receiver will be configured to send reports
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_color_control_reset_current_y_reporting(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZWBOOL direction, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Set Color Temperature Mireds Reporting
//
// @param: zmatter
// ZMatter object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: direction
// TRUE  - receiver will be configured to receiver reports
// FALSE - receiver will be configured to send reports
//
// @param: min_interval
// Minimal Reporting Interval
//
// @param: max_interval
// Maximum Reporting Interval
//
// @param: timeout
// Timeout
//
// @param: threshold
// Threshold
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_color_control_set_color_temperature_mireds_reporting(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZWBOOL direction, ZWWORD min_interval, ZWWORD max_interval, ZWWORD timeout, ZWWORD threshold, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Reset Color Temperature Mireds Reporting
//
// @param: zmatter
// ZMatter object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: direction
// TRUE  - receiver will be configured to receiver reports
// FALSE - receiver will be configured to send reports
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_color_control_reset_color_temperature_mireds_reporting(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZWBOOL direction, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Cluster Class Level Control (0x0008/8) //
// Profile ZigMatter Device Profile (0x0104/260) //
// This cluster provides an interface for controlling a characteristic of a device that can be set to a level, for example the brightness of a light, the degree of closure of a door, or the power output of a heater.
//
// @DH	currentLevel		It represents the current level of this device. The meaning of 'level' is device dependent.
// @DH	remainingTime		It represents the time remaining until the current command is complete - it is specified in 1/10ths of a second.
// @DH	onOffTransitionTime	It represents the time taken to move to or from the target level when On of Off commands are received by an On/Off cluster on the same endpoint. It is specified in 1/10ths of a second. The actual time taken SHOULD be as close to OnOffTransitionTime as the device is able. N.B. If the device is not able to move at a variable rate, the OnOffTransitionTime attribute SHOULD not be implemented.
// @DH	onLevel			It determines the value that the CurrentLevel attribute is set to when the OnOff attribute of an On/Off cluster on the same endpoint is set to On. If the OnLevel attribute is not implemented, or is set to 0xff, it has no effect.
// @DH	onTransitionTime	It represents the time taken to move the current level from the minimum level to the maximum level when an On command is received by an On/Off cluster on the same endpoint. It is specified in 10ths of a second. If this command is not implemented, or contains a value of 0xffff, the On/OffTransitionTime will be used instead
// @DH	offTransitionTime	It represents the time taken to move the current level from the maximum level to the minimum level when an Off command is received by an On/Off cluster on the same endpoint. It is specified in 10ths of a second. If this command is not implemented, or contains a value of 0xffff, the On/OffTransitionTime will be used instead.
// @DH	defaultMoveRat		It determines the movement rate, in units per second, when a Move command is received with a Rate parameter of 0xFF

// Move to Level (0x00/0)
//
// @param: zmatter 
// ZMatter object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: level
// Device dependent level
//
// @param: transition_time
// @default: 0
// Transition time
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zmatter_cc_level_control_move_to_level(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZWBYTE level, ZWWORD transition_time, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Move (0x01/1)
//
// @param: zmatter 
// ZMatter object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: move_mode
// Move mode:
//  0x00 - Up
//  0x01 - Down
//
// @param: rate
// @default: 0xFF
// The Rate field specifies the rate of movement in units per second:
//  0x00...0xFE - used value  
//  0xFF        - used value from DefaultMoveRate
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_level_control_move(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZWBYTE move_mode, ZWBYTE rate, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Step (0x02/2)
//
// @param: zmatter 
// ZMatter object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: step_mode
// Move mode:
//  0x00 - Up
//  0x01 - Down
//
// @param: step_size
// Step size
//
// @param: transition_time
// @default: 0
// The Transition time field specifies the time that SHALL be taken to perform the step, in tenths of a second.
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_level_control_step(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZWBYTE step_mode, ZWBYTE step_size, ZWWORD transition_time, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Stop (0x03/3)
//
// @param: zmatter 
// ZMatter object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_level_control_stop(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Move to Level (with On/Off) (0x04/4)
//
// @param: zmatter 
// ZMatter object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: level
// Device dependent level
//
// @param: transition_time
// @default: 0
// Transition time
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zmatter_cc_level_control_move_to_level_on_off(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZWBYTE level, ZWWORD transition_time, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Move (with On/Off) (0x05/5)
//
// @param: zmatter 
// ZMatter object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: move_mode
// Move mode:
//  0x00 - Up
//  0x01 - Down
//
// @param: rate
// @default: 0xFF
// The Rate field specifies the rate of movement in units per second:
//  0x00...0xFE - used value  
//  0xFF        - used value from DefaultMoveRate
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_level_control_move_on_off(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZWBYTE move_mode, ZWBYTE rate, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Step (with On/Off) (0x06/6)
//
// @param: zmatter 
// ZMatter object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: step_mode
// Move mode:
//  0x00 - Up
//  0x01 - Down
//
// @param: step_size
// Step size
//
// @param: transition_time
// @default: 0
// The Transition time field specifies the time that SHALL be taken to perform the step, in tenths of a second.
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_level_control_step_on_off(ZMatter zmatter,ZMNWK node_id, ZWBYTE endpoint_id, ZWBYTE step_mode, ZWBYTE step_size, ZWWORD transition_time, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Stop (with On/Off) (0x07/7)
//
// @param: zmatter 
// ZMatter object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_level_control_stop_on_off(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Level Control Get
//
// @param: zmatter 
// ZMatter object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_level_control_get(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Level Control Configuration Get
//
// @param: zmatter 
// ZMatter object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_level_control_configuration_get(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Set Current Level Reporting
//
// @param: zmatter
// ZMatter object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: direction
// TRUE  - receiver will be configured to receiver reports
// FALSE - receiver will be configured to send reports
//
// @param: min_interval
// Minimal Reporting Interval
//
// @param: max_interval
// Maximum Reporting Interval
//
// @param: timeout
// Timeout
//
// @param: threshold
// Threshold
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_level_control_set_current_level_reporting(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZWBOOL direction, ZWWORD min_interval, ZWWORD max_interval, ZWWORD timeout, ZWBYTE threshold, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Reset Current Level Reporting
//
// @param: zmatter
// ZMatter object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: direction
// TRUE  - receiver will be configured to receiver reports
// FALSE - receiver will be configured to send reports
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_level_control_reset_current_level_reporting(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZWBOOL direction, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

//-- Cluster Class Door Lock (0x0101/257) //
// Profile ZigMatter Device Profile (0x0104/260) //
// The door lock cluster provides an interface to a generic way to secure a door. The physical object that provides the locking functionality is abstracted from the cluster. The cluster has a small list of mandatory attributes and functions and a list of optional features.
//
// @DH	lockState	It describes lock states.
//				0x00 - Not fully locked
//				0x01 - Locked
//				0x02 - Unlocked
//				0xFF - Undefined
// @DH	lockType	It describes lock types.
//				0x00 - Dead bolt
//				0x01 - Magnetic
//				0x02 - Other
//				0x03 - Mortise
//				0x04 - Rim
//				0x05 - Latch Bolt
//				0x06 - Cylindrical Lock
//				0x07 - Tubular Lock
//				0x08 - Interconnected Lock
//				0x09 - Dead Latch
//				0x0A - Door Furniture
// @DH	actuatorEnabled	It describes if actuator is enable or not
//				0 - Disabled
//				1 - Enabled
// @DH	doorState	It describes possible door states.
//				0x00 Open
//				0x01 Closed
//				0x02 Error (jammed)
//				0x03 Error (forced open)
//				0x04 Error (unspecified)
//				0xFF Undefined
// @DH	doorOpenEvents					This attribute holds the number of door open events that have occurred since it was last zeroed
// @DH	doorClosedEvents				This attribute holds the number of door closed events that have occurred since it was last zeroed.
// @DH	openPeriod					This attribute holds the number of minutes the door has been open since the last time it transitioned from closed to open.
// @DH	numberOfLogRecordsSupported			The number of available log records
// @DH	numberOfTotalUsersSupported			Number of total users supported by the lock. This value is equal to the higher one of [# of PIN Users Supported] and [# of RFID Users Supported]
// @DH	numberOfPINUsersSupported			The number of PIN users supported.
// @DH	numberOfRFIDUsersSupported			The number of RFID users supported.
// @DH	numberOfWeekDaySchedulesSupportedPerUser	The number of configurable week day schedule supported per user.
// @DH	numberOfYearDaySchedulesSupportedPerUser	The number of configurable year day schedule supported per user
// @DH	numberOfHolidaySchedulesSupported		The number of holiday schedules supported for the entire door lock device.
// @DH	maxPINCodeLength				An 8 bit value indicates the maximum length in bytes of a PIN Code on this device. The default is set to 8 since most lock manufacturers currently allow PIN Codes of 8 bytes or less
// @DH	minPINCodeLength				An 8 bit value indicates the minimum length in bytes of a PIN Code on this device. The default is set to 4 since most lock manufacturers do not support PIN Codes that are shorter than 4 bytes.
// @DH	maxRFIDCodeLength				An 8 bit value indicates the maximum length in bytes of a RFID Code on this device. The value depends on the RFID code range specified by the manufacturer, if media anti-collision identifiers (UID) are used as RFID code, a value of 20 (equals 10 Byte ISO 14443A UID) is recommended
// @DH	minRFIDCodeLength				An 8 bit value indicates the minimum length in bytes of a RFID Code on this device. The value depends on the RFID code range specified by the manufacturer, if media anti-collision identifiers (UID) are used as RFID code, a value of 8 (equals 4 Byte ISO 14443A UID) is recommended.
// @DH	enableLogging					Enable/disable event logging. When event logging is enabled, all event messages are stored on the lock for retrieval. Logging events can be but not limited to Tamper Alarm, Lock, Unlock, Autolock, User Code Added, User Code Deleted, Schedule Added, and Schedule Deleted. For a full detail of all the possible alarms and events, please refer to the full list in the Alarm and Event Masks Attribute Set.
// @DH	language					Modifies the language for the on-screen or audible user interface using three bytes from ISO-639-1. It consists of one byte of length and two bytes for the language code. For example if the language is set to English, the value would be "02 65 6E" for the language code "en".
// @DH	ledSettings					The settings for the LED support three different modes
//								0x00 - Never use LED for signalization
//								0x01 - Use LED signalization except for access allowed events
//								0x02 - Use LED signalization for all events
// @DH	autoRelockTime				The number of seconds to wait after unlocking a lock before it automatically locks again. 0=disabled. If set, unlock operations from any source will be timed. For one time unlock with timeout use the specific command.                
// @DH	soundVolume				The sound volume on a door lock has three possible setting
//				0x00 Silent Mode
//				0x01 Low Volume
//				0x02 High Volume
// @DH	operatingMode				It describes operating modes
//				0x00 - Normal            KeyPad - E   RF - E    E   - RFID
//				0x01 - Vacation          KeyPad - D   RF - E    E   - RFID
//				0x02 - Privacy           KeyPad - D   RF - D    D   - RFID
//				0x03 - No RF Lock/Unlock KeyPad - E   RF - D    E   - RFID
//				0x04 - Passage           KeyPad - N/A RF - N/A  N/A - RFID               
// @DH	supportedOperatingModes			This bitmap contains all operating bits of the Operating Mode Attribute supported by the lock.
//				0 - Normal Mode Supported
//				1 - Vacation Mode Supported
//				2 - Privacy Mode Supported
//				3 - No RF Lock or Unlock Mode Supported
//				4 - Passage Mode Supported
// @DH	defaultConfigurationRegister		This attribute represents the default configurations as they are physically set on the device          
//				0th bit
//				 1 - Enable Local Programming Attribute default value is 0 (disabled)
//				 0 - Enable Local Programming Attribute default value is 1 (enabled)
//				1st bit
//				 1 - Keypad Interface default access is disabled
//				 0 - Keypad Interface default access is enabled
//				2nd bit
//				 1 - RF Interface default access is disabled
//				 0 - RF Interface default access is enabled
//				5th bit
//				 1 -  Sound Volume attribute default value is 0 (Slight Mode)
//				 0 -  Sound Volume attribute default value is equal to something other than 0x00
//				6th bit
//				 1 -  Auto Relock Time attribute default value = 0x00
//				 0 -  Auto Relock Time attribute default value is equal to something other than 0x00
//				7th bit
//				 1 - Led Settings attribute default value = 0x00
//				 0 - Led Settings attribute default value is equal to something other than 0x00
// @DH	enableLocalProgramming			Enable/disable local programming on the door lock. The local programming features includes but not limited to adding new user codes, deleting existing user codes, add new schedule, deleting existing schedule on the local door lock interfaces. If this value is set to 0x01 or TRUE then local programming is enabled on the door lock. If it is set to 0x00 or FALSE then local programming is disabled on the door lock. Local programming is enabled by default.
// @DH	enableOneTouchLocking			Enable/disable the ability to lock the door lock with a single touch on the door lock
// @DH	enableInsideStatusLED			Enable/disable an inside LED that allows the user to see at a glance if the door is locked.
// @DH	enablePrivacyModeButton			Enable/disable a button inside the door that is used to put the lock into privacy mode. When the lock is in privacy mode it cannot be manipulated from the outside
// @DH	wrongCodeEntryLimit			The number of incorrect codes or RFID presentment attempts a user is allowed to enter before the door will enter a lockout state. The lockout state will be for the duration of UserCodeTemporaryDisableTime
// @DH	userCodeTemporaryDisableTime		The number of seconds that the lock shuts down following wrong code entry. 1-255 seconds. Device can shutdown to lock user out for specified amount of time. (Makes it difficult to try and guess a PIN for the device.)
// @DH	sendPINOverTheAir			Boolean set to True if it is ok for the door lock server to send PINs over the air. This attribute determines the behavior of the server’s TX operation. If it is false, then it is not ok for the device to send PIN in any messages over the air. The PIN field within any door lock cluster message SHALL keep the first octet unchanged and masks the actual code by replacing with 0xFF. For example (PIN "1234" ): If the attribute value is True, 0x04 0x31 0x32 0x33 0x34 SHALL be used in the PIN field in any door lock cluster message payload. If the attribute value is False, 0x04 0xFF 0xFF 0xFF 0xFF SHALL be used
// @DH	requirePINforRFOperation		Boolean set to True if the door lock server requires that an optional PINs be included in the payload of RF lock operation events like Lock, Unlock and Toggle in order to function.
// @DH	zigMatterSecurityLevel			The Security Level attribute allows the door lock manufacturer to indicate what level of security the door lock requires.
//				0 - Network Security (default)
//				1 - APS Security
// @DH	alarmMask	The alarm mask is used to turn on/off alarms for particular functions
//				0th bit - 0x00 (Deadbolt Jammed)
//				1st bit - 0x01 (Lock Reset to Factory Defaults)
//				2nd bit - 0x02 (Reserved)
//				3rd bit - 0x03 (RF Module Power Cycled)
//				4th bit - 0x04 (Tamper Alarm – wrong code entry limit)
//				5th bit - 0x05 (Tamper Alarm - front escutcheon removed from main)
//				6th bit - 0x06 (Forced Door Open under Door Locked Condition)
// @DH	keypadOperationEventMask		Event mask used to turn on and off the transmission of keypad operation events. This mask DOES NOT apply to the storing of events in the report table.
//              0th bit - 0x00 (Event Source) 0x00 (Operation Event Code) Unknown or manufacturer-specific keypad operation event
//              1st bit - 0x00 (Event Source) 0x01 (Operation Event Code) Lock, source: keypad
//              2nd bit - 0x00 (Event Source) 0x02 (Operation Event Code) Unlock, source: keypad
//              3rd bit - 0x00 (Event Source) 0x03 (Operation Event Code) Lock, source: keypad, error: invalid PIN
//              4th bit - 0x00 (Event Source) 0x04 (Operation Event Code) Lock, source: keypad, error: invalid schedule
//              5th bit - 0x00 (Event Source) 0x05 (Operation Event Code) Unlock, source: keypad, error: invalid code
//              6th bit - 0x00 (Event Source) 0x06 (Operation Event Code) Unlock, source: keypad, error: invalid schedule
//              7th bit - 0x00 (Event Source) 0x0F (Operation Event Code) Non-Access User operation event, source keypad.
// @DH	rfOperationEventMask			Event mask used to turn on and off the transmission of RF operation events. This mask DOES NOT apply to the storing of events in the report table.
//              0th bit - 0x01 (Event Source) 0x00 (Operation Event Code)  Unknown or manufacturer-specific RF operation event
//              1st bit - 0x01 (Event Source) 0x01 (Operation Event Code)  Lock, source: RF
//              2nd bit - 0x01 (Event Source) 0x02 (Operation Event Code)  Unlock, source: RF
//              3rd bit - 0x01 (Event Source) 0x03 (Operation Event Code)  Lock, source: RF, error: invalid code
//              4th bit - 0x01 (Event Source) 0x04 (Operation Event Code)  Lock, source: RF, error: invalid schedule
//              5th bit - 0x01 (Event Source) 0x05 (Operation Event Code)  Unlock, source: RF, error: invalid code
//              6th bit - 0x01 (Event Source) 0x06 (Operation Event Code)  Unlock, source: RF, error: invalid schedule
// @DH	manualOperationEventMask		Event mask used to turn on and off manual operation events. This mask DOES NOT apply to the storing of events in the report table.
//              0th bit - 0x02 (Event Source) 0x00 (Operation Event Code)  Unknown or manufacturer-specific manual operation event
//              1st bit - 0x02 (Event Source) 0x01 (Operation Event Code)  Thumbturn Lock
//              2nd bit - 0x02 (Event Source) 0x02 (Operation Event Code)  Thumbturn Unlock
//              3rd bit - 0x02 (Event Source) 0x07 (Operation Event Code)  One touch lock
//              4th bit - 0x02 (Event Source) 0x08 (Operation Event Code)  Key Lock
//              5th bit - 0x02 (Event Source) 0x09 (Operation Event Code)  Key Unlock
//              6th bit - 0x02 (Event Source) 0x0A (Operation Event Code)  Auto lock
//              7th bit - 0x02 (Event Source) 0x0B (Operation Event Code)  Schedule Lock
//              8th bit - 0x02 (Event Source) 0x0C (Operation Event Code)  Schedule Unlock
//              9th bit - 0x02 (Event Source) 0x0D (Operation Event Code)  Manual Lock (Key or Thumbturn)
//              1st bit - 0x02 (Event Source) 0x0E (Operation Event Code) ) Manual Unlock (Key or Thumbturn)
// @DH	rfidOperandonEventMask			Event mask used to turn on and off RFID operation events. This mask DOES NOT apply to the storing of events in the report table
//              0rd bit - 0x03 (Event Source) 0x00 (Operation Event Code)  Unknown or manufacturer-specific keypad operation even
//              1th bit - 0x03 (Event Source) 0x01 (Operation Event Code)  Lock, source: RFID
//              2th bit - 0x03 (Event Source) 0x02 (Operation Event Code)  Unlock, source: RFID
//              3th bit - 0x03 (Event Source) 0x03 (Operation Event Code)  Lock, source: RFID, error: invalid RFID ID
//              4th bit - 0x03 (Event Source) 0x04 (Operation Event Code)  Lock, source: RFID, error: invalid schedule
//              5th bit - 0x03 (Event Source) 0x05 (Operation Event Code)  Unlock, source: RFID, error: invalid RFID ID
//              6th bit - 0x03 (Event Source) 0x06 (Operation Event Code)  Unlock, source: RFID, error: invalid schedule
// @DH	keypadProgrammingEventMask		Event mask used to turn on and off keypad programming events. This mask DOES NOT apply to the storing of events in the report table.
//              0th bit - 0x00 (Event Source) 0x00 (Operation Event Code)  Unknown or manufacturer-specific keypad programming event
//              1th bit - 0x00 (Event Source) 0x01 (Operation Event Code)  Master code changed, source: keypad User ID: master user ID. PIN: default or master code if codes can be sent over the air per attribute. User type: default User Status: default
//              2th bit - 0x00 (Event Source) 0x02 (Operation Event Code)  PIN added, source: keypad User ID: user ID that was added. PIN: code that was added (if codes can be sent over the air per attribute.) User type: default or type added. User Status: default or status added.
//              3th bit - 0x00 (Event Source) 0x03 (Operation Event Code)  PIN deleted, source: keypad User ID: user ID that was deleted. PIN: code that was deleted (if codes can be sent over the air per attribute.) User type: default or type deleted. User Status: default or status deleted.
//              4th bit - 0x00 (Event Source) 0x04 (Operation Event Code)  PIN changed Source: keypad User ID: user ID that was changed PIN: code that was changed (if codes can be sent over the air per attribute.) User type: default or type changed. User Status: default or status changed.
// @DH	rfProgrammingEventMask			Event mask used to turn on and off RF programming events. This mask DOES NOT apply to the storing of events in the report table.
//              0th bit - 0x01 (Event Source) 0x00 (Operation Event Code)  Unknown or manufacturer-specific RF programming event.
//              2th bit - 0x01 (Event Source) 0x02 (Operation Event Code)  PIN added, source RF Same as keypad source above
//              3th bit - 0x01 (Event Source) 0x03 (Operation Event Code)  PIN deleted, source RF Same as keypad source above.
//              4th bit - 0x01 (Event Source) 0x04 (Operation Event Code)  PIN changed Source RF Same as keypad source above
//              5th bit - 0x01 (Event Source) 0x05 (Operation Event Code)  RFID code added, Source RF
//              6th bit - 0x01 (Event Source) 0x06 (Operation Event Code)  RFID code deleted, Source RF
// @DH	rfidProgrammingEventMask		Event mask used to turn on and off RFID programming events. This mask DOES NOT apply to the storing of events in the report table.
//              0th bit - 0x03 (Event Source) 0x00 (Operation Event Code)  Unknown or manufacturer-specific keypad programming event
//              5th bit - 0x03 (Event Source) 0x05 (Operation Event Code)  ID Added, Source: RFID User ID: user ID that was added. ID: ID that was added (if codes can be sent over the air per at- tribute.) User Type: default or type added. User Status: default or status added.
//              6th bit - 0x03 (Event Source) 0x06 (Operation Event Code)  ID Deleted, Source: RFID User ID: user ID that was deleted. ID: ID that was deleted (if codes can be sent over the air per attribute.) User Type: default or type deleted. User Status: default or status deleted.

// Lock Door (0x00/0)
//
// @param: zmatter 
// ZMatter object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: size
// Pin Code Size
//
// @param: code
// Pin Code
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_door_lock_lock_door(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZWBYTE size, const ZWBYTE *code, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Unlock Door (0x01/1)
//
// @param: zmatter 
// ZMatter object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: size
// Pin Code Size
//
// @param: code
// Pin Code
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_door_lock_unlock_door(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZWBYTE size, const ZWBYTE *code, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Toggle (0x02/2)
//
// @param: zmatter 
// ZMatter object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: size
// Pin Code Size
//
// @param: code
// Pin Code
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_door_lock_toggle(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZWBYTE size, const ZWBYTE *code, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Unlock with Timeout (0x03/3)
//
// @param: zmatter 
// ZMatter object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: timeout
// Relock Timeout
//
// @param: size
// Pin Code Size
//
// @param: code
// Pin Code
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_door_lock_unlock_with_timeout(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZWWORD timeout, ZWBYTE size, const ZWBYTE *code, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Get Log Record (0x04/4)
//
// @param: zmatter 
// ZMatter object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: log_index
// Log Entry Index
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_door_lock_get_log_record(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZWWORD log_index, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Set PIN Code (0x05/5)
//
// @param: zmatter 
// ZMatter object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: user_id
// User Identifier
//
// @param: user_status
// User Status
//
// @param: user_type
// User Type
//
// @param: size
// Pin Code Size
//
// @param: pin
// Pin Code
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_door_lock_set_pin_code(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZWWORD user_id, ZWBYTE user_status, ZWBYTE user_type, ZWBYTE size, const ZWBYTE *pin, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Get PIN Code (0x06/6)
//
// @param: zmatter 
// ZMatter object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: user_id
// User Identifier
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_door_lock_get_pin_code(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZWWORD user_id, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Clear PIN Code (0x07/7)
//
// @param: zmatter 
// ZMatter object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: user_id
// User Identifier
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_door_lock_clear_pin_code(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZWWORD user_id, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Clear All PIN Codes (0x08/8)
//
// @param: zmatter 
// ZMatter object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_door_lock_clear_all_pin_codes(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Set User Status (0x09/9)
//
// @param: zmatter 
// ZMatter object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: user_id
// User Identifier
//
// @param: user_type
// User Type
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_door_lock_set_user_status(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZWWORD user_id, ZWBYTE user_type, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Get User Status (0x0A/10)
//
// @param: zmatter 
// ZMatter object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: user_id
// User Identifier
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_door_lock_get_user_status(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZWWORD user_id, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Set Weekday Schedule (0x0B/11)
//
// @param: zmatter 
// ZMatter object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: id
// Schedule Identifier
//
// @param: user_id
// User Type
//
// @param: days
// Days Mask
//
// @param: start_hour
// Start Hour
//
// @param: start_minute
// Start Minute
//
// @param: end_hour
// End Hour
//
// @param: end_minute
// End Minute
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_door_lock_set_weekday_schedule(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZWBYTE id, ZWWORD user_id, ZWBYTE days, ZWBYTE start_hour, ZWBYTE start_minute, ZWBYTE end_hour, ZWBYTE end_minute, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Get Weekday Schedule (0x0C/12)
//
// @param: zmatter 
// ZMatter object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: id
// Schedule Identifier
//
// @param: user_id
// User Identifier
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_door_lock_get_weekday_schedule(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZWBYTE id, ZWWORD user_id, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Clear Weekday Schedule (0x0D/13)
//
// @param: zmatter 
// ZMatter object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: id
// Schedule Identifier
//
// @param: user_id
// User Identifier
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_door_lock_clear_weekday_schedule(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZWBYTE id, ZWWORD user_id, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Set Year Day Schedule (0x0E/14)
//
// @param: zmatter 
// ZMatter object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: id
// Schedule Identifier
//
// @param: user_id
// User Type
//
// @param: start_time
// Start Time
//
// @param: end_time
// Start Minute
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_door_lock_set_year_day_schedule(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZWBYTE id, ZWWORD user_id, unsigned int start_time, unsigned int end_time, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Get Year Day Schedule (0x0F/15)
//
// @param: zmatter 
// ZMatter object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: id
// Schedule Identifier
//
// @param: user_id
// User Identifier
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_door_lock_get_year_day_schedule(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZWBYTE id, ZWWORD user_id, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Clear Year Day Schedule (0x10/16)
//
// @param: zmatter 
// ZMatter object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: id
// Schedule Identifier
//
// @param: user_id
// User Identifier
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_door_lock_clear_year_day_schedule(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZWBYTE id, ZWWORD user_id, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Set Holiday Schedule (0x11/17)
//
// @param: zmatter 
// ZMatter object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: id
// Schedule Identifier
//
// @param: start_time
// Start Time
//
// @param: end_time
// Start Minute
//
// @param: mode
// Operating Mode During Holiday
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_door_lock_set_holiday_schedule(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZWBYTE id, unsigned int start_time, unsigned int end_time, ZWBYTE mode, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Get Holiday Schedule (0x12/18)
//
// @param: zmatter 
// ZMatter object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: id
// Schedule Identifier
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_door_lock_get_holiday_schedule(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZWBYTE id, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Clear Holiday Schedule (0x13/19)
//
// @param: zmatter 
// ZMatter object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: id
// Schedule Identifier
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_door_lock_clear_holiday_schedule(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZWBYTE id, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Set User Type (0x14/20)
//
// @param: zmatter 
// ZMatter object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: user_id
// User Identifier
//
// @param: user_type
// User Type
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_door_lock_set_user_type(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZWWORD user_id, ZWBYTE user_type, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Get User Status (0x15/21)
//
// @param: zmatter 
// ZMatter object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: user_id
// User Identifier
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_door_lock_get_user_type(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZWWORD user_id, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Set RFID Code (0x16/22)
//
// @param: zmatter 
// ZMatter object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: user_id
// User Identifier
//
// @param: user_status
// User Status
//
// @param: user_type
// User Type
//
// @param: size
// RFID Code Size
//
// @param: code
// RFID Code
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_door_lock_set_rfid_code(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZWWORD user_id, ZWBYTE user_status, ZWBYTE user_type, ZWBYTE size, const ZWBYTE *code, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Get RFID Code (0x17/23)
//
// @param: zmatter 
// ZMatter object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: user_id
// User Identifier
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_door_lock_get_rfid_code(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZWWORD user_id, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Clear RFID Code (0x18/24)
//
// @param: zmatter 
// ZMatter object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: user_id
// User Identifier
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_door_lock_clear_rfid_code(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZWWORD user_id, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Clear All RFID Code (0x19/25)
//
// @param: zmatter 
// ZMatter object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_door_lock_clear_all_rfid_codes(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Color Control Get State
//
// @param: zmatter 
// ZMatter object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_door_lock_get_state(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Color Control Configuration Get
//
// @param: zmatter 
// ZMatter object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_door_lock_configuration_get(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

//-- Cluster Class OccupancySensing (0x0406/1030) //
// Profile Zigbee Device Profile (0x0104/260) //
// The server cluster provides an interface to occupancy sensing functionality, including configuration and provision of notifications of occupancy status.
//
// @DH	occupancy								Occupied State
//				0 - Unoccupied
//				1 - Occupied
// @DH	occupancySensorType						Specifies the type of the occupancy sensor
// @DH	pirOccupiedToUnoccupiedDelay			Specifies the time delay, in seconds, before the PIR sensor changes to its unoccupied state after the last detection of movement in the sensed area.
// @DH	pirUnoccupiedToOccupiedDelay			Specifies the time delay, in seconds, before the PIR sensor changes to its occupied state after the detection of movement in the sensed area
// @DH	pirOccupiedToUnoccupiedThreshold		Specifies the number of movement detection events that must occur in the period PIRUnoccupiedToOccupiedDelay, before the PIR sensor changes to its occupied state
// @DH	ultrasonicOccupiedToUnoccupiedDelay		Specifies the time delay, in seconds, before the Ultrasonic sensor changes to its unoccupied state after the last detection of movement in the sensed area.
// @DH	ultrasonicUnoccupiedToOccupiedDelay		Specifies the time delay, in seconds, before the Ultrasonic sensor changes to its occupied state after the detection of movement in the sensed area.
// @DH	ultrasonicOccupiedToUnoccupiedThreshold	Specifies the number of movement detection events that must occur in the period UltrasonicUnoccupiedToOccupiedDelay, before the Ultrasonic sensor changes to its occupied state

// Set Occupancy Reporting
//
// @param: zmatter
// ZMatter object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: direction
// TRUE  - receiver will be configured to receiver reports
// FALSE - receiver will be configured to send reports
//
// @param: min_interval
// Minimal Reporting Interval
//
// @param: max_interval
// Maximum Reporting Interval
//
// @param: timeout
// Timeout
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWEXPORT ZWError zmatter_cc_occupancy_sensing_set_occupancy_reporting(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZWBOOL direction, ZWWORD min_interval, ZWWORD max_interval, ZWWORD timeout, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Reset Occupancy Reporting
//
// @param: zmatter
// ZMatter object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: direction
// TRUE  - receiver will be configured to receiver reports
// FALSE - receiver will be configured to send reports
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function succes or failure
//
//--ZWError zmatter_cc_occupancy_sensing_reset_occupancy_reporting(ZMatter zmatter, ZMNWK node_id, ZWBYTE endpoint_id, ZWBOOL direction, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

#endif
