//
//  ZMatterLib.h
//  Part of Z-Matter.C library
//
//  Created by Kirill Rogozhkin on 21/9/22.
//  Based on Z-Way source code written by Alex Skalozub and Serguei Poltorak
//
//  Copyright (c) 2022 Z-Wave.Me
//  All rights reserved
//  info@z-wave.me
//
//  This source file is subject to the terms and conditions of the
//  Z-Wave.Me Software License Agreement which restricts the manner
//  in which it may be used.
//

#ifndef zmatter_h
#define zmatter_h

#include "ZErrors.h"
#include "ZLog.h"
#include "ZDefsPublic.h"
#include "ZDataExt.h"

// Allocate and initialize a ZMatter object
//
// @param pzmatter
// pointer that gets a ZMatter object
// Note! ZMatter structure must be filled with zeros first!
// 
// @param bleDev
// for ZMEBLELib: sr:path where path is a path to HCI device (e.g. sr:/dev/ttyUSB1)
// for bluez: string with the index of the HCI device
//
// @param speed:
// UART speed in kbps
// 115200 by default
// 
// @param bleExtDH
// if external BLE commissioner use (bleExt.mode = TRUE), use bleExt.tx/rx data holder for communication with the external app (TRUE) or WebSocket server at port bleExt.port (FALSE)
// The value of bleExt.ws data holder equals to !bleExtDH
// 
// @param config_folder
// path to config files
// NULL for current folder
// 
// @param paa_folder:
// path to paa folder
// NULL for current folder
// 
// @param cd_folder:
// path to CD folder
// NULL for current folder
//
// @param name
// Instance name to distinguish messages in log
//
// @param logger
// Logger to write messages to
//
ZWEXPORT ZWError zmatter_init(ZMatter *pzmatter, ZWCSTR bleDev, speed_t speed, ZWBOOL bleExtDH, ZWCSTR config_folder, ZWCSTR paa_folder, ZWCSTR cd_folder, ZWCSTR name, ZWLog logger);

// Save state, close all handles and deallocate a ZMatter object
//
// @param pzmatter:
// pointer to ZMatter object
// set to NULL on return
//
ZWEXPORT void zmatter_terminate(ZMatter *pzmatter);

// Start worker thread and open port
//
// @param zmatter:
// ZMatter object instance
//
// @param termination_callback:
// function called on termination of Z-Matter worker thread (underlying device has gone)
// Syntax is (void) termination_callback(ZMatter zmatter)
// NULL if callback is not needed
// Note, that the callback is called in Z-Matter worker thread.
// Jobs are not processed anymore, but Data Holders are still available.
//
// @param arg:
// optional arg passed to termination callback
//
ZWEXPORT ZWError zmatter_start(ZMatter zmatter, ZMTerminationCallback termination_callback, void* arg);

// Stop processing of commands and terminate worker thread
//
// @param zmatter:
// ZMatter object instance
//
ZWEXPORT ZWError zmatter_stop(ZMatter zmatter);

// Returns logger instance used by zmatter object
//
// @param zmatter:
// ZMatter object instance
//
ZWEXPORT ZWLog zmatter_get_logger(ZMatter zmatter);

// Returns instance name for zmatter object
//
// @param zmatter:
// ZMatter object instance
//
ZWEXPORT ZWCSTR zmatter_get_name(ZMatter zmatter);

// Discover PAN ID, get network topology, version, capabilities etc.
// Function exits only then discovery finished (successfully or unsuccessfully).
//
// @param zmatter:
// ZMatter object instance
//
ZWEXPORT ZWError zmatter_discover(ZMatter zmatter);

// Check if queue is empty (or has only jobs in state 'Done')
//
// @param zmatter:
// ZMatter object instance
//
ZWEXPORT ZWBOOL zmatter_is_idle(ZMatter zmatter);

// Check that Z-Matter is still running (Z-Matter working thread still works).
// termination_callback parameter of zmatter_start() can be used as alternative to this function.
//
// @param zmatter:
// ZMatter object instance
//
ZWEXPORT ZWBOOL zmatter_is_running(ZMatter zmatter);

// Returns the time of last change in the list of devices
//
// @param zmatter:
// ZMatter object instance
//
ZWEXPORT time_t zmatter_get_devices_update_time(const ZMatter zmatter);

// Returns the time of last change in the list instances of a device
//
// @param zmatter:
// ZMatter object instance
//
// @param node_id:
// NodeId
//
ZWEXPORT time_t zmatter_get_endpoints_update_time(const ZMatter zmatter, ZMNWK node_id);

// Returns the time of last change in the list clusters of an endpoint of a device
//
// @param zmatter:
// ZMatter object instance
//
// @param node_id:
// NodeId
//
// @param endpoint_id:
// Endpoint Id
//
ZWEXPORT time_t zmatter_get_clusters_update_time(const ZMatter zmatter, ZMNWK node_id, ZMEndpointId endpoint_id);

// Send NoOperation to a node and wake up it's queue.
// Used to explicitelly check if mains powered is reachable or the battery device is currently awake and mark it as failed if not reachable.
//
// @param zmatter:
// ZMatter object instance
//
// @param node_id:
// NodeId
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
// Even on non-delivered packet successCallback is called after IsFailed call
// To check device reachability look on isFailed flag
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function success or failure
//
ZWEXPORT ZWError zmatter_device_send_nop(ZMatter zmatter, ZMNWK node_id, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void* callbackArg);

// Force queue wakeup for a device
// Try to deliver jobs in the queue for the specified device.
// Note that jobs in the queue might be marked again as W (wait for wakeup) after first unsuccessful delivery.
//
// @param zmatter:
// ZMatter object instance
//
// @param node_id:
// NodeId
//
ZWEXPORT void zmatter_device_awake_queue(const ZMatter zmatter, ZMNWK node_id);

// Attach callback function from Device change event.
// Duplicate address add of function callback is suppressed (one callback can be binded only once).
//
// @param zmatter:
// Z-Matter object instance
//
// @param mask:
// Mask of which events to watch
//
// @param callback:
// Function to be attached to change event
//
// @param arg:
// Custom argument for callback function
//
ZWEXPORT ZWError zmatter_device_add_callback(const ZMatter zmatter, ZMDeviceChangeType mask, ZMDeviceCallback callback, void *arg);

// Detach callback function from Device change event
// Delete all callbacks with same address of function callback
//
// @param zmatter:
// Z-Matter object instance
//
// @param callback:
// Function to be detached from change event
//
ZWEXPORT ZWError zmatter_device_remove_callback(const ZMatter zmatter, ZMDeviceCallback callback);

// Attach callback function from Device change event
// Allows many callbacks with same address of callback function (but supress duplicate callback/arg pair)
//
// @param zmatter:
// Z-Matter object instance
//
// @param mask:
// Mask of which events to watch
//
// @param callback:
// Function to be attached to change event
//
// @param arg:
// Custom argument for callback function
//
ZWEXPORT ZWError zmatter_device_add_callback_ex(const ZMatter zmatter, ZMDeviceChangeType mask, ZMDeviceCallback callback, void *arg);

// Detach callback function from Device change event
// Remove only once callback referred as function address and argument pair
//
// @param zmatter:
// Z-Matter object instance
//
// @param callback:
// Function to be detached from change event
//
ZWEXPORT ZWError zmatter_device_remove_callback_ex(const ZMatter zmatter, ZMDeviceCallback callback, void *arg);

// Run Cluster Interview.
//
// @param zmatter:
// Z-Matter object instance
//
// @param device_id:
// Node Id
//
// @param endpoint_id:
// Endpoint Id
//
// @param cc_id:
// Cluster Id
//
ZWEXPORT ZWError zmatter_cluster_interview(const ZMatter zmatter, ZMNWK device_id, ZMEndpointId endpoint_id, ZMClusterId cc_id);

// Force re-interview of the device.
// Deletes Clusters and Endpoint structures,
// renders them from scratch based on the device interview.
//
// @param zmatter:
// Z-Matter object instance
//
// @param device_id:
// Node Id
//
ZWEXPORT ZWError zmatter_device_interview_force(const ZMatter zmatter, ZMNWK device_id);

// Check if device interview is done (all Clusters interview are done).
//
// @param zmatter:
// Z-Matter object instance
//
// @param device_id:
// Node Id
//
ZWEXPORT ZWBOOL zmatter_device_is_interview_done(const ZMatter zmatter, ZMNWK device_id);

// Pair a new device
//
// @param: zmatter 
// ZMatter object instance
//
ZWEXPORT ZWError zmatter_controller_add_node_to_network(const ZMatter zmatter);

// Cancel ongoing pairing process
//
// @param: zmatter 
// ZMatter object instance
//
ZWEXPORT ZWError zmatter_controller_add_node_to_network_cancel(const ZMatter zmatter);

// Start/stop exclusion of a node
//
// @param: zmatter
// Z-Matter object instance
//
// @param: node_id
// Node ID to remove
//
ZWEXPORT ZWError zmatter_controller_remove_node_from_network(ZMatter zmatter, ZMNWK node_id);

// Reset the controller
// Note: this function will delete ALL data from the Matter factory!
//
// @param: zmatter
// Z-Matter object instance
//
ZWEXPORT ZWError zmatter_controller_set_default(ZMatter zmatter);

// Saves controller configuration, defaults and other needed files as tgz archive
//
// @param: zmatter
// Z-Matter object instance
//
// @param: data
// Pointer to a variable that will receive an allocated data buffer
// It should be freed by caller when it is no longer needed
//
// @param: length
// Pointer to a variable that will receive data length
//
ZWEXPORT ZWError zmatter_controller_config_save(ZMatter zmatter, ZWBYTE **data, size_t *length);

// Restores controller configuration, defaults and other needed files from tgz archive
// Note: This function restores network Factory ID, nodeId and information about nodes in network.
// 
// It has a lot of side effects! After this function consider:
//
// @param: zmatter
// Z-Matter object instance
//
// @param: data
// Data buffer with archive contents
//
// @param: length
// Data length
//
// @param: full
// Perform full restore (including home id and network topology)
//
ZWEXPORT ZWError zmatter_controller_config_restore(ZMatter zmatter, const ZWBYTE *data, size_t length, ZWBOOL full);

// Save all Z-Matter data to the disc
// This function runs at Z-Matter library termination
//
// Run this function from time to time to have current data for persistance.
//
// The file is named config/zddx/<factoryId>-MatterData.xml (see config_folder parameter in zmatter_init) and is read only on Z-Matter library startup.
//
// @param zmatter:
// Z-Matter object instance
//
ZWEXPORT ZWError zmddx_save_to_xml(const ZMatter zmatter);

// Returns 0 terminated list of registered devices Node Id.
// Note that device 0 is not included in the list (since it is zero terminated), but is present via explicit addressing (if exists)
// NULL is returned if incorrect Z-Matter object is given.
//
// @param zmatter:
// Z-Matter object instance
//
ZWEXPORT ZMDevicesList zmatter_devices_list(const ZMatter zmatter);

// Free list of registered devices Node Id previously allocated by zmatter_devices_list.
//
// @param list:
// List to be freed
//
ZWEXPORT void zmatter_devices_list_free(ZMDevicesList list);

// Returns 0 terminated list of registered endpoints Id for specified device.
// Note, that endpoint 0 is not included in the list (since it is zero terminated), but is ALWAYS present via excplicit addressing..
// NULL is returned if incorrect Z-Matter object is given or device does not exist.
//
// @param zmatter:
// Z-Matter object instance
//
// @param deviceId:
// Device in question
//
ZWEXPORT ZMEndPointsList zmatter_endpoints_list(const ZMatter zmatter, ZMNWK deviceId);

// Free list of registered endpoints Id for specified device previously allocated by zmatter_endpoints_list.
//
// @param list:
// List to be freed
//
ZWEXPORT void zmatter_endpoints_list_free(ZMEndPointsList list);

// Returns 0 terminated list of registered Clusters Id for specified endpoint of device
// Note that cluster 0 is not included in the list (since it is zero terminated), but is present via excplicit addressing (if exists)
// NULL is returned if incorrect Z-Matter object is given or device does not exist or endpoint does not exist.
//
// @param zmatter:
// Z-Matter object instance
//
// @param device_id:
// Device in question
//
// @param endpoint_id:
// Endpoint in question
//
ZWEXPORT ZMClustersList zmatter_clusters_list(const ZMatter zmatter, ZMNWK device_id, ZMEndpointId endpoint_id, ZWBOOL inOut);

// Free list of registered Clusters Id for specified endpoint of device previously allocated by zmatter_clusters_list.
//
// @param list:
// List to be freed
//
ZWEXPORT void zmatter_clusters_list_free(ZMClustersList list);

// Returns if cluster both exists for specified endpoint of device, and is rendered as supported
//
// @param zmatter:
// Z-Matter object instance
//
// @param node_id:
// Device in question
//
// @param node_id:
// Endpoint in question
//
// @param cluster_id:
// Cluster in question
//
ZWEXPORT ZWBOOL zmatter_cluster_is_supported(const ZMatter zmatter, ZMNWK node_id, ZMEndpointId endpoint_id, ZMClusterId cluster_id);

// Returns the cluster name by id or NULL if not supported by the library
// 
// @param cl_id:
// Cluster in question
//
ZWEXPORT ZWCSTR zmatter_get_cluster_name(ZMClusterId cl_id);

// Prints into provided stream debug information about queue content.
// Queue includes pending jobs, processing jobs as well as done jobs for some period to help analyse outgoing queue.
// The output contains one job per line with few fields describing the job:
//  Send count
//  Status
//  Wait for Serial API ACK
//  Wait for Serial API Response
//  Wait for Serial API Callback
//  Wait for Device Reply
//  Timeout in ms
//  Destination node ID (or 0 if not to node)
//  Frame type
//  Serial API Function ID
//  Payload
//  Serial API Callback ID
//  Job description
// The format is %hhi %c %c %c %c %c %3.2f %3hhi ( %02hhX %02hhX %s %02hhX) %s\n
// Status:
//  D - done
//  W - waiting for wakeup of the device
//  S - waiting for security session
//  (space) - job to be processed
// Wait for Serial API ... and Wait for Reply:
//  (space) - no need to wait for message of that type
//  - - wait for message
//  + - message received
//
// @param zmatter:
// Z-Matter object instance
//
// @param stream:
// Stream with text representation of the queue
//
ZWEXPORT void zmatter_queue_inspect(const ZMatter zmatter, FILE * stream);

// Send a Cluster Command
//
// @param: zmatter 
// ZMatter object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param cluster_id
// Destination Cluster Id
//
// @param command_id
// Command Id
//
// @param length
// Payload length
//
// @param payload
// Payload
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zmatter_send_command(const ZMatter zmatter, ZMNWK node_id, ZMEndpointId endpoint_id, ZMClusterId cluster_id, ZMCommandId command_id, size_t length, const uint8_t * payload, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void *arg);

// Reads Cluster Attribute value
//
// @param: zmatter 
// ZMatter object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param cluster_id
// Destination Cluster Id
//
// @param attribute_id
// Attribute Id
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zmatter_send_attribute_read(const ZMatter zmatter, ZMNWK node_id, ZMEndpointId endpoint_id, ZMClusterId cluster_id, ZMAttributeId attribute_id, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void *arg);


// Writes Cluster Attribute value
//
// @param: zmatter 
// ZMatter object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param cluster_id
// Destination Cluster Id
//
// @param attribute_id
// Attribute Id
//
// @param length
// Value payload length
//
// @param payload
// Value payload
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zmatter_send_attribute_write(const ZMatter zmatter, ZMNWK nodeId, ZMEndpointId endpointId, ZMClusterId clusterId, ZMAttributeId attributeId, size_t length, const uint8_t * payload, ZMJobCustomCallback successCallback, ZMJobCustomCallback failureCallback, void *arg);

// Set up WiFi credentials for pairing of Matter over WiFi devices
//
// @param: zmatter 
// ZMatter object instance
//
// @param: ssid
// WiFi ESSID
//
// @param credentials
// WiFi password
//
ZWEXPORT ZWError zmatter_set_wifi_config(const ZMatter zmatter, ZWCSTR ssid, ZWCSTR credentials);

// Set up Thread credentials for pairing of Matter over Thread devices
//
// @param: zmatter 
// ZMatter object instance
//
// @param credentials
// Thread credentials
//
ZWEXPORT ZWError zmatter_set_thread_config(const ZMatter UNUSED(zmatter), ZWCSTR credentials);

// Set Matter device credential for pairing
//
// @param: zmatter 
// ZMatter object instance
//
// @param setupCode
// QR code content (starting with MT:) or manual code
//
// @param rendezvousType
// Override rendezvous type
// Used with manual code
// Can be NULL or "" to use from the setup code, "ip" (on-netowrk pairing), "ble" (Bleutooth pairing) or "wifi" (WiFi pairing)
//
ZWEXPORT ZWError zmatter_set_setup_code(const ZMatter zmatter, ZWCSTR setupCode, ZWCSTR rendezvousType);

// Pair a new device WiFi over Bluetooth Low Energy
//
// @param: zmatter 
// ZMatter object instance
//
// @param: discriminator 
// Device discriminator
//
// @param setupPINCode
// Device PIN code
//
// @param: node_id 
// Node Id to be assigned
//
ZWEXPORT ZWError zmatter_pair_ble(const ZMatter zmatter, uint16_t discriminator, uint32_t setupPINCode, ZMNWK node_id);

// Pair a new WiFi or Thread device over IP
// For diveces already commissioned by any other Matter gateway
//
// @param: zmatter 
// ZMatter object instance
//
// @param: discriminator 
// Device discriminator
//
// @param setupPINCode
// Device PIN code
//
// @param: node_id 
// Node Id to be assigned
//
ZWEXPORT ZWError zmatter_pair_ip(const ZMatter zmatter, uint32_t setupPINCode, ZMNWK node_id);

// Encode attribute value in TLV format
//
// @param: value 
// Integer value of attribute. 
//
// @param: type 
// Type of attribute value. One of ATTRIBUTE_VALUE_TYPE_*
//
// @param dataPayload
// Byte array to encode attribute value.
//
// @param: dataPayloadLen 
// Pointer to the encoded attribute value size. The maximal dataPayload array size should be passed. The actual encoded data size is returned (or error)
//
ZWEXPORT ZWError zmatter_encode_tlv_attribute_value(uint32_t value, uint8_t type, uint8_t * dataPayload, size_t * dataPayloadLen);

#endif