//
//  ZPointer.h
//  z-way-js
//
//  Created by Alex Skalozub on 7/16/12.
//
//  Copyright (c) 2012 Z-Wave.Me
//

#ifndef zway_js_pointer_h
#define zway_js_pointer_h

class ZRefCounter
{
private:
    volatile long m_refCount;
    
public:
    ZRefCounter(int val = 0) : m_refCount(val) { }
    virtual ~ZRefCounter() { }
    
    operator long() const throw() { return m_refCount; }
    
    long operator++() throw()
    {
#ifdef _WINDOWS
        return (long)InterlockedIncrement(&m_refCount);
#else
        return (long)__sync_add_and_fetch(&m_refCount, 1);
#endif
    }
    
    long operator--() throw()
    {
#ifdef _WINDOWS
        return (long)InterlockedDecrement(&m_refCount);
#else
        return (long)__sync_sub_and_fetch(&m_refCount, 1);
#endif
    }
    
    long operator++(int) throw()
    {
#ifdef _WINDOWS
        return (long)InterlockedIncrement(&m_refCount) - 1;
#else
        return (long)__sync_add_and_fetch(&m_refCount, 1) - 1;
#endif
    }
    
    long operator--(int) throw()
    {
#ifdef _WINDOWS
        return (long)InterlockedDecrement(&m_refCount) + 1;
#else
        return (long)__sync_sub_and_fetch(&m_refCount, 1) + 1;
#endif
    }
    
};

class ZWJSEXPORT ZRefCountedObject
{
private:
    ZRefCounter m_counter;
    
public:
    void AddRef() throw() { ++m_counter; }
    void ReleaseRef() { if (--m_counter == 0) Release(); }
    
    ZRefCountedObject() { }
    virtual ~ZRefCountedObject() { }
    
protected:
    virtual void Release() { delete this; }
};

template<typename T>
class ZRefCountedPointer
{
private:
    T* m_ptr;
    
public:
    typedef ZRefCountedPointer<T> TSelf;
    
    inline ZRefCountedPointer() throw() : m_ptr(NULL) {}
    inline ZRefCountedPointer(T* p_ptr) throw() : m_ptr(NULL) { copy(p_ptr); }
    inline ZRefCountedPointer(const TSelf & p_source) throw() : m_ptr(NULL) { copy(p_source); }
    
    inline ~ZRefCountedPointer() throw() { if (m_ptr != NULL) m_ptr->ReleaseRef(); }
    
    inline void copy(T* p_ptr) throw()
    {
        T* torel = m_ptr;
        m_ptr = p_ptr;
        
        if (m_ptr != NULL) m_ptr->AddRef();
        if (torel != NULL) torel->ReleaseRef();
    }
    
    inline void copy(const TSelf & p_source) throw()
    {
        copy(p_source.get_ptr());
    }
    
    inline TSelf & operator=(const TSelf & p_source) throw() { copy(p_source); return *this; }
    inline TSelf & operator=(T* p_ptr) throw() { copy(p_ptr); return *this; }
    
    inline void release() throw()
    {
        T * torel = m_ptr;
        m_ptr = NULL;
        
        if (torel != NULL) torel->ReleaseRef();
    }
    
    
    inline T& operator*() const throw() { return *m_ptr; }
    inline T* operator->() const throw() { return m_ptr; }
    inline T* get_ptr() const throw() { return m_ptr; }
    
    inline bool is_valid() const throw() { return m_ptr != NULL; }
    inline bool is_empty() const throw() { return m_ptr == NULL; }
    
    inline bool operator ==(const TSelf & p_item) const throw() { return m_ptr == p_item.get_ptr(); }
    inline bool operator !=(const TSelf & p_item) const throw() { return m_ptr != p_item.get_ptr(); }
    inline bool operator >(const TSelf & p_item) const throw() { return m_ptr > p_item.get_ptr(); }
    inline bool operator <(const TSelf & p_item) const throw() { return m_ptr < p_item.get_ptr(); }

    inline T* detach() throw()
    {
        T* ret = m_ptr;
        m_ptr = NULL;
        return ret;
    }
        
    inline void attach(T * p_ptr) throw()
    {
        release();
        m_ptr = p_ptr;
    }
	
    inline TSelf & operator<<(TSelf & p_source) throw() { attach(p_source.detach()); return *this; }
    inline TSelf & operator>>(TSelf & p_dest) throw() { p_dest.attach(detach()); return *this; }
};


#endif
