//
//  ClustersPublic.h
//  Part of Z-Bee.C library
//
//  Created by Kirill Rogozhkin on 11/11/22.
//  Based on Z-Way source code written by Alex Skalozub and Serguei Poltorak
//
//  Copyright (c) 2022 Z-Wave.Me
//  All rights reserved
//  info@z-wave.me
//
//  This source file is subject to the terms and conditions of the
//  Z-Wave.Me Software License Agreement which restricts the manner
//  in which it may be used.
//

#ifndef zbee_cluster_public_h
#define zbee_cluster_public_h

#include "ZPlatform.h"
#include "ZDefsPublic.h"

// Cluster Class Node Descriptor Request (0x0002/2) //
// Profile Zigbee Device Profile (0x0000/0) //
// The Node_Desc_req command is generated from a local device wishing to inquire
// as to the node descriptor of a remote device. This command shall be unicast either
// to the remote device itself or to an alternative device that contains the discovery
// information of the remote device

// Node Descriptor Request
// @JSName: Run
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// EndPoint Id
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
// @report: Node Descriptor updated and called Active_EP_req
//
ZWEXPORT ZWError zbee_cc_node_descriptor_request(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);


// Cluster Class Active EndPoints Request (0x0005/5) //
// Profile Zigbee Device Profile (0x0000/0) //
// The Active_EP_req command is generated from a local device wishing to acquire
// the list of endpoints on a remote device with simple descriptors. This command
// shall be unicast either to the remote device itself or to an alternative device that
// contains the discovery information of the remote device.

// Active EndPoints Request
// @JSName: Run
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// EndPoint Id
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
// @report: list of EndPoints updated and called Simple_Desc_req for each EndPoint
//
ZWEXPORT ZWError zbee_cc_active_endpoints_request(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);


// Cluster Class Simple Descriptor Request (0x0004/4) //
// Profile Zigbee Device Profile (0x0000/0) //
// The Simple_Desc_req command is generated from a local device wishing to
// inquire as to the simple descriptor of a remote device on a specified endpoint. This
// command shall be unicast either to the remote device itself or to an alternative
// device that contains the discovery information of the remote device

// Simple Descriptor Request
// @JSName: Run
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: new_endpoint_id
// Inquired EndPoint Id
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_simple_descriptor_request(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBYTE new_endpoint_id, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);


// Cluster Class End Device Announce (0x0013/19) //
// Profile Zigbee Device Profile (0x0000/0) //
// The End_Device_annce is provided to enable Zigbee end devices on the network
// to notify other Zigbee devices that the end device has joined or re-joined the
// network, identifying the end devices 64-bit IEEE address and new 16-bit
// NWKaddress and informing the Remote Devices of the capability of the Zigbee
// EndDevice. The destination addressing on this primitive is broadcast to all
// devices

// End Device Announce
// @JSName: Run
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: ieee
// IEEE address for Local Device
//
// @param: capability
// Capability of the local device
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_end_device_announce(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, const ZBEUI64 ieee, ZWBYTE capability, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);


// Cluster Class Bind Request (0x0021/33) //
// Profile Zigbee Device Profile (0x0000/0) //
// The End_Device_annce is provided to enable Zigbee end devices on the network
// to notify other Zigbee devices that the end device has joined or re-joined the
// network, identifying the end devices 64-bit IEEE address and new 16-bit
// NWKaddress and informing the Remote Devices of the capability of the Zigbee
// EndDevice. The destination addressing on this primitive is broadcast to all
// devices

// Bind By Group Address
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: src_address
// The IEEE address for the source.
//
// @param: src_endpoint_id
// The source endpoint for the binding entry.
//
// @param: cluster_id
// The identifier of the cluster on the source device that is bound to the destination.
//
// @param: group_addr
// The destination address for the binding entry.
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_bind_request_by_group_addr(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, const ZBEUI64 src_address, ZWBYTE src_endpoint_id, ZWWORD cluster_id, ZWWORD group_addr, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Bind Device to Device By Id
//
// @param: zbee
// ZBee object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: src_endpoint_id
// The source endpoint for the binding entry.
//
// @param: cluster_id
// The identifier of the cluster on the source device that is bound to the destination.
//
// @param: dst_node_id
// Destination Node Id
//
// @param: dst_endpoint_id
// The destination endpoint for the binding entry.
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_bind_request_device_to_device_by_id(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBYTE src_endpoint_id, ZWWORD cluster_id, ZBNWK dst_node_id, ZWBYTE dst_endpoint_id, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Bind By IEEE
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: src_address
// The IEEE address for the source.
//
// @param: src_endpoint_id
// The source endpoint for the binding entry.
//
// @param: cluster_id
// The identifier of the cluster on the source device that is bound to the destination.
//
// @param: dst_addr
// The destination address for the binding entry.
//
// @param: dst_endpoint_id
// It is the destination endpoint for the binding entry.
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_bind_request_by_ieee(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, const ZBEUI64 src_address, ZWBYTE src_endpoint_id, ZWWORD cluster_id, const ZBEUI64 dst_addr, ZWBYTE dst_endpoint_id, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Bind By Device Id
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: src_endpoint_id
// The source endpoint for the binding entry.
//
// @param: cluster_id
// The identifier of the cluster on the source device that is bound to the destination.
//
//
// @param: dst_endpoint_id
// It is the destination endpoint for the binding entry.
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_bind_request_to_local_device_by_device_id(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBYTE src_endpoint_id, ZWWORD cluster_id, ZWBYTE dst_endpoint_id, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);


// Cluster Class Power Descriptor Request (0x0003/3) //
// Profile Zigbee Device Profile (0x0000/0) //
// The Power_Desc_req command is generated from a local device wishing to inquire 
// as to the power descriptor of a remote device. This command shall be unicast 
// either to the remote device itself or 
// to an alternative device that contains the discovery information of the remote device.

// Power Descriptor Request
// @JSName: Run
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// EndPoint Id
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_power_descriptor_request(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);


// Cluster Class Network Address Request (0x0000/0) //
// Profile Zigbee Device Profile (0x0000/0) //
// The NWK_addr_req is generated from a Local Device wishing to inquire as to the
// 16 bit address of the Remote Device based on its known IEEE address. The
// destination addressing on this command shall be broadcast to all RxOnWhenIdle
// devices.

// Network Address Request
// @JSName: Run
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: ieee
// The IEEE address to be matched by the Remote Device
//
// @param: request_type
// Request Type for this command:
//  0x00        - Single device response
//  0x01        - Extended response
//  0x02...0xFF - Reserved
//
// @param: start_index
// If the Request type for this command is 
// Extended response, the start_index
// provides the starting index for the
// requested elements of the associated
// devices list
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_network_address_request(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, const ZBEUI64 ieee, ZWBYTE request_type, ZWBYTE start_index, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);


// Cluster Class OnOff (0x0006/6) //
// Profile Zigbee Device Profile (0x0104/260) //
// Attributes and commands for switching devices between ‘On’ and ‘Off’ states.
//
// @DH	onOff			Current state
// @DH	globalSceneControl	Global Scene state
// @DH	onTime			Time the “on” state SHALL be maintained before automatically transitioning to the “off” state when using the On with timed off command. 
// @DH	offWaitTime		Time the “off” state SHALL be guarded to prevent an on command turning the device back to its “on” state

// Send On (0x00/0) or Off (0x01/1)
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: value
// New onOff state:
//  TRUE  - send On command
//  FALSE - send Off command
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_on_off_set(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBOOL value, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Send Toggle (0x02/2)
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_on_off_toggle(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Send Off with Effect (0x40/64)
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: effect_id
// Effect Id:
//  0x00        - Delayed All Off
//  0x01        - Dying Light
//  0x02...0xFF - Reserved
//
// @param: effect_var
// Effect Variant:
//  if effect_id 0x00:
//  @default: 0x00
//   0x00        - Fade to off in 0.8 seconds
//   0x01        - No fade
//   0x02        - 50% dim down in 0.8 seconds then fade to off in 12 seconds
//   0x03...0xFF - Reserved
//  if effect_id 0x01:
//  @default: 0x00
//   0x00        - 20% dim up in 0.5s then fade to off in 1 second
//   0x01...0xFF - Reserved
//  if effect_id 0x02...0xFF:
//   0x00...0xFF - Reserved
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_on_off_off_with_effect(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBYTE effect_id, ZWBYTE effect_var, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Send On with Recall Global Scene (0x41/65)
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_on_off_on_with_recall_global_scene(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Send On with Time Off (0x42/66)
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: control
// Contains information on how the device is to be operated.
//  0x00        -  The On With Timed Off command SHALL be processed unconditionally.
//  0x01...0x7F - Reserved
//  0x80        - The On With Timed Off command SHALL only be accepted if the OnOff attribute is equal to 0x01.
//  0x81...0xFF - Reserved
//
// @param: on_time
// The length of time (in 1/10ths second) that the device is to remain “on”.
//  0x0000...0xFFFE - time in 1/10th second
//  0xFFFF          - Reserved
//
// @param: off_wait_time
// The length of time (in 1/10ths second) that the device SHALL remain “off”.
//  0x0000...0xFFFE - time in 1/10th second
//  0xFFFF          - Reserved
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_on_off_on_with_timed_off(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBOOL control, ZWWORD on_time, ZWWORD off_wait_time, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Set Time of On Time 
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: on_time
// The length of time (in 1/10ths second) that the device SHALL remain “off”.
//  0x0000...0xFFFE - time in 1/10th second
//  0xFFFF          - Reserved
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_on_off_set_on_time(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWWORD on_time, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Set Time of On with Time Off
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: off_wait_time
// The length of time (in 1/10ths second) that the device SHALL remain “off”.
//  0x0000...0xFFFE - time in 1/10th second
//  0xFFFF          - Reserved
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_on_off_set_off_wait_time(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWWORD off_wait_time, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// On Off Get
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_on_off_get(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// On Off Configuration Get
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_on_off_configuration_get(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Set On Off Reporting
//
// @param: zbee
// ZBee object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: direction
// TRUE  - receiver will be configured to receiver reports
// FALSE - receiver will be configured to send reports
//
// @param: min_interval
// Minimal Reporting Interval
//
// @param: max_interval
// Maximum Reporting Interval
//
// @param: timeout
// Timeout
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_on_off_set_on_off_reporting(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBOOL direction, ZWWORD min_interval, ZWWORD max_interval, ZWWORD timeout, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Set On Off Reporting
//
// @param: zbee
// ZBee object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: direction
// TRUE  - receiver will be configured to receiver reports
// FALSE - receiver will be configured to send reports
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_on_off_reset_on_off_reporting(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBOOL direction, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);


// Cluster Class PollControl (0x0020/32) //
// Profile Zigbee Device Profile (0x0104/260) //
// This cluster provides a mechanism for the management of an end device’s MAC Data Request rate. For the purposes of this cluster, the term “poll” always refers to the sending of a MAC Data Request from the end device to the end device’s parent.
//
// @DH	checkInInterval			The Check-inInterval represents the default amount of time between check-ins by the poll control server with the poll control client. The Check-inInterval is measured in quarterseconds. A value of 0 indicates that the Poll Control Server is turned off and the poll control server will not check-in with the poll control client.
// @DH	longPollInterval    	The LongPollInterval defines the frequency of polling that an end device does when it is NOT in fast poll mode. The LongPollInterval SHOULD be longer than the ShortPollInterval attribute but shorter than the Check-inInterval attribute.
// @DH	shortPollInterval		An end device that implements the Poll Control server MAY optionally expose the ShortPollInterval attribute. The ShortPollInterval represents the number of quarterseconds that an end device waits between MAC Data Requests to its parent when it is expecting data (i.e., in fast poll mode
// @DH	fastPollTimeout		    The FastPollTimeout attribute represents the number of quarterseconds that an end device will stay in fast poll mode by default. It is suggested that the FastPollTimeout attribute value be greater than 7.68 seconds.
// @DH	checkInIntervalMin		The Poll Control Server MAY optionally provide its own minimum value for the Check-inInterval to protect against the Check-inInterval being set too low and draining the battery on the end device implementing the Poll Control Server
// @DH	shortPollIntervalMin	The Poll Control Server MAY optionally provide its own minimum value for the LongPollInterval to protect against another device setting the value to too short a time resulting in an inadvertent power drain on the device.
// @DH	fastPollTimeoutMax		The Poll Control Server MAY optionally provide its own maximum value for the FastPollTimeout to avoid it being set to too high a value resulting in an inadvertent power drain on the device

// Fast Poll Stop
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_poll_control_fast_poll_stop(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Set Long Poll Interval
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: new_long_poll_interval
// New Long Poll Interval
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_poll_control_set_long_poll_interval(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, int new_long_poll_interval, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Set Short Poll Interval
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: new_short_poll_interval
// New Short Poll Interval
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_poll_control_set_short_poll_interval(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWWORD new_short_poll_interval, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Poll Control Configuration Get
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_poll_control_configuration_get(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);


// Cluster Class Mgmt Permit Joining Request (0x0036/54) //
// Profile Zigbee Device Profile (0x0000/0) //
// The Mgmt_Permit_Joining_req is generated from a Local Device requesting that
// a remote device or devices allow or disallow association. The
// Mgmt_Permit_Joining_req is generated by a management application or
// commissioning tool which directs the request to a remote device(s) where the
// NLME-PERMIT-JOINING.request is executed using the PermitDuration
// parameter supplied by Mgmt_Permit_Joining_req. Additionally, if the remote
// device is the Trust Center and TC_Significance is set to 1, the trust center
// authentication policy will be affected. The addressing may be unicast or
// ‘broadcast to all RxOnWhenIdle devices’

// Mgmt Permit Joining Request
// @JSName: Run
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: permit_duration
// Permit duration in seconds
//
// @param: tc_significance
// Effect on Trust Center:
//  TRUE  - If the remote device is the trust center, the command affects the trust center authentication policy.
//  FALSE - There is no effect on the trust center.
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_mgmt_permit_joining_request(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBYTE permit_duration, ZWBOOL tc_significance, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);


// Cluster Class Mgmt Leave Request (0x0034/52) //
// Profile Zigbee Device Profile (0x0000/0) //
// The Mgmt_Leave_req is generated from a Local Device requesting that a Remote
// Device leave the network or to request that another device leave the network. The
// Mgmt_Leave_req is generated by a management application which directs the
// request to a Remote Device where the NLME-LEAVE.request is to be executed
// using the parameter supplied by Mgmt_Leave_req.

// Mgmt Leave Request
// @JSName: Run
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Deleting Device Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: remove_children
// Remove children flag:
//  TRUE  - If the device being asked to leave the network is also being asked to remove its child devices, if any.
//  FALSE - None children will be removed.
//
// @param: rejoin
// Effect on Trust Center:
//  TRUE  - If the device being asked to leave from the current parent is requested to rejoin the network.
//  FALSE - Otherwise.
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_mgmt_leave_request(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBOOL remove_children, ZWBOOL rejoin, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Force Remove
//
// @param: zbee
// ZBee object instance
//
// @param: node_id
// Deleting Device Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: remove_children
// @default: FALSE
// Remove children flag:
//  TRUE  - If the device being asked to leave the network is also being asked to remove its child devices, if any.
//  FALSE - None children will be removed.
//
// @param: rejoin
// @default: FALSE
// Effect on Trust Center:
//  TRUE  - If the device being asked to leave from the current parent is requested to rejoin the network.
//  FALSE - Otherwise.
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_mgmt_leave_request_force_remove(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBOOL remove_children, ZWBOOL rejoin, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);


// Cluster Class Basic (0x0000/0) //
// Profile ZigBee Device Profile (0x0104/260) //
// This cluster supports an interface to the node or physical device. 
// It provides attributes and commands for determining basic information, 
// setting user information such as location, and resetting to factory defaults.
//
// @DH	zclVersion		The ZCLVersion attribute represents a published set of foundation items, such as global commands and functional descriptions.
// @DH	applicationVersion	The ApplicationVersion attribute is 8 bits in length and specifies the version number of the application software contained in the device. The usage of this attribute is manufacturer dependent
// @DH	stackVersion		The StackVersion attribute is 8 bits in length and specifies the version number of the implementation of the ZigBee stack contained in the device. The usage of this attribute is manufacturer dependent.
// @DH	hwVersion		The HWVersion attribute is 8 bits in length and specifies the version number of the hardware of the device. The usage of this attribute is manufacturer dependent
// @DH	manufacturerName	The ManufacturerName attribute is a maximum of 32 bytes in length and specifies the name of the manufacturer as a ZigBee character string
// @DH	modelIdentifier		The ModelIdentifier attribute is a maximum of 32 bytes in length and specifies the model number (or other identifier) assigned by the manufacturer as a ZigBee character string.
// @DH	dateCode		The DateCode attribute is a ZigBee character string with a maximum length of 16 bytes. The first 8 characters specify the date of manufacturer of the device in international date notation according to ISO 8601, i.e., YYYYMMDD, e.g., 20060814. The final 8 characters MAY include country, factory, line, shift or other related information at the option of the manufacturer. The format of this information is manufacturer dependent
// @DH	powerSource		The PowerSource attribute is 8 bits in length and specifies the source(s) of power available to the device. Bits b0–b6 of this attribute represent the primary power source of the device and bit b7 indicates whether the device has a secondary power source in the form of a battery backup.
// 				0x00 - Unknown
// 				0x01 - Mains (single phase)
// 				0x02 - Mains (3 phase)
// 				0x03 - Battery
// 				0x04 - DC Source
// 				0x05 - Emergency mains constrants
// 				0x06 - Emergency mains and transfer switch
// @DH	locationDescription	The LocationDescription attribute is a maximum of 16 bytes in length and describes the physical location of the device as a ZigBee character string. This location description MAY be added into the device during commissioning
// @DH	physicalEnvironment	The PhysicalEnvironment attribute is 8 bits in length and specifies the type of physical environment in which the device will operate. This attribute SHALL be set to one of the non-reserved values. All values are valid for endpoints supporting all profiles except when noted
// 				0x00 - Unspecified environment
// 				0x01 - Mirror (ZSE Profile)
// 				0x01 - Atrium
// 				0x02 - Bar
// 				0x03 - Courtyard
// 				0x04 - Bathroom
// 				0x05 - Bedroom
// 				0x06 - Billiard Room
// 				0x07 - Utility Room
// 				0x08 - Cellar
// 				0x09 - Storage Closet
// 				0x0a - Theater
// 				0x0b - Office
// 				0x0c - Deck
// 				0x0d - Den
// 				0x0e - Dining Room
// 				0x0f - Electrical Room
// 				0x10 - Elevator
// 				0x11 - Entry
// 				0x12 - Family Room
// 				0x13 - Main Floor
// 				0x14 - Upstairs
// 				0x15 - Downstairs
// 				0x16 - Basement/Lower Level
// 				0x17 - Gallery
// 				0x18 - Game Room
// 				0x19 - Garage
// 				0x1a - Gym
// 				0x1b - Hallway
// 				0x1c - House
// 				0x1d - Kitchen
// 				0x1e - Laundry Room
// 				0x1f - Library
// 				0x20 - Master Bedroom
// 				0x21 - Mud Room (small room for coats and boots)
// 				0x22 - Nursery
// 				0x23 - Pantry
// 				0x24 - Office
// 				0x25 - Outside
// 				0x26 - Pool
// 				0x27 - Porch
// 				0x28 - Sewing Room
// 				0x29 - Sitting Room
// 				0x2a - Stairway
// 				0x2b - Yard
// 				0x2c - Attic
// 				0x2d - Hot Tub
// 				0x2e - Living Room
// 				0x2f - Sauna
// 				0x30 - Shop/Workshop
// 				0x31 - Guest Bedroom
// 				0x32 - Guest Bath
// 				0x33 - Powder Room (1/2 bath)
// 				0x34 - Back Yard
// 				0x35 - Front Yard
// 				0x36 - Patio
// 				0x37 - Driveway
// 				0x38 - Sun Room
// 				0x39 - Living Room
// 				0x3a - Spa
// 				0x3b - Whirlpool
// 				0x3c - Shed
// 				0x3d - Equipment Storage
// 				0x3e - Hobby/Craft Room
// 				0x3f - Fountain
// 				0x40 - Pond
// 				0x41 - Reception Room
// 				0x42 - Breakfast Room
// 				0x43 - Nook
// 				0x44 - Garden
// 				0x45 - Balcony
// 				0x46 - Panic Room
// 				0x47 - Terrace
// 				0x48 - Roof
// 				0x49 - Toilet
// 				0x4a - Toilet Main
// 				0x4b - Outside Toilet
// 				0x4c - Shower room
// 				0x4d - Study
// 				0x4e - Front Garden
// 				0x4f - Back Garden
// 				0x50 - Kettle
// 				0x51 - Television
// 				0x52 - Stove
// 				0x53 - Microwave
// 				0x54 - Toaster
// 				0x55 - Vacuum
// 				0x56 - Appliance
// 				0x57 - Front Door
// 				0x58 - Back Door
// 				0x59 - Fridge Door
// 				0x60 - Medication Cabinet Door
// 				0x61 - Wardrobe Door
// 				0x62 - Front Cupboard Door
// 				0x63 - Other Door
// 				0x64 - Waiting Room
// 				0x65 - Triage Room
// 				0x66 - Doctor’s Office
// 				0x67 - Patient’s Private Room
// 				0x68 - Consultation Room
// 				0x69 - Nurse Station
// 				0x6a - Ward
// 				0x6b - Corridor
// 				0x6c - Operating Theatre
// 				0x6d - Dental Surgery Room
// 				0x6e - Medical Imaging Room
// 				0x6f - Decontamination Room
// 				0xff - Unknown environment
// @DH	deviceEnabled		The DeviceEnabled attribute is a Boolean and specifies whether the device is enabled or disabled. This attribute SHALL be set to one of the non-reserved value 'Disabled' means that the device does not send or respond to application level commands, other than commands to read or write attributes. Values of attributes which depend on the operation of the application MAY be invalid, and any functionality triggered by writing to such attributes MAY be disabled. ZigBee networking functionality remains operational. If implemented, the identify cluster cannot be disabled, i.e., it remains functional regardless of this setting
// 				0x00 - Disable
// 				0x01 - Enable
// @DH	alarmMask		The AlarmMask attribute is 8 bits in length and specifies which of a number of general alarms MAY be generated. These alarms are provided as basic alarms that a device MAY use even if no other clusters with alarms are present on the device
// 				0x00 - General hardware fault
// 				0x01 - General software fault
// @DH	disableLocalConfig	The DisableLocalConfig attribute allows a number of local device configuration functions to be disable. The intention of this attribute is to allow disabling of any local configuration user interface, for example to prevent reset or binding buttons being activated by non-authorized persons in a public building. Bit 0 of the DisableLocalConfig attribute disables any factory reset button (or equivalent) on the device. Bit 1 disables any device configuration button(s) (or equivalent)—for example, a bind button.
// 				0th bit
// 				 0 - Reset (to factory defaults) enabled
// 				 1 - Reset (to factory defaults) disabled
// 				1st bit
// 				 0 - Device configuration enabled
// 				 1 - Device configuration disabled
// @DH	swBuildId		The SWBuildID attribute represents a detailed, manufacturer-specific reference to the version of the software

// Basic Reset
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_basic_reset(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Configuration Get
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_basic_configuration_get(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id,ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);


// Cluster Class Power Configuration (0x0001/1) //
// Profile ZigBee Device Profile (0x0104/260) //
// Attributes for determining detailed information about a device’s power source(s) and for configuring under/over voltage alarms.
//
// @DH	mainsVoltage		It  is 16 bits in length and specifies the actual (measured) RMS voltage (or DC voltage in the case of a DC supply) currently applied to the device, measured in units of 100mV.
// @DH	mainsFrequency		It  is 8 bits in length and represents the frequency, in Hertz, of the mains as determined by the device as follows: MainsFrequency = 0.5 x measured frequency Where 2 Hz <= measured frequency <= 506 Hz, corresponding to a MainsFrequency in the range 1 to 0xfd. The maximum resolution this format allows is 2 Hz. The following special values of MainsFrequency apply. In the case of a DC supply, this attribute SHALL also have the value zero.
//			0x00 indicates a frequency that is too low to be measured.
//			0xfe indicates a frequency that is too high to be measured.
//			0xff indicates that the frequency could not be measured.
// @DH	mainsAlarmMask		The MainsAlarmMask attribute is 8 bits in length and specifies which mains alarms MAY be generated, as listed in Table. A ‘1’ in each bit position enables the alarm
//			0 - Mains Voltage too low
//			1 - Mains Voltage too high
//			2 - Mains power supply lost/unavailable (i.e., device is running on battery)
// @DH	mainsVoltageMinThreshold		The MainsVoltageMinThreshold attribute is 16 bits in length and specifies the lower alarm threshold, measured in units of 100mV, for the MainsVoltage attribute. The value of this attribute SHALL be less than MainsVoltageMaxThreshold. If the value of MainsVoltage drops below the threshold specified by MainsVoltageMinThreshold, the device SHALL start a timer to expire after MainsVoltageDwellTripPoint seconds. If the value of this attribute increases to greater than or equal to MainsVoltageMinThreshold before the timer expires, the device SHALL stop and reset the timer. If the timer expires, an alarm SHALL be generated. The Alarm Code field included in the generated alarm SHALL be 0x00. If this attribute takes the value 0xffff then this alarm SHALL NOT be generated
// @DH	mainsVoltageMaxThreshold		The MainsVoltageMaxThreshold attribute is 16 bits in length and specifies the upper alarm threshold, measured in units of 100mV, for the MainsVoltage attribute. The value of this attribute SHALL be greater than MainsVoltageMinThreshold. If the value of MainsVoltage rises above the threshold specified by MainsVoltageMaxThreshold, the device SHALL start a timer to expire after MainsVoltageDwellTripPoint seconds. If the value of this attribute drops to lower than or equal to MainsVoltageMaxThreshold before the timer expires, the device SHALL stop and reset the timer. If the timer expires, an alarm SHALL be generated. The Alarm Code field included in the generated alarm SHALL be 0x01. If this attribute takes the value 0xffff then this alarm SHALL NOT be generated.
// @DH	mainsVoltageDwellTripPoint		The MainsVoltageDwellTripPoint attribute is 16 bits in length and specifies the length of time, in seconds that the value of MainsVoltage MAY exist beyond either of its thresholds before an alarm is generated. If this attribute takes the value 0xffff then the associated alarms SHALL NOT be generated.
// @DH	batteryVoltage		The BatteryVoltage attribute is 8 bits in length and specifies the current actual (measured) battery voltage, in units of 100mV. The value 0xff indicates an invalid or unknown reading.
// @DH	batteryPercentageRemaining		Specifies the remaining battery life as a half integer percentage of the full battery capacity (e.g., 34.5%, 45%, 68.5%, 90%) with a range between zero and 100%, with 0x00 = 0%, 0x64 = 50%, and 0xC8 = 100%. This is particularly suited for devices with rechargeable batteries. The value 0xff indicates an invalid or unknown reading. This attribute SHALL be configurable for attribute reporting.
// @DH	batteryManufacturer		The BatteryManufacturer attribute is a maximum of 16 bytes in length and specifies the name of the battery manufacturer as a ZigBee character string
// @DH	batterySize		The BatterySize attribute is an enumeration which specifies the type of battery being used by the device. This attribute SHALL be set to one of the non-reserved values listed in Table
//			0x00 No battery
//			0x01 Built in
//			0x02 Other
//			0x03 AA
//			0x04 AAA
//			0x05 C
//			0x06 D
//			0x07 CR2 (IEC: CR17355 / ANSI: 5046LC)
//			0x08 CR123A (IEC: CR17345 / ANSI: 5018LC)
//			0xff Unknown
// @DH	batteryAHrRating		The BatteryAHrRating attribute is 16 bits in length and specifies the Ampere-hour rating of the battery, measured in units of 10mAHr.
// @DH	batteryQuantity		The BatteryQuantity attribute is 8 bits in length and specifies the number of battery cells used to power the device.
// @DH	batteryRatedVoltage		The BatteryRatedVoltage attribute is 8 bits in length and specifies the rated voltage of the battery being used in the device, measured in units of 100mV.
// @DH	batteryAlarmMask		The BatteryAlarmMask attribute specifies which battery alarms must be generated, as listed in Table. A ‘1’ in each bit position enables the alarm. Manufacturers are responsible for determining the capability to sense and levels at which the alarms are generated
//			0th bit (Battery voltage too low to continue operating the device’s radio (i.e., BatteryVoltageMinThreshold value has been reached))
//			1st bit (Battery Alarm 1 (i.e., Battery Voltage Threshold 1 or Battery Percentage Threshold 1 value has been reached))
//			2nd bit (Battery Alarm 2 (i.e., Battery Voltage Threshold 2 or Battery Percentage Threshold 2 value has been reached))
//			3rd bit (Battery Alarm 3 (i.e., Battery Voltage Threshold 3 or Battery Percentage Threshold 3 value has been reached))
// @DH	batteryVoltageMinThreshold		Specifies the low battery voltage alarm threshold, measured in units of 100mV at which the device can no longer operate or transmit via its radio (i.e., last gasp). If the value of BatteryVoltage drops below the threshold specified by BatteryVoltageMinThreshold, an appropriate alarm SHALL be generated and/or the corresponding bit SHALL be updated in the BatteryAlarmState attribute. In order to report to Power Configuration clients, servers that implement BatteryVoltageMinThreshold attribute SHALL implement alarming via the Alarm Cluster, attribute reporting via the BatteryAlarmState attribute, or both. For servers that implement alarming via the Alarm Cluster, the appropriate alarm is specified in the Alarm Code field included in the generated alarm and SHALL be one of the values in Table. The host determines which alarm code to populate based on the BatteryAlarmMask attribute and the BatteryVoltageMinThreshold attribute reached. For example, when the BatteryVoltage attribute reaches the value specified by the BatteryVoltageMinThreshold attribute, an alarm with the Alarm Code Field Enumeration “0x10” SHALL be generated. For servers that implement battery alarm reporting via the BatteryAlarmState attribute, the bit corresponding to the threshold level reached SHALL be set to TRUE. See the BatteryAlarmState attribute details for more information.
//			0x10 BatteryVoltageMinThreshold or BatteryPercentageMinThreshold reached for Battery Source 1
//			0x11 BatteryVoltageThreshold1 or BatteryPercentageThreshold1 reached for Battery Source 1
//			0x12 BatteryVoltageThreshold2 or BatteryPercentageThreshold2 reached for Battery Source 1
//			0x13 BatteryVoltageThreshold3 or BatteryPercentageThreshold3 reached for Battery Source 1
//			0x20 BatteryVoltageMinThreshold or BatteryPercentageMinThreshold reached for Battery Source 2
//			0x21 BatteryVoltageThreshold1 or BatteryPercentageThreshold1 reached for Battery Source 2
//			0x22 BatteryVoltageThreshold2 or BatteryPercentageThreshold2 reached Battery Source 2
//			0x23 BatteryVoltageThreshold3 or BatteryPercentageThreshold3 reached Battery Source 2
//			0x30 BatteryVoltageMinThreshold or BatteryPercentageMinThreshold reached for Battery Source 3
//			0x31 BatteryVoltageThreshold1 or BatteryPercentageThreshold1 reached for Battery Source 3
//			0x32 BatteryVoltageThreshold2 or BatteryPercentageThreshold2 reached Battery Source 3
//			0x33 BatteryVoltageThreshold3 or BatteryPercentageThreshold3 reached Battery Source 3
//			0x3a Mains power supply lost/unavailable (i.e., device is running on battery)14
//			0xff Alarm SHALL NOT be generated
// @DH	batteryVoltageThreshold1		Specify the low voltage alarm thresholds, measured in units of 100mV, for the BatteryVoltage attribute. If the value of BatteryVoltage drops below the threshold specified by a BatteryVoltageThreshold, an appropriate alarm SHALL be generated and/or the corresponding bit SHALL be updated in the BatteryAlarmState attribute. The BatteryVoltageThreshold1-3 attributes SHALL be ordered in ascending order such that the BatteryVoltage level specified to trigger: BatteryVoltageThreshold3 is higher than the level specified to trigger BatteryVoltageThreshold2 BatteryVoltageThreshold2 is higher than the level specified to trigger BatteryVoltageThreshold BatteryVoltageThreshold1 is higher than the level specified to trigger BatteryVoltageMinThreshold The appropriate alarm is specified in the Alarm Code field included in the generated alarm and SHALL be one of the values in Table. The host determines which alarm code to populate based on the BatteryAlarmMask attribute and the BatteryVoltageThreshold1-3 attribute reached. If this attribute takes the value 0xff then this alarm SHALL NOT be generated.
// @DH	batteryVoltageThreshold2		Specify the low voltage alarm thresholds, measured in units of 100mV, for the BatteryVoltage attribute. If the value of BatteryVoltage drops below the threshold specified by a BatteryVoltageThreshold, an appropriate alarm SHALL be generated and/or the corresponding bit SHALL be updated in the BatteryAlarmState attribute. The BatteryVoltageThreshold1-3 attributes SHALL be ordered in ascending order such that the BatteryVoltage level specified to trigger: BatteryVoltageThreshold3 is higher than the level specified to trigger BatteryVoltageThreshold2 BatteryVoltageThreshold2 is higher than the level specified to trigger BatteryVoltageThreshold BatteryVoltageThreshold1 is higher than the level specified to trigger BatteryVoltageMinThreshold The appropriate alarm is specified in the Alarm Code field included in the generated alarm and SHALL be one of the values in Table. The host determines which alarm code to populate based on the BatteryAlarmMask attribute and the BatteryVoltageThreshold1-3 attribute reached. If this attribute takes the value 0xff then this alarm SHALL NOT be generated.
// @DH	batteryVoltageThreshold3		Specify the low voltage alarm thresholds, measured in units of 100mV, for the BatteryVoltage attribute. If the value of BatteryVoltage drops below the threshold specified by a BatteryVoltageThreshold, an appropriate alarm SHALL be generated and/or the corresponding bit SHALL be updated in the BatteryAlarmState attribute. The BatteryVoltageThreshold1-3 attributes SHALL be ordered in ascending order such that the BatteryVoltage level specified to trigger: BatteryVoltageThreshold3 is higher than the level specified to trigger BatteryVoltageThreshold2 BatteryVoltageThreshold2 is higher than the level specified to trigger BatteryVoltageThreshold BatteryVoltageThreshold1 is higher than the level specified to trigger BatteryVoltageMinThreshold The appropriate alarm is specified in the Alarm Code field included in the generated alarm and SHALL be one of the values in Table. The host determines which alarm code to populate based on the BatteryAlarmMask attribute and the BatteryVoltageThreshold1-3 attribute reached. If this attribute takes the value 0xff then this alarm SHALL NOT be generated.
// @DH	batteryPercentageMinThreshold		Specifies the low battery percentage alarm threshold, measured in percentage (i.e., zero to 100%), for the BatteryPercentageRemaining attribute. If the value of BatteryPercentageRemaining drops below the threshold specified by a BatteryPercentageThreshold, an appropriate alarm SHALL be generated. The appropriate alarm is specified in the Alarm Code field included in the generated alarm and SHALL be the value in Table that corresponds with this threshold being reached for a given battery source. The host determines which alarm code to populate based on the BatteryAlarmMask attribute. If this attribute takes the value 0xff then this alarm SHALL NOT be generated.
// @DH	batteryPercentageThreshold1		Specify the low battery percentage alarm thresholds, measured in percentage (i.e., zero to 100%), for the BatteryPercentageRemaining attribute. If the value of BatteryPercentageRemaining drops below the threshold specified by a BatteryPercentageThreshold, an appropriate alarm SHALL be generated. The BatteryPercentageThreshold1-3 attributes SHALL be ordered in ascending order such that the BatteryPercentageRemaining level specified to trigger: BatteryPercentageThreshold3 is higher than the level specified to trigger BatteryPercentageThreshold2 BatteryPercentageThreshold2 is higher than the level specified to trigger BatteryPercentageThreshold BatteryPercentageThreshold1 is higher than the level specified to trigger BatteryPercentageMinThreshold The appropriate alarm is specified in the Alarm Code field included in the generated alarm and SHALL be one of the values in Table. The host determines which alarm code to populate based on the BatteryAlarmMask attribute and the BatteryPercentageThreshold1-3 attribute reached. If this attribute takes the value 0xff then this alarm SHALL NOT be generated.
// @DH	batteryPercentageThreshold2		Specify the low battery percentage alarm thresholds, measured in percentage (i.e., zero to 100%), for the BatteryPercentageRemaining attribute. If the value of BatteryPercentageRemaining drops below the threshold specified by a BatteryPercentageThreshold, an appropriate alarm SHALL be generated. The BatteryPercentageThreshold1-3 attributes SHALL be ordered in ascending order such that the BatteryPercentageRemaining level specified to trigger: BatteryPercentageThreshold3 is higher than the level specified to trigger BatteryPercentageThreshold2 BatteryPercentageThreshold2 is higher than the level specified to trigger BatteryPercentageThreshold BatteryPercentageThreshold1 is higher than the level specified to trigger BatteryPercentageMinThreshold The appropriate alarm is specified in the Alarm Code field included in the generated alarm and SHALL be one of the values in Table. The host determines which alarm code to populate based on the BatteryAlarmMask attribute and the BatteryPercentageThreshold1-3 attribute reached. If this attribute takes the value 0xff then this alarm SHALL NOT be generated.
// @DH	batteryPercentageThreshold3		Specify the low battery percentage alarm thresholds, measured in percentage (i.e., zero to 100%), for the BatteryPercentageRemaining attribute. If the value of BatteryPercentageRemaining drops below the threshold specified by a BatteryPercentageThreshold, an appropriate alarm SHALL be generated. The BatteryPercentageThreshold1-3 attributes SHALL be ordered in ascending order such that the BatteryPercentageRemaining level specified to trigger: BatteryPercentageThreshold3 is higher than the level specified to trigger BatteryPercentageThreshold2 BatteryPercentageThreshold2 is higher than the level specified to trigger BatteryPercentageThreshold BatteryPercentageThreshold1 is higher than the level specified to trigger BatteryPercentageMinThreshold The appropriate alarm is specified in the Alarm Code field included in the generated alarm and SHALL be one of the values in Table. The host determines which alarm code to populate based on the BatteryAlarmMask attribute and the BatteryPercentageThreshold1-3 attribute reached. If this attribute takes the value 0xff then this alarm SHALL NOT be generated.
// @DH	batteryAlarmState		Specifies the current state of the device's battery alarms. This attribute provides a persistent record of a device's battery alarm conditions as well as a mechanism for reporting changes to those conditions, including the elimination of battery alarm states (e.g., when a battery is replaced). If implemented, the server SHALL support attribute reporting for BatteryAlarmState attribute. This provides clients with a mechanism for reading the current state in case they missed the initial attribute report and also reduces network and battery use due to repeated polling of this attribute when it has not changed. It also provides a way of notifying clients when battery alarm conditions no longer exist (e.g., when the batteries have been replaced).
//			0th bit (BatteryVoltageMinThreshold or BatteryPercentageMinThreshold reached for Battery Source 1)
//			1st bit (BatteryVoltageThreshold1 or BatteryPercentageThreshold1 reached for Battery Source 1)
//			2nd bit (BatteryVoltageThreshold2 or BatteryPercentageThreshold2 reached for Battery Source 1)
//			3rd bit (BatteryVoltageThreshold3 or BatteryPercentageThreshold3 reached for Battery Source 1)
//			10th bit (BatteryVoltageMinThreshold or BatteryPercentageMinThreshold reached for Battery Source 2)
//			11st bit (BatteryVoltageThreshold1 or BatteryPercentageThreshold1 reached for Battery Source 2)
//			12nd bit (BatteryVoltageThreshold2 or BatteryPercentageThreshold2 reached Battery Source 2)
//			13rd bit (BatteryVoltageThreshold3 or BatteryPercentageThreshold3 reached Battery Source 2)
//			20th bit (BatteryVoltageMinThreshold or BatteryPercentageMinThreshold reached for Battery Source 3)
//			21st bit (BatteryVoltageThreshold1 or BatteryPercentageThreshold1 reached for Battery Source 3)
//			22nd bit (BatteryVoltageThreshold2 or BatteryPercentageThreshold2 reached Battery Source 3)
//			23rd bit (BatteryVoltageThreshold3 or BatteryPercentageThreshold3 reached Battery Source 3)
//			30th bit (Mains power supply lost/unavailable (i.e., device is running on battery))
// @DH	batteryVoltage2		The BatteryVoltage attribute is 8 bits in length and specifies the current actual (measured) battery voltage, in units of 100mV. The value 0xff indicates an invalid or unknown reading.
// @DH	batteryPercentageRemaining2		Specifies the remaining battery life as a half integer percentage of the full battery capacity (e.g., 34.5%, 45%, 68.5%, 90%) with a range between zero and 100%, with 0x00 = 0%, 0x64 = 50%, and 0xC8 = 100%. This is particularly suited for devices with rechargeable batteries. The value 0xff indicates an invalid or unknown reading. This attribute SHALL be configurable for attribute reporting.
// @DH	batteryManufacturer2		The BatteryManufacturer attribute is a maximum of 16 bytes in length and specifies the name of the battery manufacturer as a ZigBee character string
// @DH	batterySize2		The BatterySize attribute is an enumeration which specifies the type of battery being used by the device. This attribute SHALL be set to one of the non-reserved values listed in Table
//			0x00 No battery
//			0x01 Built in
//			0x02 Other
//			0x03 AA
//			0x04 AAA
//			0x05 C
//			0x06 D
//			0x07 CR2 (IEC: CR17355 / ANSI: 5046LC)
//			0x08 CR123A (IEC: CR17345 / ANSI: 5018LC)
//			0xff Unknown
// @DH	batteryAHrRating2		The BatteryAHrRating attribute is 16 bits in length and specifies the Ampere-hour rating of the battery, measured in units of 10mAHr.
// @DH	batteryQuantity2		The BatteryQuantity attribute is 8 bits in length and specifies the number of battery cells used to power the device.
// @DH	batteryRatedVoltage2		The BatteryRatedVoltage attribute is 8 bits in length and specifies the rated voltage of the battery being used in the device, measured in units of 100mV.
// @DH	batteryAlarmMask2		The BatteryAlarmMask attribute specifies which battery alarms must be generated, as listed in Table. A ‘1’ in each bit position enables the alarm. Manufacturers are responsible for determining the capability to sense and levels at which the alarms are generated
//			0th bit (Battery voltage too low to continue operating the device’s radio (i.e., BatteryVoltageMinThreshold value has been reached))
//			1st bit (Battery Alarm 1 (i.e., Battery Voltage Threshold 1 or Battery Percentage Threshold 1 value has been reached))
//			2nd bit (Battery Alarm 2 (i.e., Battery Voltage Threshold 2 or Battery Percentage Threshold 2 value has been reached))
//			3rd bit (Battery Alarm 3 (i.e., Battery Voltage Threshold 3 or Battery Percentage Threshold 3 value has been reached))
// @DH	batteryVoltageMinThreshold2		Specifies the low battery voltage alarm threshold, measured in units of 100mV at which the device can no longer operate or transmit via its radio (i.e., last gasp). If the value of BatteryVoltage drops below the threshold specified by BatteryVoltageMinThreshold, an appropriate alarm SHALL be generated and/or the corresponding bit SHALL be updated in the BatteryAlarmState attribute. In order to report to Power Configuration clients, servers that implement BatteryVoltageMinThreshold attribute SHALL implement alarming via the Alarm Cluster, attribute reporting via the BatteryAlarmState attribute, or both. For servers that implement alarming via the Alarm Cluster, the appropriate alarm is specified in the Alarm Code field included in the generated alarm and SHALL be one of the values in Table. The host determines which alarm code to populate based on the BatteryAlarmMask attribute and the BatteryVoltageMinThreshold attribute reached. For example, when the BatteryVoltage attribute reaches the value specified by the BatteryVoltageMinThreshold attribute, an alarm with the Alarm Code Field Enumeration “0x10” SHALL be generated. For servers that implement battery alarm reporting via the BatteryAlarmState attribute, the bit corresponding to the threshold level reached SHALL be set to TRUE. See the BatteryAlarmState attribute details for more information.
//			0x10 BatteryVoltageMinThreshold or BatteryPercentageMinThreshold reached for Battery Source 1
//			0x11 BatteryVoltageThreshold1 or BatteryPercentageThreshold1 reached for Battery Source 1
//			0x12 BatteryVoltageThreshold2 or BatteryPercentageThreshold2 reached for Battery Source 1
//			0x13 BatteryVoltageThreshold3 or BatteryPercentageThreshold3 reached for Battery Source 1
//			0x20 BatteryVoltageMinThreshold or BatteryPercentageMinThreshold reached for Battery Source 2
//			0x21 BatteryVoltageThreshold1 or BatteryPercentageThreshold1 reached for Battery Source 2
//			0x22 BatteryVoltageThreshold2 or BatteryPercentageThreshold2 reached Battery Source 2
//			0x23 BatteryVoltageThreshold3 or BatteryPercentageThreshold3 reached Battery Source 2
//			0x30 BatteryVoltageMinThreshold or BatteryPercentageMinThreshold reached for Battery Source 3
//			0x31 BatteryVoltageThreshold1 or BatteryPercentageThreshold1 reached for Battery Source 3
//			0x32 BatteryVoltageThreshold2 or BatteryPercentageThreshold2 reached Battery Source 3
//			0x33 BatteryVoltageThreshold3 or BatteryPercentageThreshold3 reached Battery Source 3
//			0x3a Mains power supply lost/unavailable (i.e., device is running on battery)14
//			0xff Alarm SHALL NOT be generated
// @DH	batteryVoltageThreshold12		Specify the low voltage alarm thresholds, measured in units of 100mV, for the BatteryVoltage attribute. If the value of BatteryVoltage drops below the threshold specified by a BatteryVoltageThreshold, an appropriate alarm SHALL be generated and/or the corresponding bit SHALL be updated in the BatteryAlarmState attribute. The BatteryVoltageThreshold1-3 attributes SHALL be ordered in ascending order such that the BatteryVoltage level specified to trigger: BatteryVoltageThreshold3 is higher than the level specified to trigger BatteryVoltageThreshold2 BatteryVoltageThreshold2 is higher than the level specified to trigger BatteryVoltageThreshold BatteryVoltageThreshold1 is higher than the level specified to trigger BatteryVoltageMinThreshold The appropriate alarm is specified in the Alarm Code field included in the generated alarm and SHALL be one of the values in Table. The host determines which alarm code to populate based on the BatteryAlarmMask attribute and the BatteryVoltageThreshold1-3 attribute reached. If this attribute takes the value 0xff then this alarm SHALL NOT be generated.
// @DH	batteryVoltageThreshold22		Specify the low voltage alarm thresholds, measured in units of 100mV, for the BatteryVoltage attribute. If the value of BatteryVoltage drops below the threshold specified by a BatteryVoltageThreshold, an appropriate alarm SHALL be generated and/or the corresponding bit SHALL be updated in the BatteryAlarmState attribute. The BatteryVoltageThreshold1-3 attributes SHALL be ordered in ascending order such that the BatteryVoltage level specified to trigger: BatteryVoltageThreshold3 is higher than the level specified to trigger BatteryVoltageThreshold2 BatteryVoltageThreshold2 is higher than the level specified to trigger BatteryVoltageThreshold BatteryVoltageThreshold1 is higher than the level specified to trigger BatteryVoltageMinThreshold The appropriate alarm is specified in the Alarm Code field included in the generated alarm and SHALL be one of the values in Table. The host determines which alarm code to populate based on the BatteryAlarmMask attribute and the BatteryVoltageThreshold1-3 attribute reached. If this attribute takes the value 0xff then this alarm SHALL NOT be generated.
// @DH	batteryVoltageThreshold32		Specify the low voltage alarm thresholds, measured in units of 100mV, for the BatteryVoltage attribute. If the value of BatteryVoltage drops below the threshold specified by a BatteryVoltageThreshold, an appropriate alarm SHALL be generated and/or the corresponding bit SHALL be updated in the BatteryAlarmState attribute. The BatteryVoltageThreshold1-3 attributes SHALL be ordered in ascending order such that the BatteryVoltage level specified to trigger: BatteryVoltageThreshold3 is higher than the level specified to trigger BatteryVoltageThreshold2 BatteryVoltageThreshold2 is higher than the level specified to trigger BatteryVoltageThreshold BatteryVoltageThreshold1 is higher than the level specified to trigger BatteryVoltageMinThreshold The appropriate alarm is specified in the Alarm Code field included in the generated alarm and SHALL be one of the values in Table. The host determines which alarm code to populate based on the BatteryAlarmMask attribute and the BatteryVoltageThreshold1-3 attribute reached. If this attribute takes the value 0xff then this alarm SHALL NOT be generated.
// @DH	batteryPercentageMinThreshold2		Specifies the low battery percentage alarm threshold, measured in percentage (i.e., zero to 100%), for the BatteryPercentageRemaining attribute. If the value of BatteryPercentageRemaining drops below the threshold specified by a BatteryPercentageThreshold, an appropriate alarm SHALL be generated. The appropriate alarm is specified in the Alarm Code field included in the generated alarm and SHALL be the value in Table that corresponds with this threshold being reached for a given battery source. The host determines which alarm code to populate based on the BatteryAlarmMask attribute. If this attribute takes the value 0xff then this alarm SHALL NOT be generated.
// @DH	batteryPercentageThreshold12		Specify the low battery percentage alarm thresholds, measured in percentage (i.e., zero to 100%), for the BatteryPercentageRemaining attribute. If the value of BatteryPercentageRemaining drops below the threshold specified by a BatteryPercentageThreshold, an appropriate alarm SHALL be generated. The BatteryPercentageThreshold1-3 attributes SHALL be ordered in ascending order such that the BatteryPercentageRemaining level specified to trigger: BatteryPercentageThreshold3 is higher than the level specified to trigger BatteryPercentageThreshold2 BatteryPercentageThreshold2 is higher than the level specified to trigger BatteryPercentageThreshold BatteryPercentageThreshold1 is higher than the level specified to trigger BatteryPercentageMinThreshold The appropriate alarm is specified in the Alarm Code field included in the generated alarm and SHALL be one of the values in Table. The host determines which alarm code to populate based on the BatteryAlarmMask attribute and the BatteryPercentageThreshold1-3 attribute reached. If this attribute takes the value 0xff then this alarm SHALL NOT be generated.
// @DH	batteryPercentageThreshold22		Specify the low battery percentage alarm thresholds, measured in percentage (i.e., zero to 100%), for the BatteryPercentageRemaining attribute. If the value of BatteryPercentageRemaining drops below the threshold specified by a BatteryPercentageThreshold, an appropriate alarm SHALL be generated. The BatteryPercentageThreshold1-3 attributes SHALL be ordered in ascending order such that the BatteryPercentageRemaining level specified to trigger: BatteryPercentageThreshold3 is higher than the level specified to trigger BatteryPercentageThreshold2 BatteryPercentageThreshold2 is higher than the level specified to trigger BatteryPercentageThreshold BatteryPercentageThreshold1 is higher than the level specified to trigger BatteryPercentageMinThreshold The appropriate alarm is specified in the Alarm Code field included in the generated alarm and SHALL be one of the values in Table. The host determines which alarm code to populate based on the BatteryAlarmMask attribute and the BatteryPercentageThreshold1-3 attribute reached. If this attribute takes the value 0xff then this alarm SHALL NOT be generated.
// @DH	batteryPercentageThreshold32		Specify the low battery percentage alarm thresholds, measured in percentage (i.e., zero to 100%), for the BatteryPercentageRemaining attribute. If the value of BatteryPercentageRemaining drops below the threshold specified by a BatteryPercentageThreshold, an appropriate alarm SHALL be generated. The BatteryPercentageThreshold1-3 attributes SHALL be ordered in ascending order such that the BatteryPercentageRemaining level specified to trigger: BatteryPercentageThreshold3 is higher than the level specified to trigger BatteryPercentageThreshold2 BatteryPercentageThreshold2 is higher than the level specified to trigger BatteryPercentageThreshold BatteryPercentageThreshold1 is higher than the level specified to trigger BatteryPercentageMinThreshold The appropriate alarm is specified in the Alarm Code field included in the generated alarm and SHALL be one of the values in Table. The host determines which alarm code to populate based on the BatteryAlarmMask attribute and the BatteryPercentageThreshold1-3 attribute reached. If this attribute takes the value 0xff then this alarm SHALL NOT be generated.
// @DH	batteryAlarmState2		Specifies the current state of the device's battery alarms. This attribute provides a persistent record of a device's battery alarm conditions as well as a mechanism for reporting changes to those conditions, including the elimination of battery alarm states (e.g., when a battery is replaced). If implemented, the server SHALL support attribute reporting for BatteryAlarmState attribute. This provides clients with a mechanism for reading the current state in case they missed the initial attribute report and also reduces network and battery use due to repeated polling of this attribute when it has not changed. It also provides a way of notifying clients when battery alarm conditions no longer exist (e.g., when the batteries have been replaced).
//			0th bit (BatteryVoltageMinThreshold or BatteryPercentageMinThreshold reached for Battery Source 1)
//			1st bit (BatteryVoltageThreshold1 or BatteryPercentageThreshold1 reached for Battery Source 1)
//			2nd bit (BatteryVoltageThreshold2 or BatteryPercentageThreshold2 reached for Battery Source 1)
//			3rd bit (BatteryVoltageThreshold3 or BatteryPercentageThreshold3 reached for Battery Source 1)
//			10th bit (BatteryVoltageMinThreshold or BatteryPercentageMinThreshold reached for Battery Source 2)
//			11st bit (BatteryVoltageThreshold1 or BatteryPercentageThreshold1 reached for Battery Source 2)
//			12nd bit (BatteryVoltageThreshold2 or BatteryPercentageThreshold2 reached Battery Source 2)
//			13rd bit (BatteryVoltageThreshold3 or BatteryPercentageThreshold3 reached Battery Source 2)
//			20th bit (BatteryVoltageMinThreshold or BatteryPercentageMinThreshold reached for Battery Source 3)
//			21st bit (BatteryVoltageThreshold1 or BatteryPercentageThreshold1 reached for Battery Source 3)
//			22nd bit (BatteryVoltageThreshold2 or BatteryPercentageThreshold2 reached Battery Source 3)
//			23rd bit (BatteryVoltageThreshold3 or BatteryPercentageThreshold3 reached Battery Source 3)
//			30th bit (Mains power supply lost/unavailable (i.e., device is running on battery))
// @DH	batteryVoltage3		The BatteryVoltage attribute is 8 bits in length and specifies the current actual (measured) battery voltage, in units of 100mV. The value 0xff indicates an invalid or unknown reading.
// @DH	batteryPercentageRemaining3		Specifies the remaining battery life as a half integer percentage of the full battery capacity (e.g., 34.5%, 45%, 68.5%, 90%) with a range between zero and 100%, with 0x00 = 0%, 0x64 = 50%, and 0xC8 = 100%. This is particularly suited for devices with rechargeable batteries. The value 0xff indicates an invalid or unknown reading. This attribute SHALL be configurable for attribute reporting.
// @DH	batteryManufacturer3		The BatteryManufacturer attribute is a maximum of 16 bytes in length and specifies the name of the battery manufacturer as a ZigBee character string
// @DH	batterySize3		The BatterySize attribute is an enumeration which specifies the type of battery being used by the device. This attribute SHALL be set to one of the non-reserved values listed in Table
//			0x00 No battery
//			0x01 Built in
//			0x02 Other
//			0x03 AA
//			0x04 AAA
//			0x05 C
//			0x06 D
//			0x07 CR2 (IEC: CR17355 / ANSI: 5046LC)
//			0x08 CR123A (IEC: CR17345 / ANSI: 5018LC)
//			0xff Unknown
// @DH	batteryAHrRating3		The BatteryAHrRating attribute is 16 bits in length and specifies the Ampere-hour rating of the battery, measured in units of 10mAHr.
// @DH	batteryQuantity3		The BatteryQuantity attribute is 8 bits in length and specifies the number of battery cells used to power the device.
// @DH	batteryRatedVoltage3		The BatteryRatedVoltage attribute is 8 bits in length and specifies the rated voltage of the battery being used in the device, measured in units of 100mV.
// @DH	batteryAlarmMask3		The BatteryAlarmMask attribute specifies which battery alarms must be generated, as listed in Table. A ‘1’ in each bit position enables the alarm. Manufacturers are responsible for determining the capability to sense and levels at which the alarms are generated
//			0th bit (Battery voltage too low to continue operating the device’s radio (i.e., BatteryVoltageMinThreshold value has been reached))
//			1st bit (Battery Alarm 1 (i.e., Battery Voltage Threshold 1 or Battery Percentage Threshold 1 value has been reached))
//			2nd bit (Battery Alarm 2 (i.e., Battery Voltage Threshold 2 or Battery Percentage Threshold 2 value has been reached))
//			3rd bit (Battery Alarm 3 (i.e., Battery Voltage Threshold 3 or Battery Percentage Threshold 3 value has been reached))
// @DH	batteryVoltageMinThreshold3		Specifies the low battery voltage alarm threshold, measured in units of 100mV at which the device can no longer operate or transmit via its radio (i.e., last gasp). If the value of BatteryVoltage drops below the threshold specified by BatteryVoltageMinThreshold, an appropriate alarm SHALL be generated and/or the corresponding bit SHALL be updated in the BatteryAlarmState attribute. In order to report to Power Configuration clients, servers that implement BatteryVoltageMinThreshold attribute SHALL implement alarming via the Alarm Cluster, attribute reporting via the BatteryAlarmState attribute, or both. For servers that implement alarming via the Alarm Cluster, the appropriate alarm is specified in the Alarm Code field included in the generated alarm and SHALL be one of the values in Table. The host determines which alarm code to populate based on the BatteryAlarmMask attribute and the BatteryVoltageMinThreshold attribute reached. For example, when the BatteryVoltage attribute reaches the value specified by the BatteryVoltageMinThreshold attribute, an alarm with the Alarm Code Field Enumeration “0x10” SHALL be generated. For servers that implement battery alarm reporting via the BatteryAlarmState attribute, the bit corresponding to the threshold level reached SHALL be set to TRUE. See the BatteryAlarmState attribute details for more information.
//			0x10 BatteryVoltageMinThreshold or BatteryPercentageMinThreshold reached for Battery Source 1
//			0x11 BatteryVoltageThreshold1 or BatteryPercentageThreshold1 reached for Battery Source 1
//			0x12 BatteryVoltageThreshold2 or BatteryPercentageThreshold2 reached for Battery Source 1
//			0x13 BatteryVoltageThreshold3 or BatteryPercentageThreshold3 reached for Battery Source 1
//			0x20 BatteryVoltageMinThreshold or BatteryPercentageMinThreshold reached for Battery Source 2
//			0x21 BatteryVoltageThreshold1 or BatteryPercentageThreshold1 reached for Battery Source 2
//			0x22 BatteryVoltageThreshold2 or BatteryPercentageThreshold2 reached Battery Source 2
//			0x23 BatteryVoltageThreshold3 or BatteryPercentageThreshold3 reached Battery Source 2
//			0x30 BatteryVoltageMinThreshold or BatteryPercentageMinThreshold reached for Battery Source 3
//			0x31 BatteryVoltageThreshold1 or BatteryPercentageThreshold1 reached for Battery Source 3
//			0x32 BatteryVoltageThreshold2 or BatteryPercentageThreshold2 reached Battery Source 3
//			0x33 BatteryVoltageThreshold3 or BatteryPercentageThreshold3 reached Battery Source 3
//			0x3a Mains power supply lost/unavailable (i.e., device is running on battery)14
//			0xff Alarm SHALL NOT be generated
// @DH	batteryVoltageThreshold13		Specify the low voltage alarm thresholds, measured in units of 100mV, for the BatteryVoltage attribute. If the value of BatteryVoltage drops below the threshold specified by a BatteryVoltageThreshold, an appropriate alarm SHALL be generated and/or the corresponding bit SHALL be updated in the BatteryAlarmState attribute. The BatteryVoltageThreshold1-3 attributes SHALL be ordered in ascending order such that the BatteryVoltage level specified to trigger: BatteryVoltageThreshold3 is higher than the level specified to trigger BatteryVoltageThreshold2 BatteryVoltageThreshold2 is higher than the level specified to trigger BatteryVoltageThreshold BatteryVoltageThreshold1 is higher than the level specified to trigger BatteryVoltageMinThreshold The appropriate alarm is specified in the Alarm Code field included in the generated alarm and SHALL be one of the values in Table. The host determines which alarm code to populate based on the BatteryAlarmMask attribute and the BatteryVoltageThreshold1-3 attribute reached. If this attribute takes the value 0xff then this alarm SHALL NOT be generated.
// @DH	batteryVoltageThreshold23		Specify the low voltage alarm thresholds, measured in units of 100mV, for the BatteryVoltage attribute. If the value of BatteryVoltage drops below the threshold specified by a BatteryVoltageThreshold, an appropriate alarm SHALL be generated and/or the corresponding bit SHALL be updated in the BatteryAlarmState attribute. The BatteryVoltageThreshold1-3 attributes SHALL be ordered in ascending order such that the BatteryVoltage level specified to trigger: BatteryVoltageThreshold3 is higher than the level specified to trigger BatteryVoltageThreshold2 BatteryVoltageThreshold2 is higher than the level specified to trigger BatteryVoltageThreshold BatteryVoltageThreshold1 is higher than the level specified to trigger BatteryVoltageMinThreshold The appropriate alarm is specified in the Alarm Code field included in the generated alarm and SHALL be one of the values in Table. The host determines which alarm code to populate based on the BatteryAlarmMask attribute and the BatteryVoltageThreshold1-3 attribute reached. If this attribute takes the value 0xff then this alarm SHALL NOT be generated.
// @DH	batteryVoltageThreshold33		Specify the low voltage alarm thresholds, measured in units of 100mV, for the BatteryVoltage attribute. If the value of BatteryVoltage drops below the threshold specified by a BatteryVoltageThreshold, an appropriate alarm SHALL be generated and/or the corresponding bit SHALL be updated in the BatteryAlarmState attribute. The BatteryVoltageThreshold1-3 attributes SHALL be ordered in ascending order such that the BatteryVoltage level specified to trigger: BatteryVoltageThreshold3 is higher than the level specified to trigger BatteryVoltageThreshold2 BatteryVoltageThreshold2 is higher than the level specified to trigger BatteryVoltageThreshold BatteryVoltageThreshold1 is higher than the level specified to trigger BatteryVoltageMinThreshold The appropriate alarm is specified in the Alarm Code field included in the generated alarm and SHALL be one of the values in Table. The host determines which alarm code to populate based on the BatteryAlarmMask attribute and the BatteryVoltageThreshold1-3 attribute reached. If this attribute takes the value 0xff then this alarm SHALL NOT be generated.
// @DH	batteryPercentageMinThreshold3		Specifies the low battery percentage alarm threshold, measured in percentage (i.e., zero to 100%), for the BatteryPercentageRemaining attribute. If the value of BatteryPercentageRemaining drops below the threshold specified by a BatteryPercentageThreshold, an appropriate alarm SHALL be generated. The appropriate alarm is specified in the Alarm Code field included in the generated alarm and SHALL be the value in Table that corresponds with this threshold being reached for a given battery source. The host determines which alarm code to populate based on the BatteryAlarmMask attribute. If this attribute takes the value 0xff then this alarm SHALL NOT be generated.
// @DH	batteryPercentageThreshold13		Specify the low battery percentage alarm thresholds, measured in percentage (i.e., zero to 100%), for the BatteryPercentageRemaining attribute. If the value of BatteryPercentageRemaining drops below the threshold specified by a BatteryPercentageThreshold, an appropriate alarm SHALL be generated. The BatteryPercentageThreshold1-3 attributes SHALL be ordered in ascending order such that the BatteryPercentageRemaining level specified to trigger: BatteryPercentageThreshold3 is higher than the level specified to trigger BatteryPercentageThreshold2 BatteryPercentageThreshold2 is higher than the level specified to trigger BatteryPercentageThreshold BatteryPercentageThreshold1 is higher than the level specified to trigger BatteryPercentageMinThreshold The appropriate alarm is specified in the Alarm Code field included in the generated alarm and SHALL be one of the values in Table. The host determines which alarm code to populate based on the BatteryAlarmMask attribute and the BatteryPercentageThreshold1-3 attribute reached. If this attribute takes the value 0xff then this alarm SHALL NOT be generated.
// @DH	batteryPercentageThreshold23		Specify the low battery percentage alarm thresholds, measured in percentage (i.e., zero to 100%), for the BatteryPercentageRemaining attribute. If the value of BatteryPercentageRemaining drops below the threshold specified by a BatteryPercentageThreshold, an appropriate alarm SHALL be generated. The BatteryPercentageThreshold1-3 attributes SHALL be ordered in ascending order such that the BatteryPercentageRemaining level specified to trigger: BatteryPercentageThreshold3 is higher than the level specified to trigger BatteryPercentageThreshold2 BatteryPercentageThreshold2 is higher than the level specified to trigger BatteryPercentageThreshold BatteryPercentageThreshold1 is higher than the level specified to trigger BatteryPercentageMinThreshold The appropriate alarm is specified in the Alarm Code field included in the generated alarm and SHALL be one of the values in Table. The host determines which alarm code to populate based on the BatteryAlarmMask attribute and the BatteryPercentageThreshold1-3 attribute reached. If this attribute takes the value 0xff then this alarm SHALL NOT be generated.
// @DH	batteryPercentageThreshold33		Specify the low battery percentage alarm thresholds, measured in percentage (i.e., zero to 100%), for the BatteryPercentageRemaining attribute. If the value of BatteryPercentageRemaining drops below the threshold specified by a BatteryPercentageThreshold, an appropriate alarm SHALL be generated. The BatteryPercentageThreshold1-3 attributes SHALL be ordered in ascending order such that the BatteryPercentageRemaining level specified to trigger: BatteryPercentageThreshold3 is higher than the level specified to trigger BatteryPercentageThreshold2 BatteryPercentageThreshold2 is higher than the level specified to trigger BatteryPercentageThreshold BatteryPercentageThreshold1 is higher than the level specified to trigger BatteryPercentageMinThreshold The appropriate alarm is specified in the Alarm Code field included in the generated alarm and SHALL be one of the values in Table. The host determines which alarm code to populate based on the BatteryAlarmMask attribute and the BatteryPercentageThreshold1-3 attribute reached. If this attribute takes the value 0xff then this alarm SHALL NOT be generated.
// @DH	batteryAlarmState3		Specifies the current state of the device's battery alarms. This attribute provides a persistent record of a device's battery alarm conditions as well as a mechanism for reporting changes to those conditions, including the elimination of battery alarm states (e.g., when a battery is replaced). If implemented, the server SHALL support attribute reporting for BatteryAlarmState attribute. This provides clients with a mechanism for reading the current state in case they missed the initial attribute report and also reduces network and battery use due to repeated polling of this attribute when it has not changed. It also provides a way of notifying clients when battery alarm conditions no longer exist (e.g., when the batteries have been replaced).
//			0th bit (BatteryVoltageMinThreshold or BatteryPercentageMinThreshold reached for Battery Source 1)
//			1st bit (BatteryVoltageThreshold1 or BatteryPercentageThreshold1 reached for Battery Source 1)
//			2nd bit (BatteryVoltageThreshold2 or BatteryPercentageThreshold2 reached for Battery Source 1)
//			3rd bit (BatteryVoltageThreshold3 or BatteryPercentageThreshold3 reached for Battery Source 1)
//			10th bit (BatteryVoltageMinThreshold or BatteryPercentageMinThreshold reached for Battery Source 2)
//			11st bit (BatteryVoltageThreshold1 or BatteryPercentageThreshold1 reached for Battery Source 2)
//			12nd bit (BatteryVoltageThreshold2 or BatteryPercentageThreshold2 reached Battery Source 2)
//			13rd bit (BatteryVoltageThreshold3 or BatteryPercentageThreshold3 reached Battery Source 2)
//			20th bit (BatteryVoltageMinThreshold or BatteryPercentageMinThreshold reached for Battery Source 3)
//			21st bit (BatteryVoltageThreshold1 or BatteryPercentageThreshold1 reached for Battery Source 3)
//			22nd bit (BatteryVoltageThreshold2 or BatteryPercentageThreshold2 reached Battery Source 3)
//			23rd bit (BatteryVoltageThreshold3 or BatteryPercentageThreshold3 reached Battery Source 3)
//			30th bit (Mains power supply lost/unavailable (i.e., device is running on battery))

// Power Configuration Set Battery Percentage Remaining Reporting
//
// @param: zbee
// ZBee object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: direction
// TRUE  - receiver will be configured to receiver reports
// FALSE - receiver will be configured to send reports
//
// @param: min_interval
// Minimal Reporting Interval
//
// @param: max_interval
// Maximum Reporting Interval
//
// @param: timeout
// Timeout
//
// @param: threshold
// Threshold
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_power_configuration_set_battery_percentage_remaining_reporting(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBOOL direction, ZWWORD min_interval, ZWWORD max_interval, ZWWORD timeout, ZWBYTE threshold, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Power Configuration Reset Battery Percentage Remaining Reporting
//
// @param: zbee
// ZBee object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: direction
// TRUE  - receiver will be configured to receiver reports
// FALSE - receiver will be configured to send reports
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_power_configuration_reset_battery_percentage_remaining_reporting(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBOOL direction, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Power Configuration Set Battery Percentage Remaining 2 Reporting
//
// @param: zbee
// ZBee object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: direction
// TRUE  - receiver will be configured to receiver reports
// FALSE - receiver will be configured to send reports
//
// @param: min_interval
// Minimal Reporting Interval
//
// @param: max_interval
// Maximum Reporting Interval
//
// @param: timeout
// Timeout
//
// @param: threshold
// Threshold
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_power_configuration_set_battery_percentage_remaining_2_reporting(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBOOL direction, ZWWORD min_interval, ZWWORD max_interval, ZWWORD timeout, ZWBYTE threshold, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Power Configuration Reset Battery Percentage Remaining 2 Reporting
//
// @param: zbee
// ZBee object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: direction
// TRUE  - receiver will be configured to receiver reports
// FALSE - receiver will be configured to send reports
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_power_configuration_reset_battery_percentage_remaining_2_reporting(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBOOL direction, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Power Configuration Set Battery Percentage Remaining 3 Reporting
//
// @param: zbee
// ZBee object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: direction
// TRUE  - receiver will be configured to receiver reports
// FALSE - receiver will be configured to send reports
//
// @param: min_interval
// Minimal Reporting Interval
//
// @param: max_interval
// Maximum Reporting Interval
//
// @param: timeout
// Timeout
//
// @param: threshold
// Threshold
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_power_configuration_set_battery_percentage_remaining_3_reporting(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBOOL direction, ZWWORD min_interval, ZWWORD max_interval, ZWWORD timeout, ZWBYTE threshold, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Power Configuration Reset Battery Percentage Remaining 3 Reporting
//
// @param: zbee
// ZBee object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: direction
// TRUE  - receiver will be configured to receiver reports
// FALSE - receiver will be configured to send reports
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_power_configuration_reset_battery_percentage_remaining_3_reporting(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBOOL direction, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);


// Cluster Class Identify (0x0003/3) //
// Profile ZigBee Device Profile (0x0104/260) //
// Attributes and commands to put a device into an Identification mode (e.g., flashing a light), that indicates to an observer – e.g., an installer - which of several devices it is, also to request any device that is identifying itself to respond to the initiator.
//
// @DH	identifyTime		The IdentifyTime attribute specifies the remaining length of time, in seconds, that the device will continue to identify itself.

// Identify Identify
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: identify_time
// Identify Time
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_identify_identify(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWWORD identify_time, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Identify Identify Query
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_identify_identify_query(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Identify Trigger Effect
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: effect_id
// The Effect Identifier field specifies the identify effect to use:
//  0x00 - Blink. e.g., Light is turned on/off once.
//  0x01 - Breathe. e.g., Light turned on/off over 1 second and repeated 15 times.
//  0x02 - Okay. e.g., Colored light turns green for 1 second; noncolored light flashes twice.
//  0x0b - Channel change. e.g., Colored light turns orange for 8 seconds; noncolored light switches to maximum brightness for 0.5s and then minimum brightness for 7.5s.
//  0xfe - Finish effect. Complete the current effect sequence before terminating. e.g., if in the middle of a breathe effect (as above), first complete the current 1s breathe effect and then terminate the effect.
//  0xff - Stop effect. Terminate the effect as soon as possible
// 
// @param: effect_var
// @default: 0x00
// Effect Variant:
//   0x00 - Default
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_identify_trigger_effect(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBYTE effect_id, ZWBYTE effect_var, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Identify Set Identify Time
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: identify_time
// It specifies the remaining length of time, in seconds, that the device will continue to identify itself
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_identify_set_identify_time(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWWORD identify_time, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Identify Get
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_identify_get(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);


// Cluster Class Ias Zone (0x0500/1280) //
// Profile ZigBee Device Profile (0x0104/260) //
// The IAS Zone cluster defines an interface to the functionality of an IAS security zone device. IAS Zone supports up to two alarm types per zone, low battery reports and supervision of the IAS network
//
// @DH	zoneState	The ZoneState attribute contains the values summarized in Table.
// 			0x00 - NotEnroll
//			0x01 - Enroll   
// @DH	zoneType	The Zone Type dictates the meaning of Alarm1 and Alarm2 bits of the ZoneStatus attribute, as also indicated in this table.
//			0x0000 - StandardCEI
//			0x000d - MotionSensor
//			0x0015 - ContactSwitch
//			0x0028 - FireSensor
//			0x002a - WaterSensor
//			0x002b - COSensor
//			0x002c - PersonalEmergencyDevice
//			0x002d - VibrationMovementSensor
//			0x010f - RemoteControl
//			0x0115 - KeyFob
//			0x021d - Keypad
//			0x0225 - StandardWarningDevice
//			0x0226 - GlassBreakSensor
//			0x0229 - SecurityRepeater
//			0xffff - InvalidZoneType
// @DH	zoneStatus	The ZoneStatus attribute is a bit map.
//			0th bit (Alarm1)
//			 1 – opened or alarmed
//			 0 – closed or not alarmed
//			1st bit (Alarm2)
//			 1 – opened or alarmed
//			 0 – closed or not alarmed
//			2nd bit (Tamper)
//			 1 – Tampered
//			 0 – Not tampered
//			3rd bit (Battery)
//			 1 – Low battery
//			 0 – Battery OK
//			4th bit (Supervision reports (Note 1) )
//			 1 – Reports
//			 0 – Does not report
//			5th bit (Restore reports (Note 2) )
//			 1 – Reports restore
//			 0 – Does not report restore
//			6th bit (Trouble )
//			 1 – Trouble/Failure
//			 0 – OK
//			7th bit (AC (mains))
//			 1 – AC/Mains fault
//			 0 – AC/Mains OK
//			8th bit (Test )
//			 1 – Sensor is in test mode
//			 0 – Sensor is in operation mode
//			9th bit (Battery Defect)
//			 1 – Sensor detects a defective battery
//			 0 – Sensor battery is functioning normally
// @DH	iasCieAddress	The IAS_CIE_Address attribute specifies the address that commands generated by the server SHALL be sent to. All commands received by the server must also come from this address. It is up to the zone's specific implementation to permit or deny change (write) of this attribute at specific times
// @DH	zoneId		A unique reference number allocated by the CIE at zone enrollment time.
// 			0x00...0xFE - The unique identifier of zone
// 			0xFF - Invalid zone identifier
// @DH	numberOfZoneSensitivityLevelsSupported	Provides the total number of sensitivity levels supported by the IAS Zone server. 
// @DH	CurrentZoneSensitivityLevel	Allows an IAS Zone client to query and configure the IAS Zone server’s sensitivity level.
// @DH	delay		The Delay field is defined as the amount of time, in quarter-seconds, from the moment when a change takes place in one or more bits of the Zone Status attribute and the successful transmission of the Zone Status Change Notification. This is designed to help congested networks or offline servers quantify the amount of time from when an event was detected and when it could be reported to the client.

// IAS Zone Initiate Normal Operation Mode
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_ias_zone_initiate_normal_operation_mode(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// IAS Zone Initiate Test Mode
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: test_mode_duration
// Specifies the duration, in seconds, for which the IAS Zone server SHALL operate in its test mode
//
// @param: current_zone_sensivity_level
// Specifies the sensitivity level the IAS Zone server SHALL use for the duration of the Test Mode and with which it must update its CurrentZoneSensitivityLevel attribute.
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_ias_zone_initiate_test_mode(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBYTE test_mode_duration, ZWBYTE current_zone_sensivity_level, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Enroll Device
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: zone_id
// Zone Identifier
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_ias_zone_enroll(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBYTE zone_id, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Ias Zone Get
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_ias_zone_get(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Ias Zone Configuration Get
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_ias_zone_configuration_get(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);


// Cluster Class Window Covering (0x0102/258) //
// Profile ZigBee Device Profile (0x0104/260) //
// The window covering cluster provides an interface for controlling and adjusting automatic window coverings such as drapery motors, automatic shades, and blinds
// @DH	windowCoveringType		The WindowCoveringType attribute identifies the type of window covering being controlled by this endpoint and SHALL be set to one of the non-reserved values in Table
//				0x00 Rollershade                      , Supported Action - Lift
//				0x01 Rollershade - 2 Motor            , Supported Action - Lift
//				0x02 Rollershade – Exterior           , Supported Action - Lift
//				0x03 Rollershade - Exterior - 2 Motor , Supported Action - Lift
//				0x04 Drapery                          , Supported Action - Lift
//				0x05 Awning                           , Supported Action - Lift
//				0x06 Shutter                          , Supported Action - Tilt
//				0x07 Tilt Blind - Tilt Only           , Supported Action - Tilt
//				0x08 Tilt Blind - Lift and Tilt Lift, , Supported Action - Tilt
//				0x09 Projector Screen                 , Supported Action - Lift
// @DH	physicalClosedLimitLift		The PhysicalClosedLimitLift attribute identifies the maximum possible encoder position possible (in centimeters) to position the height of the window covering – this is ignored if the device is running in Open Loop Control.
// @DH	physicalClosedLimitTilt		The PhysicalClosedLimitTilt attribute identifies the maximum possible encoder position possible (tenth of a degrees) to position the angle of the window covering – this is ignored if the device is running in Open Loop Control.
// @DH	currentPositionLift		The CurrentPositionLift attribute identifies the actual position (in centimeters) of the window covering from the top of the shade if Closed Loop Control is enabled. This attribute is ignored if the device is running in Open Loop Control.
// @DH	currentPositionTilt		The CurrentPositionTilt attribute identifies the actual tilt position (in tenth of an degree) of the window covering from Open if Closed Loop Control is enabled. This attribute is ignored if the device is running in Open Loop Control.
// @DH	numberOfActuationsLift		The NumberOfActuationsLift attribute identifies the total number of lift actuations applied to the Window Covering since the device was installed.
// @DH	numberOfActuationsTilt		The NumberOfActuationsTilt attribute identifies the total number of tilt actuations applied to the Window Covering since the device was installed.
// @DH	configStatus		The ConfigStatus attribute makes configuration and status information available. To change settings, devices SHALL write to the Mode attribute of the Window Covering Settings Attribute Set. The behavior causing the setting or clearing of each bit is vendor specific. See Table for details on each bit.
//				0th bit (Operational: This status bit defines if the Window Covering is operational.)
//				 0 - Not Operational
//				 1 - Operational
//				1st bit (Online: This status bit defines if the Window Covering is enabled for transmitting over the ZigBee network.)
//				 0 - Not Online
//				 1 - Online
//				2nd bit (Reversal – Lift commands: This status bit identifies if the direction of rotation for the Window Covering has been reversed in order for Open/Up commands to match the physical installation condition.)
//				 0 - Commands are normal
//				 1 - Open/Up Commands have been reversed
//				3rd bit (Control – Lift: This status bit identifies if the window covering supports Open Loop or Closed Loop Lift Control)
//				 0 - Lift control is Open Loop
//				 1 - Lift control is Closed Loop
//				4th bit (Control – Tilt: This status bit identifies if the window covering supports Open Loop or Closed Loop Tilt Control)
//				 0 - Tilt control is Open Loop
//				 1 - Tilt control is Closed Loop
//				5th bit (This bit is Ignored if running Lift in Open Loop Control. Encoder – Lift: This status bit identifies if a Closed Loop Controlled Window Covering is employing an encoder for positioning the height of the window covering.)
//				 0 - Timer Controlled
//				 1 - Encoder Controlled
//				6th bit (This bit is Ignored if running Tilt in Open Loop Control. Encoder – Tilt: This status bit identifies if a Closed Loop Controlled Window Covering is employing an encoder for tilting the window covering.)
//				 0 - Timer Controlled
//				 1 - Encoder Controlled
//				7th bit (Reserved)
// @DH	currentPositionLiftPercentage		The CurrentPositionLiftPercentage attribute identifies the actual position as a percentage between the InstalledOpenLimitLift attribute and the InstalledClosedLimitLift58attribute of the window covering from the up/open position if Closed Loop Control is enabled. If the device is running in Open Loop Control or the device only supports Tilt actions, this attribute is not required as an attribute but has a special interpretation when received as part of a scene command (see “Scene Table Extensions” below).
// @DH	currentPositionTiltPercentage		The CurrentPositionTiltPercentage attribute identifies the actual position as a percentage between the InstalledOpenLimitTilt attribute and the InstalledClosedLimitTilt59attribute of the window covering from the up/open position if Closed Loop Control is enabled. If the device is running in Open Loop Control or the device only support Lift actions, this attribute is not required as an attribute but has a special interpretation when received as part of a scene command (see “Scene Table Extensions” below).
// @DH	installedOpenLimitLift		The InstalledOpenLimitLift attribute identifies the Open Limit for Lifting the Window Covering whether position (in centimeters) is encoded or timed. This attribute is ignored if the device is running in Open Loop Control or only supports Tilt actions
// @DH	installedClosedLimitLift		The InstalledClosedLimitLift attribute identifies the Closed Limit for Lifting the Window Covering whether position (in centimeters) is encoded or timed. This attribute is ignored if the device is running in Open Loop Control or only supports Tilt actions.
// @DH	installedOpenLimitTilt		The InstalledOpenLimitTilt attribute identifies the Open Limit for Tilting the Window Covering whether position (in tenth of a degree) is encoded or timed. This attribute is ignored if the device is running in Open Loop Control or only supports Lift actions
// @DH	installedClosedLimitTilt		The InstalledClosedLimitTilt attribute identifies the Closed Limit for Tilting the Window Covering whether position (in tenth of a degree) is encoded or timed. This attribute is ignored if the device is running in Open Loop Control or only supports Lift actions.
// @DH	velocityLift		The VelocityLift attribute identifies the velocity (in centimeters per second) associated with Lifting the Window Covering
// @DH	accelerationTimeLift		The AccelerationTimeLift attribute identifies any ramp up times to reaching the velocity setting (in tenth of a second) for positioning the Window Covering.
// @DH	decelerationTimeLift		The DecelerationTimeLift attribute identifies any ramp down times associated with stopping the positioning (in tenth of a second) of the Window Covering.
// @DH	mode		The Mode attribute allows configuration of the Window Covering, such as: reversing the motor direction, placing the Window Covering into calibration mode, placing the motor into maintenance mode, disabling the ZigBee network, and disabling status LEDs
//				0th bit (Disables (0) or Enables (1) the reversal of the motor rotating direction associated with an UP/OPEN command. Should be set so that an UP/OPEN command matches moving the Window Covering physically in that direction.)
//				 0 - motor direction is normal
//				 1 - motor direction is reversed
//				1st bit (Disables (0) or Enables (1) placing the Window Covering into Calibration Mode where limits are either setup using physical tools or limits are learned by the controller based on physical setup of the Window Covering by an installer.)
//				 0 - run in normal mode
//				 1 - run in calibration mode
//				2nd bit (Disables (0) or Enables (1) placing the motor into Maintenance Mode where the motor cannot be moved over the network or by a switch connected to a Local Switch Input.)
//				 0 - motor is running normally
//				 1 - motor is running in maintenance mode
//				3rd bit (Disables (0) or Enables (1) the display of any feedback LEDs resident especially on the packaging of an endpoint where they may cause distraction to the occupant.)
//				 0 - LEDs are off
//				 1 - LEDs will display feedback
//				4th - 7th bits (Reserved)
// @DH	intermediateSetpointsLift		Identifies the number of Intermediate Setpoints supported by the Window Covering for Lift and then identifies the position settings for those Intermediate Setpoints if Closed Loop Control is supported. This is a comma delimited ASCII character string. For example: “2,0x0013, 0x0030”
// @DH	intermediateSetpointsTilt		Identifies the number of Intermediate Setpoints supported by the Window Covering for Tilt and then identifies the position settings for those Intermediate Setpoints if Closed Loop Control is supported. This is a comma delimited ASCII character string. For example: “2,0x0013, 0x0030”

// Window Covering Up Open
//
// @param: zbee
// ZBee object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// @default: NULL
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_window_covering_up_open(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Window Covering Down Close
//
// @param: zbee
// ZBee object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// @default: NULL
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_window_covering_down_close(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Window Covering Stop
//
// @param: zbee
// ZBee object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// @default: NULL
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_window_covering_stop(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Window Covering Go To Lift Value
//
// @param: zbee
// ZBee object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: lift_value
// Lift Value
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// @default: NULL
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_window_covering_go_to_lift_value(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWWORD lift_value, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Window Covering Go To Lift Value
//
// @param: zbee
// ZBee object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: percentage_lift_value
// Lift Value Percentage
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// @default: NULL
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_window_covering_go_to_lift_percentage(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBYTE percentage_lift_value, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Window Covering Go To Lift Value
//
// @param: zbee
// ZBee object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: tilt_value
// Tilt Value
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// @default: NULL
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_window_covering_go_to_tilt_value(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWWORD tilt_value, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Window Covering Go To Lift Value
//
// @param: zbee
// ZBee object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: percentage_tilt_value
// Tilt Value Percentage
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// @default: NULL
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_window_covering_go_to_tilt_percentage(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBYTE percentage_tilt_value, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Window Covering Set Current Possion Lift Reporting
//
// @param: zbee
// ZBee object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: direction
// TRUE  - receiver will be configured to receiver reports
// FALSE - receiver will be configured to send reports
//
// @param: min_interval
// Minimal Reporting Interval
//
// @param: max_interval
// Maximum Reporting Interval
//
// @param: timeout
// Timeout
//
// @param: threshold
// Threshold
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// @default: NULL
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_window_covering_set_current_position_lift_reporting(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBOOL direction, ZWWORD min_interval, ZWWORD max_interval, ZWWORD timeout, ZWBYTE threshold, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Window Covering Reset Current Position Lift Reporting
//
// @param: zbee
// ZBee object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: direction
// TRUE  - receiver will be configured to receiver reports
// FALSE - receiver will be configured to send reports
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// @default: NULL
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_window_covering_reset_current_position_lift_reporting(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBOOL direction, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Window Covering Set Current Possion Tilt Reporting
//
// @param: zbee
// ZBee object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: direction
// TRUE  - receiver will be configured to receiver reports
// FALSE - receiver will be configured to send reports
//
// @param: min_interval
// Minimal Reporting Interval
//
// @param: max_interval
// Maximum Reporting Interval
//
// @param: timeout
// Timeout
//
// @param: threshold
// Threshold
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// @default: NULL
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_window_covering_set_current_position_tilt_reporting(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBOOL direction, ZWWORD min_interval, ZWWORD max_interval, ZWWORD timeout, ZWBYTE threshold, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Window Covering Reset Current Position Tilt Reporting
//
// @param: zbee
// ZBee object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: direction
// TRUE  - receiver will be configured to receiver reports
// FALSE - receiver will be configured to send reports
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// @default: NULL
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_window_covering_reset_current_position_tilt_reporting(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBOOL direction, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);


// Cluster Class Color Control (0x0300/768) //
// Profile ZigBee Device Profile (0x0104/260) //
// This cluster provides an interface for changing the color of a light. Color is specified according to the Commission Internationale de l'Éclairage (CIE) specification CIE 1931 Color Space. Color control is carried out in terms of x,y values, as defined by this specification.
//
// @DH	currentHue		It contains the current hue value of the light. It is updated as fast as practical during commands that change the hue. The hue in degrees SHALL be related to the CurrentHue attribute by the relationship Hue = CurrentHue x 360 / 254 (CurrentHue in the range 0 - 254 inclusive)
//				0x00...0xFE - valid
//				0xFF - invalid
// @DH	currentSaturation	It holds the current saturation value of the light. It is updated as fast as practical during commands that change the saturation. The saturation SHALL be related to the CurrentSaturation attribute by the relationship Saturation = CurrentSaturation/254 (CurrentSaturation in the range 0 - 254 inclusive)
//				0x00...0xFE - valid
//				0xFF - invalid
// @DH	remainingTime  		It holds the time remaining, in 1/10ths of a second, until the currently active command will be complete.
// @DH	currentX		The currentX attribute contains the current value of the normalized chromaticity value x, as defined in the CIE xyY Color Space. It is updated as fast as practical during commands that change the color. The value of x SHALL be related to the CurrentX attribute by the relationship x = CurrentX / 65536 (CurrentX in the range 0 to 65279 inclusive)
// @DH	currentY		The currentY attribute contains the current value of the normalized chromaticity value y, as defined in the CIE xyY Color Space. It is updated as fast as practical during commands that change the color. The value of y SHALL be related to the CurrentY attribute by the relationship y = CurrentY / 65536 (CurrentY in the range 0 to 65279 inclusive)
// @DH	driftCompensation	It indicates what mechanism, if any, is in use for compensation for color/intensity drift over time
//				0x00 - None
//				0x01 - Other / Unknown
//				0x02 - Temperature monitoring
//				0x03 - Optical luminance monitoring and feedback
//				0x04 - Optical color monitoring and feedback
// @DH	compensationText	It holds a textual indication of what mechanism, if any, is in use to compensate for color/intensity drift over time.
// @DH	colorTemperatureMireds	The ColorTemperatureMireds attribute contains a scaled inverse of the current value of the color temperature. The unit of ColorTemperatureMireds is the mired (micro reciprocal degree), AKA mirek (micro reciprocal kelvin). It is updated as fast as practical during commands that change the color. Color temperature in kelvins = 1,000,000 / ColorTemperatureMireds, where ColorTemperatureMireds is in the range 1 to 65279 mireds inclusive, giving a color temperature range from 1,000,000 kelvins to 15.32 kelvins.
// @DH	colorMode		It indicates which attributes are currently determining the color of the device. 
//				0x00 - CurrentHue and CurrentSaturation
//				0x01 - CurrentX and CurrentY
//				0x02 - ColorTemperatureMired
// @DH	enhancedCurrentHue	It represents non-equidistant steps along the CIE 1931 color triangle, and it provides 16-bits precision. The upper 8 bits of this attribute SHALL be used as an index in the implementation specific XY lookup table to provide the non-equidistance steps (see the ZLL test specification for an example). The lower 8 bits SHALL be used to interpolate between these steps in a linear way in order to provide color zoom for the user. To provide compatibility with standard ZCL, the CurrentHue attribute SHALL contain a hue value in the range 0 to 254, calculated from the EnhancedCurrentHue attribute.
// @DH	enhancedColorMode	It specifies which attributes are currently determining the color of the device.
//				0x00 - CurrentHue and CurrentSaturation
//				0x01 - CurrentX and CurrentY
//				0x02 - ColorTemperatureMireds
//				0x03 - EnhancedCurrentHue and CurrentSaturation
// @DH	colorLoopActive		It specifies the current active status of the color loop. 
//				0x00 - the color loop SHALL not be active
//				0x01 - the color loop SHALL be active
//				0x02...0xff - are reserved.
// @DH	colorLoopDirection	It specifies the current direction of the color loop. 
//				0x00	  - the EnhancedCurrentHue attribute SHALL be decremented
//				0x01	  - the EnhancedCurrentHue attribute SHALL be incremented
//				0x02...0xff - are reserved
// @DH	colorLoopTime		It specifies the number of seconds it SHALL take to perform a full color loop, i.e., to cycle all values of the EnhancedCurrentHue attribute (between 0x0000 and 0xffff)
// @DH	colorLoopStartEnhancedHue	It specifies the value of the EnhancedCurrentHue attribute from which the color loop SHALL be started
// @DH	colorLoopStoredEnhancedHue	It pecifies the value of the EnhancedCurrentHue attribute before the color loop was started. Once the color loop is complete, the EnhancedCurrentHue attribute SHALL be restored to this value.
// @DH	colorCapabilities	It specifies the color capabilities of the device supporting the color control cluster. If a bit is set to 1, the corresponding attributes and commands SHALL become mandatory. If a bit is set to 0, the corresponding attributes and commands need not be implemented
//				0th bit (Hue/saturation supported)
//				 1 - implemented attributes: CurrentHue, CurrentSaturation; commands: Move to hue, Move hue, Step hue, Move to saturation, Move saturation, Step saturation, Move to hue and saturation, Stop move step
//				 0 - not implemented
//				1st bit (Enhanced hue supported)
//				 1 - implemented attributes: EnhancedCurrentHue; commands: Enhanced move to hue, Enhanced move hue, Enhanced step hue, Enhanced move to hue and saturation, Stop move step
//				 0 - not implemented
//				2nd bit (Color loop supported)
//				 1 - implemented attributes: ColorLoopActive, ColorLoopDirection, ColorLoopTime, ColorLoopStartEnhancedHue, ColorLoopStoredEnhancedHue; commands: Color loop set
//				 0 - not implemented
//				3rd bit (XY attributes supported)
//				 1 - implemented attributes: CurrentX, CurrentY; commands: Move to color, Move color, Step color, Stop move step
//				 0 - not implemented
//				4th bit (Color temperature supported)
//				 1 - implemented attributes: ColorTemperatureMireds, ColorTempPhysicalMinMireds, ColorTempPhysicalMaxMireds; commands: Move to color temperature, Move color temperature, Step color temperature, Stop move step
//				 0 - not implemented 
// @DH	colorTempPhysicalMinMireds	It indicates the minimum mired value supported by the hardware. ColorTempPhysicalMinMireds corresponds to the maximum color temperature in kelvins supported by the hardware. ColorTempPhysicalMinMireds <= ColorTemperatureMireds.
// @DH	colorTempPhysicalMaxMireds	It indicates the maximum mired value supported by the hardware. ColorTempPhysicalMaxMireds corresponds to the minimum color temperature in kelvins supported by the hardware. ColorTemperatureMireds <= ColorTempPhysicalMaxMireds.
// @DH	numberOfPrimaries	It contains the number of color primaries implemented on this device. A value of 0xff SHALL indicate that the number of primaries is unknown Where this attribute is implemented, the attributes below for indicating the “x” and “y” color values of the primaries SHALL also be implemented for each of the primaries from 1 to NumberOfPrimaries, without leaving gaps. Implementation of the Primary1Intensity attribute and subsequent intensity attributes is optional.
//				0x00...0xFE - number of color primaries implemented on this device
//				0xFF - unknown
// @DH	primary1X		It contains the normalized chromaticity value x for this primary, as defined in the CIE xyY Color Space. The value of x SHALL be related to the Primary1X attribute by the relationship x = Primary1X / 65536 (Primary1X in the range 0 to 65279 inclusive)
//				0x0000...0xFEFF - valid
//				0xFFFF - invalid
// @DH	primary1Y		It contains the normalized chromaticity value y for this primary, as defined in the CIE xyY Color Space. The value of y SHALL be related to the Primary1Y attribute by the relationship y = Primary1Y / 65536 (Primary1Y in the range 0 to 65279 inclusive)
//				0x0000...0xFEFF - valid
//				0xFFFF - invalid
// @DH	primary1Intensity	It contains a representation of the maximum intensity of this primary as defined in the Dimming Light Curve in the Ballast Configuration cluster, normalized such that the primary with the highest maximum intensity contains the value 0xfe. A value of 0xff SHALL indicate that this primary is not available.
//				0x00...0xFE - valid
//				0xFF - invalid
// @DH	primary2X		It contains the normalized chromaticity value x for this primary, as defined in the CIE xyY Color Space. The value of x SHALL be related to the Primary2X attribute by the relationship x = Primary2X / 65536 (Primary2X in the range 0 to 65279 inclusive)
//				0x0000...0xFEFF - valid
//				0xFFFF - invalid
// @DH	primary2Y		It contains the normalized chromaticity value y for this primary, as defined in the CIE xyY Color Space. The value of y SHALL be related to the Primary2Y attribute by the relationship y = Primary2Y / 65536 (Primary2Y in the range 0 to 65279 inclusive)
//				0x0000...0xFEFF - valid
//				0xFFFF - invalid
// @DH	primary2Intensity	It contains a representation of the maximum intensity of this primary as defined in the Dimming Light Curve in the Ballast Configuration cluster, normalized such that the primary with the highest maximum intensity contains the value 0xfe. A value of 0xff SHALL indicate that this primary is not available.
//				0x00...0xFE - valid
//				0xFF - invalid
// @DH	primary3X		It contains the normalized chromaticity value x for this primary, as defined in the CIE xyY Color Space. The value of x SHALL be related to the Primary3X attribute by the relationship x = Primary3X / 65536 (Primary3X in the range 0 to 65279 inclusive)
//				0x0000...0xFEFF - valid
//				0xFFFF - invalid
// @DH	primary3Y		It contains the normalized chromaticity value y for this primary, as defined in the CIE xyY Color Space. The value of y SHALL be related to the Primary3Y attribute by the relationship y = Primary3Y / 65536 (Primary3Y in the range 0 to 65279 inclusive)
//				0x0000...0xFEFF - valid
//				0xFFFF - invalid
// @DH	primary3Intensity	It contains a representation of the maximum intensity of this primary as defined in the Dimming Light Curve in the Ballast Configuration cluster, normalized such that the primary with the highest maximum intensity contains the value 0xfe. A value of 0xff SHALL indicate that this primary is not available.
//				0x00...0xFE - valid
//				0xFF - invalid
// @DH	primary4X		It contains the normalized chromaticity value x for this primary, as defined in the CIE xyY Color Space. The value of x SHALL be related to the Primary4X attribute by the relationship x = Primary4X / 65536 (Primary4X in the range 0 to 65279 inclusive)
//				0x0000...0xFEFF - valid
//				0xFFFF - invalid
// @DH	primary4Y		It contains the normalized chromaticity value y for this primary, as defined in the CIE xyY Color Space. The value of y SHALL be related to the Primary4Y attribute by the relationship y = Primary4Y / 65536 (Primary4Y in the range 0 to 65279 inclusive)
//				0x0000...0xFEFF - valid
//				0xFFFF - invalid
// @DH	primary4Intensity	It contains a representation of the maximum intensity of this primary as defined in the Dimming Light Curve in the Ballast Configuration cluster, normalized such that the primary with the highest maximum intensity contains the value 0xfe. A value of 0xff SHALL indicate that this primary is not available.
//				0x00...0xFE - valid
//				0xFF - invalid
// @DH	primary5X		It contains the normalized chromaticity value x for this primary, as defined in the CIE xyY Color Space. The value of x SHALL be related to the Primary5X attribute by the relationship x = Primary5X / 65536 (Primary5X in the range 0 to 65279 inclusive)
//				0x0000...0xFEFF - valid
//				0xFFFF - invalid
// @DH	primary5Y		It contains the normalized chromaticity value y for this primary, as defined in the CIE xyY Color Space. The value of y SHALL be related to the Primary5Y attribute by the relationship y = Primary5Y / 65536 (Primary5Y in the range 0 to 65279 inclusive)
//				0x0000...0xFEFF - valid
//				0xFFFF - invalid
// @DH	primary5Intensity	It contains a representation of the maximum intensity of this primary as defined in the Dimming Light Curve in the Ballast Configuration cluster, normalized such that the primary with the highest maximum intensity contains the value 0xfe. A value of 0xff SHALL indicate that this primary is not available.
//				0x00...0xFE - valid
//				0xFF - invalid
// @DH	primary6X		It contains the normalized chromaticity value x for this primary, as defined in the CIE xyY Color Space. The value of x SHALL be related to the Primary6X attribute by the relationship x = Primary6X / 65536 (Primary6X in the range 0 to 65279 inclusive)
//				0x0000...0xFEFF - valid
//				0xFFFF - invalid
// @DH	primary6Y		It contains the normalized chromaticity value y for this primary, as defined in the CIE xyY Color Space. The value of y SHALL be related to the Primary6Y attribute by the relationship y = Primary6Y / 65536 (Primary6Y in the range 0 to 65279 inclusive)
//				0x0000...0xFEFF - valid
//				0xFFFF - invalid
// @DH	primary6Intensity	It contains a representation of the maximum intensity of this primary as defined in the Dimming Light Curve in the Ballast Configuration cluster, normalized such that the primary with the highest maximum intensity contains the value 0xfe. A value of 0xff SHALL indicate that this primary is not available.
//				0x00...0xFE - valid
//				0xFF	  - invalid
// @DH	whitePointX		It contains the normalized chromaticity value x, as defined in the CIE xyY Color Space, of the current white point of the device. The value of x SHALL be related to the WhitePointX attribute by the relationship x = WhitePointX / 65536 (WhitePointX in the range 0 to 65279 inclusive)
//				0x0000...0xFEFF - valid
//				0xFFFF - invalid
// @DH	whitePointY		It contains the normalized chromaticity value y, as defined in the CIE xyY Color Space, of the current white point of the device. The value of y SHALL be related to the WhitePointY attribute by the relationship y = WhitePointY / 65536 (WhitePointY in the range 0 to 65279 inclusive)
//				0x0000...0xFEFF - valid
//				0xFFFF - invalid
// @DH	colorPointRX		It contains the normalized chromaticity value x, as defined in the CIE xyY Color Space, of the red color point of the device. The value of x SHALL be related to the ColorPointRX attribute by the relationship x = ColorPointRX / 65536 (ColorPointRX in the range 0 to 65279 inclusive)
//				0x0000...0xFEFF - valid
//				0xFFFF - invalid
// @DH	colorPointRY		It contains the normalized chromaticity value y, as defined in the CIE xyY Color Space, of the red color point of the device. The value of y SHALL be related to the ColorPointRY attribute by the relationship y = ColorPointRY / 65536 (ColorPointRY in the range 0 to 65279 inclusive)
//				0x0000...0xFEFF - valid
//				0xFFFF - invalid
// @DH	colorPointRIntensity	It contains a representation of the relative intensity of the red color point as defined in the Dimming Light Curve in the Ballast Configuration cluster, normalized such that the color point with the highest relative intensity contains the value 0xfe. A value of 0xff SHALL indicate an invalid value.
//				0x00...0xFE - valid
//				0xFF - invalid
// @DH	colorPointGX		It contains the normalized chromaticity value x, as defined in the CIE xyY Color Space, of the green color point of the device. The value of x SHALL be related to the ColorPointGX attribute by the relationship x = ColorPointRX / 65536 (ColorPointRX in the range 0 to 65279 inclusive)
//				0x0000...0xFEFF - valid
//				0xFFFF - invalid
// @DH	colorPointGY		It contains the normalized chromaticity value y, as defined in the CIE xyY Color Space, of the green color point of the device. The value of y SHALL be related to the ColorPointGY attribute by the relationship y = ColorPointRY / 65536 (ColorPointRY in the range 0 to 65279 inclusive)
//				0x0000...0xFEFF - valid
//				0xFFFF - invalid
// @DH	colorPointGIntensity	It contains a representation of the relative intensity of the red color point as defined in the Dimming Light Curve in the Ballast Configuration cluster, normalized such that the color point with the highest relative intensity contains the value 0xfe. A value of 0xff SHALL indicate an invalid value.
//				0x00...0xFE - valid
//				0xFF - invalid
// @DH	colorPointBX		It contains the normalized chromaticity value x, as defined in the CIE xyY Color Space, of the blue color point of the device. The value of x SHALL be related to the ColorPointBX attribute by the relationship x = ColorPointRX / 65536 (ColorPointRX in the range 0 to 65279 inclusive)
//				0x0000...0xFEFF - valid
//				0xFFFF - invalid
// @DH	colorPointBY		It contains the normalized chromaticity value y, as defined in the CIE xyY Color Space, of the blue color point of the device. The value of y SHALL be related to the ColorPointBY attribute by the relationship y = ColorPointRY / 65536 (ColorPointRY in the range 0 to 65279 inclusive)
//				0x0000...0xFEFF - valid
//				0xFFFF - invalid
// @DH	colorPointBIntensity	It contains a representation of the relative intensity of the red color point as defined in the Dimming Light Curve in the Ballast Configuration cluster, normalized such that the color point with the highest relative intensity contains the value 0xfe. A value of 0xff SHALL indicate an invalid value.
//				0x00...0xFE - valid
//				0xFF - invalid

// Move to Hue (0x00/0)
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: hue
// Hue
//
// @param: direction
// Direction:
//  0x00 - Shortest distance
//  0x01 - Longest distance
//  0x02 - Up
//  0x03 - Down
//  
// @param: transition_time
// The Transition Time field specifies, in 1/10ths of a second, the time that SHALL be taken to move to the new hue.
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_color_control_move_to_hue(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBYTE hue, ZWBYTE direction, ZWWORD transition_time, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Move Hue (0x01/1)
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: move_mode
// Move mode
//  0x00 - Stop
//  0x01 - Up
//  0x02 - Reserved
//  0x03 - Down
//
// @param: rate
// The Rate field specifies the rate of movement in steps per second. A step is a change in the device’s hue of one unit. If the Rate field has a value of zero, the command has no effect and a default response command is sent in response, with the status code set to INVALID_FIELD
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_color_control_move_hue(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBYTE move_mode, ZWBYTE rate, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Step Hue (0x02/2)
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: step_mode
// Move mode
//  0x00 - Reserved
//  0x01 - Up
//  0x02 - Reserved
//  0x03 - Down
//
// @param: step_size
// The change to be added to (or subtracted from) the current value of the device’s hue.
//
// @param: transition_time
// The Transition Time field specifies, in 1/10ths of a second, the time that SHALL be taken to perform the step. A step is a change in the device’s hue of ‘Step size’ units.
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_color_control_step_hue(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBYTE step_mode, ZWBYTE step_size, ZWBYTE transition_time, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Move to Saturation (0x03/3)
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: saturation
// Desired Saturation
//
// @param: transition_time
// The Transition Time field specifies, in 1/10ths of a second, the time that SHALL be taken to perform the step.
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_color_control_move_to_saturation(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBYTE saturation, ZWWORD transition_time, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Move Saturation (0x04/4)
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: move_mode
// Move Mode
//  0x00 - Stop
//  0x01 - Up
//  0x02 - Reserved
//  0x03 - Down
//
// @param: rate
// The Rate field specifies the rate of movement in steps per second. A step is a change in the device’s hue of one unit. If the Rate field has a value of zero, the command has no effect and a default response command is sent in response, with the status code set to INVALID_FIELD
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_color_control_move_saturation(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBYTE move_mode, ZWBYTE rate, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Step Saturation (0x05/5)
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: step_mode
// Move mode
//  0x00 - Reserved
//  0x01 - Up
//  0x02 - Reserved
//  0x03 - Down
//
// @param: step_size
// The change to be added to (or subtracted from) the current value of the device’s saturation.
//
// @param: transition_time
// The Transition Time field specifies, in 1/10ths of a second, the time that SHALL be taken to perform the step. A step is a change in the device’s hue of ‘Step size’ units.
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_color_control_step_saturation(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBYTE step_mode, ZWBYTE step_size, ZWBYTE transition_time, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Move to Hue and Saturation (0x06/6)
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: hue
// Desired hue
//
// @param: saturation
// Desired Saturation
//
// @param: transition_time
// The Transition Time field specifies, in 1/10ths of a second, the time that SHALL be taken to perform the step.
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_color_control_move_to_hue_and_saturation(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBYTE hue, ZWBYTE saturation, ZWWORD transition_time, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Move to Color (0x07/7)
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: color_x
// Desired Color X
//
// @param: color_y
// Desired Color Y
//
// @param: transition_time
// The Transition Time field specifies, in 1/10ths of a second, the time that SHALL be taken to perform the step.
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_color_control_move_to_color(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWWORD color_x, ZWWORD color_y, ZWWORD transition_time, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Move Color (0x08/8)
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: rate_x
// The RateX field specifies the rate of movement in steps per second. A step is a change in the device's CurrentX attribute of one unit
//
// @param: rate_y
// The RateY field specifies the rate of movement in steps per second. A step is a change in the device's CurrentY attribute of one unit
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_color_control_move_color(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWWORD rate_x, ZWWORD rate_y, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Step Color (0x09/9)
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: step_x
// The StepX fields specify the change to be added to the device's CurrentX attribute attribute.
//
// @param: step_y
// The StepY fields specify the change to be added to the device's CurrentY attribute attribute.
//
// @param: transition_time
// The Transition Time field specifies, in 1/10ths of a second, the time that SHALL be taken to perform the step.
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_color_control_step_color(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWWORD step_x, ZWWORD step_y, ZWWORD transition_time, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Step Color (0x0A/10)
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: color_temp_mireds
// Move from its current color to the color given by the Color Temperature Mireds field.
//
// @param: transition_time
// The Transition Time field specifies, in 1/10ths of a second, the time that SHALL be taken to perform the step.
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_color_control_move_to_color_temperature(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWWORD color_temp_mireds, ZWWORD transition_time, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Enhanced Move to Hue (0x40/64)
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: enhanced_hue
// Enhanced Hue
//
// @param: direction
// Direction:
//  0x00 - Shortest distance
//  0x01 - Longest distance
//  0x02 - Up
//  0x03 - Down
//  
// @param: transition_time
// The Transition Time field specifies, in 1/10ths of a second, the time that SHALL be taken to move to the new hue.
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_color_control_enhance_move_to_hue(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWWORD enhanced_hue, ZWBYTE direction, ZWWORD transition_time, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Enhanced Move Hue (0x41/65)
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: move_mode
// Move mode
//  0x00 - Stop
//  0x01 - Up
//  0x02 - Reserved
//  0x03 - Down
//
// @param: rate
// The Rate field specifies the rate of movement in steps per second. A step is a change in the device’s hue of one unit. If the Rate field has a value of zero, the command has no effect and a default response command is sent in response, with the status code set to INVALID_FIELD
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_color_control_enhance_move_hue(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBYTE move_mode, ZWWORD rate, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Enhanced Step Hue (0x42/66)
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: step_mode
// Move mode
//  0x00 - Reserved
//  0x01 - Up
//  0x02 - Reserved
//  0x03 - Down
//
// @param: step_size
// The change to be added to (or subtracted from) the current value of the device’s hue.
//
// @param: transition_time
// The Transition Time field specifies, in 1/10ths of a second, the time that SHALL be taken to perform the step. A step is a change in the device’s hue of ‘Step size’ units.
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_color_control_enhance_step_hue(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBYTE step_mode, ZWWORD step_size, ZWWORD transition_time, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Enhanced Move to Hue and Saturation (0x43/67)
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: enhanced_hue
// Desired Enhanced Hue
//
// @param: saturation
// Desired Saturation
//
// @param: transition_time
// The Transition Time field specifies, in 1/10ths of a second, the time that SHALL be taken to perform the step.
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_color_control_enhance_move_to_hue_and_saturation(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWWORD enhanced_hue, ZWBYTE saturation, ZWWORD transition_time, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Color Loop Set (0x44/68)
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: update_flags
// The Update Flags field is 8 bits in length and specifies which color loop attributes to update before the color loop is started
//
// @param: action
// The Action field is 8 bits in length and specifies the action to take for the color loop if the Update Action sub-field of the Update Flags field is set to 1
//  0x00 - De-activate the color loop.
//  0x01 - Activate the color loop from the value in the ColorLoopStartEnhancedHue field.
//  0x02 - Activate the color loop from the value of the EnhancedCurrentHue attribute.
//
// @param: direction
// The Direction field is 8 bits in length and specifies the direction for the color loop if the Update Direction field of the Update Flags field is set to 1.
//  0x00 - Decrement the hue in the color loop.
//  0x01 - Increment the hue in the color loop.
//
// @param: time
// The Time field is 16 bits in length and specifies the number of seconds over which to perform a full color loop if the Update Time field of the Update Flags field is set to 1
//
// @param: start_hue
// The Start Hue field is 16 bits in length and specifies the starting hue to use for the color loop if the Update Start Hue field of the Update Flags field is set to 1
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_color_control_color_loop_set(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBYTE update_flags, ZWBYTE action, ZWBYTE direction, ZWWORD time, ZWWORD start_hue, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Stop Move Step (0x47/71)
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_color_control_stop_move_step(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Move Color Temperature (0x4b/75)
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: move_mode
// Move mode:
//  0x00 - Stop
//  0x01 - Up
//  0x02 - Reserved
//  0x03 - Down
//
// @param: rate
// The Rate field is 16-bits in length and specifies the rate of movement in steps per second. A step is a change in the color temperature of a device by one unit. If the Rate field has a value of zero, the command has no effect and a ZCL Default Response command SHALL be sent in response, with the status code set to INVALID_FIELD.
//
// @param: color_temp_min_mireds
// The Color Temperature Minimum Mireds field is 16-bits in length and specifies a lower bound on the ColorTemperatureMireds attribute (an upper bound on the color temperature in kelvins) for the current move operation such that: ColorTempPhysicalMinMireds <= Color Temperature Minimum Mireds field <= ColorTemperatureMireds As such if the move operation takes the ColorTemperatureMireds attribute towards the value of the Color Temperature Minimum Mireds field it SHALL be clipped so that the above invariant is satisfied. If the Color Temperature Minimum Mireds field is set to 0x0000, ColorTempPhysicalMinMireds SHALL be used as the lower bound for the ColorTemperatureMireds attribute.
//
// @param: color_temp_max_mireds
// The Color Temperature Maximum Mireds field is 16-bits in length and specifies an upper bound on the ColorTemperatureMireds attribute (a lower bound on the color temperature in kelvins) for the current move operation such that: ColorTemperatureMireds <= Color Temperature Maximum Mireds field <= ColorTempPhysicalMaxMireds As such if the move operation takes the ColorTemperatureMireds attribute towards the value of the Color Temperature Maximum Mireds field it SHALL be clipped so that the above invariant is satisfied. If the Color Temperature Maximum Mireds field is set to 0x0000, ColorTempPhysicalMaxMireds SHALL be used as the upper bound for the ColorTemperatureMireds attribute.
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_color_control_move_color_temperature(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBYTE move_mode, ZWWORD rate, ZWWORD color_temp_min_mireds, ZWWORD color_temp_max_mireds, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Move Color Temperature (0x4c/76)
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: step_mode
// Move mode:
//  0x00 - Reserved
//  0x01 - Up
//  0x02 - Reserved
//  0x03 - Down
//
// @param: step_size
// The Step Size field is 16-bits in length and specifies the change to be added to (or subtracted from) the current value of the device’s color temperature.
//
// @param: transition_time
// The Transition Time field is 16-bits in length and specifies, in units of 1/10ths of a second, the time that SHALL be taken to perform the step. A step is a change to the device’s color temperature of a magnitude corresponding to the Step Size field
//
// @param: color_temp_min_mireds
// The Color Temperature Minimum Mireds field is 16-bits in length and specifies a lower bound on the ColorTemperatureMireds attribute (an upper bound on the color temperature in kelvins) for the current move operation such that: ColorTempPhysicalMinMireds <= Color Temperature Minimum Mireds field <= ColorTemperatureMireds As such if the move operation takes the ColorTemperatureMireds attribute towards the value of the Color Temperature Minimum Mireds field it SHALL be clipped so that the above invariant is satisfied. If the Color Temperature Minimum Mireds field is set to 0x0000, ColorTempPhysicalMinMireds SHALL be used as the lower bound for the ColorTemperatureMireds attribute.
//
// @param: color_temp_max_mireds
// The Color Temperature Maximum Mireds field is 16-bits in length and specifies an upper bound on the ColorTemperatureMireds attribute (a lower bound on the color temperature in kelvins) for the current move operation such that: ColorTemperatureMireds <= Color Temperature Maximum Mireds field <= ColorTempPhysicalMaxMireds As such if the move operation takes the ColorTemperatureMireds attribute towards the value of the Color Temperature Maximum Mireds field it SHALL be clipped so that the above invariant is satisfied. If the Color Temperature Maximum Mireds field is set to 0x0000, ColorTempPhysicalMaxMireds SHALL be used as the upper bound for the ColorTemperatureMireds attribute.
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_color_control_step_color_temperature(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBYTE step_mode, ZWWORD step_size, ZWWORD transition_time, ZWWORD color_temp_min_mireds, ZWWORD color_temp_max_mireds, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Color Control Get Hue, Saturation
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_color_control_get_hue_saturation(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Color Control Get Color
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_color_control_get_color(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Color Control Get Remaining Time
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_color_control_get_remaining_time(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Set Current Hue Reporting
//
// @param: zbee
// ZBee object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: direction
// TRUE  - receiver will be configured to receiver reports
// FALSE - receiver will be configured to send reports
//
// @param: min_interval
// Minimal Reporting Interval
//
// @param: max_interval
// Maximum Reporting Interval
//
// @param: timeout
// Timeout
//
// @param: threshold
// Threshold
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_color_control_set_current_hue_reporting(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBOOL direction, ZWWORD min_interval, ZWWORD max_interval, ZWWORD timeout, ZWBYTE threshold, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Reset Current Hue Reporting
//
// @param: zbee
// ZBee object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: direction
// TRUE  - receiver will be configured to receiver reports
// FALSE - receiver will be configured to send reports
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_color_control_reset_current_hue_reporting(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBOOL direction, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Set Current Saturation Reporting
//
// @param: zbee
// ZBee object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: direction
// TRUE  - receiver will be configured to receiver reports
// FALSE - receiver will be configured to send reports
//
// @param: min_interval
// Minimal Reporting Interval
//
// @param: max_interval
// Maximum Reporting Interval
//
// @param: timeout
// Timeout
//
// @param: threshold
// Threshold
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_color_control_set_current_saturation_reporting(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBOOL direction, ZWWORD min_interval, ZWWORD max_interval, ZWWORD timeout, ZWBYTE threshold, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Reset Current Saturation Reporting
//
// @param: zbee
// ZBee object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: direction
// TRUE  - receiver will be configured to receiver reports
// FALSE - receiver will be configured to send reports
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_color_control_reset_current_saturation_reporting(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBOOL direction, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Set Current X Reporting
//
// @param: zbee
// ZBee object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: direction
// TRUE  - receiver will be configured to receiver reports
// FALSE - receiver will be configured to send reports
//
// @param: min_interval
// Minimal Reporting Interval
//
// @param: max_interval
// Maximum Reporting Interval
//
// @param: timeout
// Timeout
//
// @param: threshold
// Threshold
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_color_control_set_current_x_reporting(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBOOL direction, ZWWORD min_interval, ZWWORD max_interval, ZWWORD timeout, ZWWORD threshold, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Reset Current X Reporting
//
// @param: zbee
// ZBee object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: direction
// TRUE  - receiver will be configured to receiver reports
// FALSE - receiver will be configured to send reports
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_color_control_reset_current_x_reporting(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBOOL direction, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Set Current Y Reporting
//
// @param: zbee
// ZBee object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: direction
// TRUE  - receiver will be configured to receiver reports
// FALSE - receiver will be configured to send reports
//
// @param: min_interval
// Minimal Reporting Interval
//
// @param: max_interval
// Maximum Reporting Interval
//
// @param: timeout
// Timeout
//
// @param: threshold
// Threshold
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_color_control_set_current_y_reporting(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBOOL direction, ZWWORD min_interval, ZWWORD max_interval, ZWWORD timeout, ZWWORD threshold, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Reset Current Y Reporting
//
// @param: zbee
// ZBee object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: direction
// TRUE  - receiver will be configured to receiver reports
// FALSE - receiver will be configured to send reports
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_color_control_reset_current_y_reporting(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBOOL direction, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Set Color Temperature Mireds Reporting
//
// @param: zbee
// ZBee object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: direction
// TRUE  - receiver will be configured to receiver reports
// FALSE - receiver will be configured to send reports
//
// @param: min_interval
// Minimal Reporting Interval
//
// @param: max_interval
// Maximum Reporting Interval
//
// @param: timeout
// Timeout
//
// @param: threshold
// Threshold
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_color_control_set_color_temperature_mireds_reporting(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBOOL direction, ZWWORD min_interval, ZWWORD max_interval, ZWWORD timeout, ZWWORD threshold, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Reset Color Temperature Mireds Reporting
//
// @param: zbee
// ZBee object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: direction
// TRUE  - receiver will be configured to receiver reports
// FALSE - receiver will be configured to send reports
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_color_control_reset_color_temperature_mireds_reporting(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBOOL direction, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);


// Cluster Class Thermostat (0x0201/513) //
// Profile ZigBee Device Profile (0x0104/260) //
// This cluster provides an interface to the functionality of a thermostat.
//
// @DH	currentLevel		It represents the current level of this device. The meaning of 'level' is device dependent.
// @DH	remainingTime		It represents the time remaining until the current command is complete - it is specified in 1/10ths of a second.
// @DH	onOffTransitionTime	It represents the time taken to move to or from the target level when On of Off commands are received by an On/Off cluster on the same endpoint. It is specified in 1/10ths of a second. The actual time taken SHOULD be as close to OnOffTransitionTime as the device is able. N.B. If the device is not able to move at a variable rate, the OnOffTransitionTime attribute SHOULD not be implemented.
// @DH	onLevel			It determines the value that the CurrentLevel attribute is set to when the OnOff attribute of an On/Off cluster on the same endpoint is set to On. If the OnLevel attribute is not implemented, or is set to 0xff, it has no effect.
// @DH	onTransitionTime	It represents the time taken to move the current level from the minimum level to the maximum level when an On command is received by an On/Off cluster on the same endpoint. It is specified in 10ths of a second. If this command is not implemented, or contains a value of 0xffff, the On/OffTransitionTime will be used instead
// @DH	offTransitionTime	It represents the time taken to move the current level from the maximum level to the minimum level when an Off command is received by an On/Off cluster on the same endpoint. It is specified in 10ths of a second. If this command is not implemented, or contains a value of 0xffff, the On/OffTransitionTime will be used instead.
// @DH	defaultMoveRat		It determines the movement rate, in units per second, when a Move command is received with a Rate parameter of 0xFF
// @DH	localTemperature	It represents the temperature in degrees Celsius, as measured locally or remotely (over the network) as follows: LocalTemperature = 100 x temperature in degrees Celsius. Where -273.15°C <= temperature <= 327.67 ºC, corresponding to a LocalTemperature in the range 0x954d to 0x7fff. The maximum resolution this format allows is 0.01 ºC. A LocalTemperature of 0x8000 indicates that the temperature measurement is invalid.
// @DH	outdoorTemperature	It represents the outdoor temperature in degrees Celsius, as measured locally or remotely (over the network). It is measured as described for LocalTemperature.
// @DH	occupancy		It specifies whether the heated/cooled space is occupied or not, as measured locally or remotely (over the network). If bit 0 = 1, the space is occupied, else it is unoccupied. All other bits are reserved.
// @DH	absMinHeatSetpointLimit	It specifies the absolute minimum level that the heating setpoint MAY be set to. This is a limitation imposed by the manufacturer. The value is calculated as described in the LocalTemperature attribute.
// @DH	absMaxHeatSetpointLimit	It specifies the absolute maximum level that the heating setpoint MAY be set to. This is a limitation imposed by the manufacturer. The value is calculated as described in the LocalTemperature attribute.
// @DH	absMinCoolSetpointLimit	It specifies the absolute minimum level that the cooling setpoint MAY be set to. This is a limitation imposed by the manufacturer. The value is calculated as described in the LocalTemperature attribute.
// @DH	absMaxCoolSetpointLimit	It specifies the absolute maximum level that the cooling setpoint MAY be set to. This is a limitation imposed by the manufacturer. The value is calculated as described in the LocalTemperature attribute.
// @DH	piCoolingDemand		It is 8 bits in length and specifies the level of cooling demanded by the PI (proportional integral) control loop in use by the thermostat (if any), in percent. This value is 0 when the thermostat is in “off” or “heating” mode. This attribute is reported regularly and MAY be used to control a heating device.
// @DH	piHeatingDemand		It is 8 bits in length and specifies the level of heating demanded by the PI loop in percent. This value is 0 when the thermostat is in “off” or “cooling” mode. This attribute is reported regularly and MAY be used to control a cooling device.
// @DH	hvacSystemTypeConfiguration		 It specifies the HVAC system type controlled by the thermostat. If the thermostat uses physical DIP switches to set these parameters, this information SHALL be available read-only from the DIP switches. If these parameters are set via software, there SHALL be read/write access in order to provide remote programming capability. The meanings of individual bits are detailed in Table. Each bit represents a type of system configuration.
//				0th - 1st bits (Cooling System Stage)
//				 00 - Cool Stage 1
//				 01 - Cool Stage 2
//				 10 - Cool Stage 3
//				 11 - Reserved
//				2nd - 3rd bits (Heating System Stage)
//				 00 - Heat Stage 1
//				 01 - Heat Stage 2
//				 10 - Heat Stage 3
//				 11 - Reserved
//				4th bit (Heating System Type)
//				 0 - Conventional
//				 1 - Heat Pump
//				5th bit (Heating Fuel Source)
//				 0 - Electric / B
//				 1 - Gas / O
// @DH	localTemperatureCalibration	It specifies the offset that can be added/subtracted to the actual displayed room temperature, in steps of 0.1°C. The range of this offset is –2.5 °C to +2.5 °C).
// @DH	occupiedCoolingSetpoint		It is 16 bits in length and specifies the cooling mode setpoint when the room is occupied. It SHALL be set to a value in the range defined by the MinCoolSetpointLimit and MaxCoolSetpointLimit attributes. The value is calculated as described in the LocalTemperature attribute. The OccupiedHeatingSetpoint attribute SHALL always be below the value specified in the OccupiedCooling Setpoint by at least SetpointDeadband. If an attempt is made to set it such that this condition is violated, a default response command with the status code INVALID_VALUE (see 2.5.3) SHALL be returned. This SHALL apply to all attempts to set values of attributes which violate similar conditions. If it is unknown if the room is occupied or not, this attribute SHALL be used as the cooling mode setpoint
// @DH	occupiedHeatingSetpoint		It  is 16 bits in length and specifies the heating mode setpoint when the room is occupied. It SHALL be set to a value in the range defined by the MinHeatSetpointLimit and MaxHeatSetpointLimit attributes. The value is calculated as described in the LocalTemperature attribute. The OccupiedCoolingSetpoint attribute SHALL always be above the value specified in the OccupiedHeatingSetpoint by at least SetpointDeadband. If it is unknown if the room is occupied or not, this attribute SHALL be used as the cooling mode setpoint
// @DH	unoccupiedCoolingSetpoint	It  is 16 bits in length and specifies the cooling mode setpoint when the room is unoccupied. It SHALL be set to a value in the range defined by the MinCoolSetpointLimit and MaxCoolSetpointLimit attributes. The value is calculated as described in the LocalTemperature attribute. The UnoccupiedHeatingSetpoint attribute SHALL always be below the value specified in the UnoccupiedCooling Setpoint by at least SetpointDeadband. If it is unknown if the room is occupied or not, this attribute SHALL not be used.
// @DH	unoccupiedHeatingSetpoint	It is 16 bits in length and specifies the heating mode setpoint when the room is unoccupied. It SHALL be set to a value in the range defined by the MinHeatSetpointLimit and MaxHeatSetpointLimit attributes. The value is calculated as described in the LocalTemperature attribute. The UnoccupiedCoolingSetpoint attribute SHALL always be below the value specified in the UnoccupiedHeating Setpoint by at least SetpointDeadband. If it is unknown if the room is occupied or not, this attribute SHALL not be used.
// @DH	minHeatSetpointLimit		It specifies the minimum level that the heating setpoint MAY be set to. The value is calculated as described in the LocalTemperature attribute. It must be greater than or equal to AbsMinHeatSetpointLimit. If this attribute is not present, it SHALL be taken as equal to AbsMinHeatSetpointLimit. This attribute, and the following three attributes, allow the user to define setpoint limits more constrictive than the manufacturer imposed ones. Limiting users (e.g., in a commercial building) to such setpoint limits can help conserve power.
// @DH	maxHeatSetpointLimit		It specifies the maximum level that the heating setpoint MAY be set to. The value is calculated as described in the LocalTemperature attribute. It must be less than or equal to AbsMaxHeatSetpointLimit. If this attribute is not present, it SHALL be taken as equal to AbsMaxHeatSetpointLimit.
// @DH	minCoolSetpointLimit		It specifies the minimum level that the cooling setpoint MAY be set to. The value is calculated as described in the LocalTemperature attribute. It must be greater than or equal to AbsMinCoolSetpointLimit. If this attribute is not present, it SHALL be taken as equal to AbsMinCoolSetpointLimit.
// @DH	maxCoolSetpointLimit		It specifies the maximum level that the cooling setpoint MAY be set to. The value is calculated as described in the LocalTemperature attribute. It must be less than or equal to AbsMaxCoolSetpointLimit. If this attribute is not present, it SHALL be taken as equal to AbsMaxCoolSetpointLimit.
// @DH	minSetpointDeadBand		It specifies the minimum difference between the Heat Setpoint and the Cool SetPoint, in steps of 0.1°C. Its range is 0x0a to 0x19 (1°C to 2.5°C).
// @DH	remoteSensing		It is an 8-bit bitmap that specifies whether the local temperature, outdoor temperature and occupancy are being sensed by internal sensors or remote networked sensors. The meanings of individual bits are detailed in Table
//				0th bit
//				 0 – local temperature sensed internally
//				 1 – local temperature sensed remotely
//				1st bit
//				 0 – outdoor temperature sensed internally
//				 1 – outdoor temperature sensed remotely
//				2nd bit
//				 0 – occupancy sensed internally
//				 1 – occupancy sensed remotely
// @DH	controlSequenceOfOperation	 It specifies the overall operating environment of the thermostat, and thus the possible system modes that the thermostat can operate in. It SHALL be set to one of the non-reserved values in Table. (Note: it is not mandatory to support all values)
//				0x00 (Cooling Only) Heat and Emergency are not possible
//				0x01 (Cooling With Reheat) Heat and Emergency are not possible
//				0x02 (Heating Only) Cool and precooling are not possible
//				0x03 (Heating With Reheat) Cool and precooling are not possible
//				0x04 (Cooling and Heating 4-pipes) All modes are possible
//				0x05 (Cooling and Heating 4-pipes with Reheat) All modes are possible
// @DH	systemMode		It specifies the current operating mode of the thermostat, It SHALL be set to one of the non-reserved values in Table. (Note: It is not mandatory to support all values).
//				0x00 Off
//				0x01 Auto
//				0x03 Cool
//				0x04 Heat
//				0x05 Emergency heating
//				0x06 Precooling
//				0x07 Fan only
//				0x08 Dry
//				0x09 Sleep
// @DH	alarmMask		It specifies whether each of the alarms listed in Table Alarm Codes is enabled. When the bit number corresponding to the alarm code is set to 1, the alarm is enabled, else it is disabled. Bits not corresponding to a code in the table are reserved. When the Alarms cluster is implemented on a device, and one of the alarm conditions included in this table occurs, an alarm notification is generated, with the alarm code field set as listed in the table.
//				0 Initialization failure. The device failed to complete initialization at power-up.
//				1 Hardware failure
//				2 Self-calibration failure
// @DH	thermostatRunningMode	It epresents the running mode of the thermostat. The thermostat running mode can only be Off, Cool or Heat. This attribute is intended to provide additional information when the thermostat’s system mode is in auto mode. The attribute value is maintained to have the same value as the SystemMode attribute.
//				0x00 Off
//				0x03 Cool
//				0x04 Heat
// @DH	startOfWeek		It represents the day of the week that this thermostat considers to be the start of week for weekly set point scheduling. The possible values are given in Table. If the Weekly schedule extension is supported this attribute SHALL be supported. This attribute MAY be able to be used as the base to determine if the device supports weekly scheduling by reading the attribute. Successful response means that the weekly scheduling is supported.
//				0x00 Sunday
//				0x01 Monday
//				0x02 Tuesday
//				0x03 Wednesday
//				0x04 Thursday
//				0x05 Friday
//				0x06 Saturday
// @DH	numberOfWeeklyTransitions	It determines how many weekly schedule transitions the thermostat is capable of handling.	
// @DH	numberOfDailyTransitions	It determines how many daily schedule transitions the thermostat is capable of handling.
// @DH	temperatureSetpointHold		It specifies the temperature hold status on the thermostat, as shown in Table. If hold status is on, the thermostat SHOULD maintain the temperature set point for the current mode until a system mode change. If hold status is off, the thermostat SHOULD follow the setpoint transitions specified by its internal scheduling program. If the thermostat supports setpoint hold for a specific duration, it SHOULD also implement the TemperatureSetpointHoldDuration attribute.
//				0x00 Setpoint Hold Off
//				0x01 Setpoint Hold On
// @DH	temperatureSetpointHoldDuration		It sets the period in minutes for which a setpoint hold is active. Thermostats that support hold for a specified duration SHOULD implement this attribute. The valid range is from 0x0000 – 0x05A0 (1440 minutes within a day). A value of 0xFFFF indicates the field is unused. All other values are reserved.
// @DH	thermostatProgrammingOperationMode	It determines the operational state of the thermostat’s programming. The thermostat SHALL modify its programming operation when this attribute is modified by a client and update this attribute when its programming operation is modified locally by a user. The thermostat MAY support more than one active ThermostatProgrammingOperationMode. For example, the thermostat MAY operate simultaneously in Schedule Programming Mode and Recovery Mode. If a thermostat supports Thermostat Programming Operation Mode attribute, it SHALL support attribute reporting for this attribute. Any locally-initiated changes to the ThermostatProgrammingOperationMode SHALL be updated and reported to all clients configured to receive such reports. The meanings of individual bits are detailed in Table. Each bit represents a type of operation.
//				0th
//				 0 – Simple/setpoint mode. This mode means the thermostat setpoint is altered only by manual up/down changes at the thermostat or remotely, not by internal schedule programming.
//				 1 – Schedule programming mode. This enables or disables any programmed weekly schedule configurations. Note: It does not clear or delete previous weekly schedule programming configurations.
//				1st bit
//				 0 - Auto/recovery mode set to OFF
//				 1 – Auto/recovery mode set to ON
//				2nd bit
//				 0 – Economy/EnergyStar mode set to OFF
//				 1 – Economy/EnergyStar mode set to ON
// @DH	thermostatRunningState		It represents the current relay state of the heat, cool, and fan relays, whose values are shown in Table
//				0th bit - Heat State On
//				1st bit - Cool State On
//				2nd bit - Fan State On
//				3rd bit - Heat 2nd Stage State On
//				4th bit - Cool 2nd Stage State On
//				5th bit - Fan 2nd Stage State On
//				6th bit - Fan 3rd Stage Stage On
// @DH	setpointChangeSource		It  specifies the source of the current active OccupiedCoolingSetpoint or OccupiedHeatingSetpoint (i.e., who or what determined the current setpoint). SetpointChangeSource attribute enables service providers to determine whether changes to setpoints were initiated due to occupant comfort, scheduled programming or some other source (e.g., electric utility or other service provider). Because automation services MAY initiate frequent setpoint changes, this attribute clearly differentiates the source of setpoint changes made at the thermostat.
//				0x00 Manual, user-initiated setpoint change via the thermostat
//				0x01 Schedule/internal programming-initiated setpoint change
//				0x02 Externally-initiated setpoint change (e.g., DRLC cluster command, attribute write)
// @DH	setpointChangeAmount	It specifies the delta between the current active OccupiedCoolingSetpoint or OccupiedHeatingSetpoint and the previous active setpoint. This attribute is meant to accompany the SetpointChangeSource attribute; devices implementing SetpointChangeAmount SHOULD also implement SetpointChangeSource.
//				0x0000 – 0xffff The signed difference in 0.01 degrees Celsius between the previous temperature setpoint and the new temperature setpoint.
// @DH	setpointChangeSourceTimestamp		It specifies the time in UTC at which the SetpointChangeSourceAmount attribute change was recorded.
// @DH	acType		Indicates the type of Mini Split ACType of Mini Split AC is defined depending on how Cooling and Heating condition is achieved by Mini Split AC.
//				0x00 Reserved
//				0x01 Cooling and Fixed Speed
//				0x02 Heat Pump and Fixed Speed
//				0x03 Cooling and Inverter
//				0x04 Heat Pump and Inverter
// @DH	acCapacity		Indicates capacity of Mini Split AC in terms of the format defined by the ACCapacityFormat attribute
// @DH	acRefrigerantType	Indicates type of refrigerant used within the Mini Split AC
//				0x00 Reserved
//				0x01 R22
//				0x02 R410a
//				0x03 R407c
// @DH	acCompressorType	This indicates type of Compressor used within the Mini Split AC.
//				0x00 Reserved
//				0x01 T1, Max working ambient 43 ºC
//				0x02 T2, Max working ambient 35 ºC
//				0x03 T3, Max working ambient 52 ºC
// @DH	acErrorCode		This indicates the type of errors encountered within the Mini Split AC. Error values are reported with four bytes values. Each bit within the four bytes indicates the unique error.
//				0th bit - Compressor Failure or Refrigerant Leakage
//				1st bit - Room Temperature Sensor Failure
//				2nd bit - Outdoor Temperature Sensor Failure
//				3rd bit - Indoor Coil Temperature Sensor Failure
//				4th bit - Fan Failure
// @DH	acLouverPosition	This indicates the position of Louver on the AC. Attribute values are listed in Table
//				0x01 Fully Closed
//				0x02 Fully Open
//				0x03 Quarter Open
//				0x04 Half Open
//				0x05 Three Quarters Open
// @DH	acCoilTemperature	ACCoilTemperature represents the temperature in degrees Celsius, as measured locally or remotely (over the network) as follows: ACCoilTemperature = 100 x temperature in degrees Celsius. Where -273.15°C <= temperature <= 327.67 ºC, corresponding to an ACCoilTemperature in the range 0x954d to 0x7fff. The maximum resolution this format allows is 0.01 ºC. ACCoilTemperature of 0x8000 indicates that the temperature measurement is invalid.
// @DH	acCapacityFormat	This is the format for the ACCapacity attribute
//				0x00 BTUh

// Setpoint Raise/Lower (0x00/0)
//
// @param: zbee
// ZBee object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: mode
// Mode:
//  0x00 - Heat
//  0x01 - Cool
//  0x02 - Both
//
// @param: amount
// Amount of setpoint
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_thermostat_setpoint_raise_lower(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBYTE mode, int amount, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Set weekly schedule (0x01/1)
// @NoJS TODO handling for Uint16
//
// @param: zbee
// ZBee object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: number_of_transitions
// Number Of Transition
//
// @param: day_of_week
// Days of week
//  0th bit - Sunday
//  1st bit - Monday
//  2nd bit - Tuesday
//  3rd bit - Wednesday
//  4th bit - Thursday
//  5th bit - Friday
//  6th bit - Saturday
//  7th bit - Away or Vacation
//
// @param: mode
// Mode:
//  0th bit - Heat Setpoint Field Present in Payload
//  1st bit - Cool Setpoint Field Present in Payload
//
// @param: size_transitions
// Size of transitions array
//
// @param: transitions
// Array of transitions time
//
// @param: heat_size
// Size of heat setpoints array
//
// @param: heat_set_points
// Array of heat setpoints
//
// @param: cool_size
// Size of cool setpoints array
//
// @param: cool_set_points
// Array of cool setpoints
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_thermostat_set_weekly_schedule(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBYTE number_of_transitions, ZWBYTE day_of_week, ZWBYTE mode, ZWBYTE size_transitions, const ZWWORD *transitions, ZWBYTE heat_size, const ZWWORD *heat_set_points, ZWBYTE cool_size, const ZWWORD *cool_set_points, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Get weekly schedule (0x02/2)
//
// @param: zbee
// ZBee object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: days_to_return
// Days of week
//  0th bit - Sunday
//  1st bit - Monday
//  2nd bit - Tuesday
//  3rd bit - Wednesday
//  4th bit - Thursday
//  5th bit - Friday
//  6th bit - Saturday
//  7th bit - Away or Vacation
//
// @param: mode_to_return
// Mode:
//  0th bit - Heat Setpoint Field Present in Payload
//  1st bit - Cool Setpoint Field Present in Payload
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_thermostat_get_weekly_schedule(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBYTE days_to_return, ZWBYTE mode_to_return, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Clear weekly schedule (0x03/3)
//
// @param: zbee
// ZBee object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_thermostat_clear_weekly_schedule(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Get Relay Status Log (0x04/4)
//
// @param: zbee
// ZBee object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_thermostat_get_relay_status_log(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Thermostat Configuration Get
//
// @param: zbee
// ZBee object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_thermostat_configuration_get(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Thermostat Set Local Temperature Reporting
//
// @param: zbee
// ZBee object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: direction
// TRUE  - receiver will be configured to receiver reports
// FALSE - receiver will be configured to send reports
//
// @param: min_interval
// Minimal Reporting Interval
//
// @param: max_interval
// Maximum Reporting Interval
//
// @param: timeout
// Timeout
//
// @param: threshold
// Threshold
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_thermostat_set_local_temperature_reporting(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBOOL direction, ZWWORD min_interval, ZWWORD max_interval, ZWWORD timeout, ZWWORD threshold, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Thermostat Set PI Cooling Demand Reporting
//
// @param: zbee
// ZBee object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: direction
// TRUE  - receiver will be configured to receiver reports
// FALSE - receiver will be configured to send reports
//
// @param: min_interval
// Minimal Reporting Interval
//
// @param: max_interval
// Maximum Reporting Interval
//
// @param: timeout
// Timeout
//
// @param: threshold
// Threshold
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_thermostat_set_pi_cooling_demand_reporting(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBOOL direction, ZWWORD min_interval, ZWWORD max_interval, ZWWORD timeout, ZWBYTE threshold, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Thermostat Set PI Heating Demand Reporting
//
// @param: zbee
// ZBee object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: direction
// TRUE  - receiver will be configured to receiver reports
// FALSE - receiver will be configured to send reports
//
// @param: min_interval
// Minimal Reporting Interval
//
// @param: max_interval
// Maximum Reporting Interval
//
// @param: timeout
// Timeout
//
// @param: threshold
// Threshold
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_thermostat_set_pi_heating_demand_reporting(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBOOL direction, ZWWORD min_interval, ZWWORD max_interval, ZWWORD timeout, ZWBYTE threshold, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Thermostat Set Occupied Cooling Setpoint Reporting
//
// @param: zbee
// ZBee object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: direction
// TRUE  - receiver will be configured to receiver reports
// FALSE - receiver will be configured to send reports
//
// @param: min_interval
// Minimal Reporting Interval
//
// @param: max_interval
// Maximum Reporting Interval
//
// @param: timeout
// Timeout
//
// @param: threshold
// Threshold
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_thermostat_set_occupied_cooling_setpoint_reporting(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBOOL direction, ZWWORD min_interval, ZWWORD max_interval, ZWWORD timeout, ZWWORD threshold, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Thermostat Set Occupied Heating Setpoint Reporting
//
// @param: zbee
// ZBee object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: direction
// TRUE  - receiver will be configured to receiver reports
// FALSE - receiver will be configured to send reports
//
// @param: min_interval
// Minimal Reporting Interval
//
// @param: max_interval
// Maximum Reporting Interval
//
// @param: timeout
// Timeout
//
// @param: threshold
// Threshold
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_thermostat_set_occupied_heating_setpoint_reporting(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBOOL direction, ZWWORD min_interval, ZWWORD max_interval, ZWWORD timeout, ZWWORD threshold, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Thermostat Set System Mode Reporting
//
// @param: zbee
// ZBee object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: direction
// TRUE  - receiver will be configured to receiver reports
// FALSE - receiver will be configured to send reports
//
// @param: min_interval
// Minimal Reporting Interval
//
// @param: max_interval
// Maximum Reporting Interval
//
// @param: timeout
// Timeout
//
// @param: threshold
// Threshold
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_thermostat_set_system_mode_reporting(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBOOL direction, ZWWORD min_interval, ZWWORD max_interval, ZWWORD timeout, ZWBYTE threshold, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Thermostat State Get
// This command is used to get the current state of the thermostat (local temperature, heating setpoint, cooling setpoint, system mode, running state)
//
// @param: zbee
// ZBee object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_thermostat_state_get(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Thermostat Set System Mode
//
// @param: zbee
// ZBee object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: mode
// Desired system mode between 0x00 (SYSTEM_MODE_OFF) and 0x09 (SYSTEM_MODE_SLEEP)
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_thermostat_set_mode(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBYTE mode, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Thermostat Set Occupied Heating Setpoint
//
// @param: zbee
// ZBee object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: value
// Desired heating setpoint temperature in float between [-327.67, 327.67]
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_thermostat_set_heat_setpoint(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, float value, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Thermostat Set Occupied Cooling Setpoint
//
// @param: zbee
// ZBee object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: value
// Desired cooling setpoint temperature in float between [-327.67, 327.67]
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_thermostat_set_cool_setpoint(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, float value, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);


// Cluster Class FanControl (0x0202/514) //
// Profile ZigBee Device Profile (0x0104/260) //
// This cluster specifies an interface to control the speed of a fan as part of a heating / cooling system
//
// @DH	fanMode		        The FanMode attribute is an 8-bit value that specifies the current speed of the fan.
//				0x00 Off
//				0x01 Low
//				0x02 Medium
//				0x03 High
//				0x04 On
//				0x05 Auto (the fan speed is self-regulated)
//				0x06 Smart (when the heated/cooled space is occupied, the fan is always on
// @DH	fanModeSequence		The FanModeSequence attribute is an 8-bit value that specifies the possible fan speeds that the thermostat can set.
//				0x00 Low/Med/High
//				0x01 Low/High
//				0x02 Low/Med/High/Auto
//				0x03 Low/High/Auto
//				0x04 On/Auto

// Set Fan Mode
//
// @param: zbee
// ZBee object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: fan_mode
// Fan Mode
//				0x00 Off
//				0x01 Low
//				0x02 Medium
//				0x03 High
//				0x04 On
//				0x05 Auto (the fan speed is self-regulated)
//				0x06 Smart (when the heated/cooled space is occupied, the fan is always on
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_fan_control_set_fan_mode(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBYTE fan_mode, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Set Fan Mode Sequence
//
// @param: zbee
// ZBee object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: fan_mode_sequence
// Fan Mode Sequence
//				0x00 Low/Med/High
//				0x01 Low/High
//				0x02 Low/Med/High/Auto
//				0x03 Low/High/Auto
//				0x04 On/Auto
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_fan_control_set_fan_mode_sequence(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBYTE fan_mode_sequence, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);


// Cluster Class Thermostat User Interface (0x0204/516) //
// Profile ZigBee Device Profile (0x0104/260) //
// This cluster provides an interface to allow configuration of the user interface for a thermostat, or a thermostat controller device, that supports a keypad and LCD screen
//
// @DH	temperatureDisplayMode			The TemperatureDisplayMode attribute specifies the units of the temperature displayed on the thermostat screen. This attribute SHALL be set to one of the non-reserved values in Table
//			0x00 Temperature in oC
//			0x01 Temperature in oF
// @DH	keypadLockout			The KeypadLockout attribute specifies the level of functionality that is available to the user via the keypad. This attribute SHALL be set to one of the non-reserved values Table. The interpretation of the various levels is device-dependent.
//			0x00 No lockout
//			0x01 Level 1 lockout
//			0x02 Level 2 lockout
//			0x03 Level 3 lockout
//			0x04 Level 4 lockout
//			0x05 Level 5 lockout (least functionality available to the user)
// @DH	scheduleProgrammingVisibility			The ScheduleProgrammingVisibility attribute is used to hide the weekly schedule programming functionality or menu on a thermostat from a user to prevent local user programming of the weekly schedule. The schedule programming MAY still be performed via a remote interface, and the thermostat MAY operate in schedule programming mode. This command is designed to prevent local tampering with or disabling of schedules that MAY have been programmed by users or service providers via a more capable remote interface. The programming schedule SHALL continue to run even though it is not visible to the user locally at the thermostat. It SHALL be set to one of the non-reserved values in Table
//			0x00 Local schedule programming functionality is enabled at the thermostat
//			0x01 Local schedule programming functionality is disabled at the thermostat

// Set Temperature Display Mode
//
// @param: zbee
// ZBee object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: mode
// Mode of displaying temperature
//			0x00 Temperature in Celsius
//			0x01 Temperature in Fahrenheit
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_thermostat_user_interface_set_temperature_display_mode(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWWORD mode, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Thermostat User Information Set Keypad Lockout
//
// @param: zbee
// ZBee object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: mode
// Mode of keypad lock
//			0x00 No lockout
//			0x01 Level 1 lockout
//			0x02 Level 2 lockout
//			0x03 Level 3 lockout
//			0x04 Level 4 lockout
//			0x05 Level 5 lockout (least functionality available to the user)
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_thermostat_user_interface_set_keypad_lockout(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBYTE mode, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Thermostat User Information Set Schedule Programming Visibility
//
// @param: zbee
// ZBee object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: mode
// Mode of keypad lock
//			0x00 No lockout
//			0x01 Level 1 lockout
//			0x02 Level 2 lockout
//			0x03 Level 3 lockout
//			0x04 Level 4 lockout
//			0x05 Level 5 lockout (least functionality available to the user)
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_thermostat_user_interface_set_schedule_programming_visibility(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBOOL mode, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);


// Cluster Class Level Control (0x0008/8) //
// Profile ZigBee Device Profile (0x0104/260) //
// This cluster provides an interface for controlling a characteristic of a device that can be set to a level, for example the brightness of a light, the degree of closure of a door, or the power output of a heater.
//
// @DH	currentLevel		It represents the current level of this device. The meaning of 'level' is device dependent.
// @DH	remainingTime		It represents the time remaining until the current command is complete - it is specified in 1/10ths of a second.
// @DH	onOffTransitionTime	It represents the time taken to move to or from the target level when On of Off commands are received by an On/Off cluster on the same endpoint. It is specified in 1/10ths of a second. The actual time taken SHOULD be as close to OnOffTransitionTime as the device is able. N.B. If the device is not able to move at a variable rate, the OnOffTransitionTime attribute SHOULD not be implemented.
// @DH	onLevel			It determines the value that the CurrentLevel attribute is set to when the OnOff attribute of an On/Off cluster on the same endpoint is set to On. If the OnLevel attribute is not implemented, or is set to 0xff, it has no effect.
// @DH	onTransitionTime	It represents the time taken to move the current level from the minimum level to the maximum level when an On command is received by an On/Off cluster on the same endpoint. It is specified in 10ths of a second. If this command is not implemented, or contains a value of 0xffff, the On/OffTransitionTime will be used instead
// @DH	offTransitionTime	It represents the time taken to move the current level from the maximum level to the minimum level when an Off command is received by an On/Off cluster on the same endpoint. It is specified in 10ths of a second. If this command is not implemented, or contains a value of 0xffff, the On/OffTransitionTime will be used instead.
// @DH	defaultMoveRat		It determines the movement rate, in units per second, when a Move command is received with a Rate parameter of 0xFF

// Move to Level (0x00/0)
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: level
// Device dependent level
//
// @param: transition_time
// @default: 0
// Transition time
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_level_control_move_to_level(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBYTE level, ZWWORD transition_time, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Move (0x01/1)
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: move_mode
// Move mode:
//  0x00 - Up
//  0x01 - Down
//
// @param: rate
// @default: 0xFF
// The Rate field specifies the rate of movement in units per second:
//  0x00...0xFE - used value  
//  0xFF        - used value from DefaultMoveRate
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_level_control_move(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBYTE move_mode, ZWBYTE rate, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Step (0x02/2)
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: step_mode
// Move mode:
//  0x00 - Up
//  0x01 - Down
//
// @param: step_size
// Step size
//
// @param: transition_time
// @default: 0
// The Transition time field specifies the time that SHALL be taken to perform the step, in tenths of a second.
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_level_control_step(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBYTE step_mode, ZWBYTE step_size, ZWWORD transition_time, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Stop (0x03/3)
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_level_control_stop(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Move to Level (with On/Off) (0x04/4)
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: level
// Device dependent level
//
// @param: transition_time
// @default: 0
// Transition time
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_level_control_move_to_level_on_off(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBYTE level, ZWWORD transition_time, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Move (with On/Off) (0x05/5)
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: move_mode
// Move mode:
//  0x00 - Up
//  0x01 - Down
//
// @param: rate
// @default: 0xFF
// The Rate field specifies the rate of movement in units per second:
//  0x00...0xFE - used value  
//  0xFF        - used value from DefaultMoveRate
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_level_control_move_on_off(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBYTE move_mode, ZWBYTE rate, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Step (with On/Off) (0x06/6)
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: step_mode
// Move mode:
//  0x00 - Up
//  0x01 - Down
//
// @param: step_size
// Step size
//
// @param: transition_time
// @default: 0
// The Transition time field specifies the time that SHALL be taken to perform the step, in tenths of a second.
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_level_control_step_on_off(ZBee zbee,ZBNWK node_id, ZWBYTE endpoint_id, ZWBYTE step_mode, ZWBYTE step_size, ZWWORD transition_time, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Stop (with On/Off) (0x07/7)
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_level_control_stop_on_off(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Level Control Get
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_level_control_get(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Level Control Configuration Get
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_level_control_configuration_get(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Set Current Level Reporting
//
// @param: zbee
// ZBee object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: direction
// TRUE  - receiver will be configured to receiver reports
// FALSE - receiver will be configured to send reports
//
// @param: min_interval
// Minimal Reporting Interval
//
// @param: max_interval
// Maximum Reporting Interval
//
// @param: timeout
// Timeout
//
// @param: threshold
// Threshold
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_level_control_set_current_level_reporting(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBOOL direction, ZWWORD min_interval, ZWWORD max_interval, ZWWORD timeout, ZWBYTE threshold, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Reset Current Level Reporting
//
// @param: zbee
// ZBee object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: direction
// TRUE  - receiver will be configured to receiver reports
// FALSE - receiver will be configured to send reports
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_level_control_reset_current_level_reporting(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBOOL direction, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);


// Cluster Class Door Lock (0x0101/257) //
// Profile ZigBee Device Profile (0x0104/260) //
// The door lock cluster provides an interface to a generic way to secure a door. The physical object that provides the locking functionality is abstracted from the cluster. The cluster has a small list of mandatory attributes and functions and a list of optional features.
//
// @DH	lockState	It describes lock states.
//				0x00 - Not fully locked
//				0x01 - Locked
//				0x02 - Unlocked
//				0xFF - Undefined
// @DH	lockType	It describes lock types.
//				0x00 - Dead bolt
//				0x01 - Magnetic
//				0x02 - Other
//				0x03 - Mortise
//				0x04 - Rim
//				0x05 - Latch Bolt
//				0x06 - Cylindrical Lock
//				0x07 - Tubular Lock
//				0x08 - Interconnected Lock
//				0x09 - Dead Latch
//				0x0A - Door Furniture
// @DH	actuatorEnabled	It describes if actuator is enable or not
//				0 - Disabled
//				1 - Enabled
// @DH	doorState	It describes possible door states.
//				0x00 Open
//				0x01 Closed
//				0x02 Error (jammed)
//				0x03 Error (forced open)
//				0x04 Error (unspecified)
//				0xFF Undefined
// @DH	doorOpenEvents					This attribute holds the number of door open events that have occurred since it was last zeroed
// @DH	doorClosedEvents				This attribute holds the number of door closed events that have occurred since it was last zeroed.
// @DH	openPeriod					This attribute holds the number of minutes the door has been open since the last time it transitioned from closed to open.
// @DH	numberOfLogRecordsSupported			The number of available log records
// @DH	numberOfTotalUsersSupported			Number of total users supported by the lock. This value is equal to the higher one of [# of PIN Users Supported] and [# of RFID Users Supported]
// @DH	numberOfPINUsersSupported			The number of PIN users supported.
// @DH	numberOfRFIDUsersSupported			The number of RFID users supported.
// @DH	numberOfWeekDaySchedulesSupportedPerUser	The number of configurable week day schedule supported per user.
// @DH	numberOfYearDaySchedulesSupportedPerUser	The number of configurable year day schedule supported per user
// @DH	numberOfHolidaySchedulesSupported		The number of holiday schedules supported for the entire door lock device.
// @DH	maxPINCodeLength				An 8 bit value indicates the maximum length in bytes of a PIN Code on this device. The default is set to 8 since most lock manufacturers currently allow PIN Codes of 8 bytes or less
// @DH	minPINCodeLength				An 8 bit value indicates the minimum length in bytes of a PIN Code on this device. The default is set to 4 since most lock manufacturers do not support PIN Codes that are shorter than 4 bytes.
// @DH	maxRFIDCodeLength				An 8 bit value indicates the maximum length in bytes of a RFID Code on this device. The value depends on the RFID code range specified by the manufacturer, if media anti-collision identifiers (UID) are used as RFID code, a value of 20 (equals 10 Byte ISO 14443A UID) is recommended
// @DH	minRFIDCodeLength				An 8 bit value indicates the minimum length in bytes of a RFID Code on this device. The value depends on the RFID code range specified by the manufacturer, if media anti-collision identifiers (UID) are used as RFID code, a value of 8 (equals 4 Byte ISO 14443A UID) is recommended.
// @DH	enableLogging					Enable/disable event logging. When event logging is enabled, all event messages are stored on the lock for retrieval. Logging events can be but not limited to Tamper Alarm, Lock, Unlock, Autolock, User Code Added, User Code Deleted, Schedule Added, and Schedule Deleted. For a full detail of all the possible alarms and events, please refer to the full list in the Alarm and Event Masks Attribute Set.
// @DH	language					Modifies the language for the on-screen or audible user interface using three bytes from ISO-639-1. It consists of one byte of length and two bytes for the language code. For example if the language is set to English, the value would be "02 65 6E" for the language code "en".
// @DH	ledSettings					The settings for the LED support three different modes
//								0x00 - Never use LED for signalization
//								0x01 - Use LED signalization except for access allowed events
//								0x02 - Use LED signalization for all events
// @DH	autoRelockTime				The number of seconds to wait after unlocking a lock before it automatically locks again. 0=disabled. If set, unlock operations from any source will be timed. For one time unlock with timeout use the specific command.                
// @DH	soundVolume				The sound volume on a door lock has three possible setting
//				0x00 Silent Mode
//				0x01 Low Volume
//				0x02 High Volume
// @DH	operatingMode				It describes operating modes
//				0x00 - Normal            KeyPad - E   RF - E    E   - RFID
//				0x01 - Vacation          KeyPad - D   RF - E    E   - RFID
//				0x02 - Privacy           KeyPad - D   RF - D    D   - RFID
//				0x03 - No RF Lock/Unlock KeyPad - E   RF - D    E   - RFID
//				0x04 - Passage           KeyPad - N/A RF - N/A  N/A - RFID               
// @DH	supportedOperatingModes			This bitmap contains all operating bits of the Operating Mode Attribute supported by the lock.
//				0 - Normal Mode Supported
//				1 - Vacation Mode Supported
//				2 - Privacy Mode Supported
//				3 - No RF Lock or Unlock Mode Supported
//				4 - Passage Mode Supported
// @DH	defaultConfigurationRegister		This attribute represents the default configurations as they are physically set on the device          
//				0th bit
//				 1 - Enable Local Programming Attribute default value is 0 (disabled)
//				 0 - Enable Local Programming Attribute default value is 1 (enabled)
//				1st bit
//				 1 - Keypad Interface default access is disabled
//				 0 - Keypad Interface default access is enabled
//				2nd bit
//				 1 - RF Interface default access is disabled
//				 0 - RF Interface default access is enabled
//				5th bit
//				 1 -  Sound Volume attribute default value is 0 (Slight Mode)
//				 0 -  Sound Volume attribute default value is equal to something other than 0x00
//				6th bit
//				 1 -  Auto Relock Time attribute default value = 0x00
//				 0 -  Auto Relock Time attribute default value is equal to something other than 0x00
//				7th bit
//				 1 - Led Settings attribute default value = 0x00
//				 0 - Led Settings attribute default value is equal to something other than 0x00
// @DH	enableLocalProgramming			Enable/disable local programming on the door lock. The local programming features includes but not limited to adding new user codes, deleting existing user codes, add new schedule, deleting existing schedule on the local door lock interfaces. If this value is set to 0x01 or TRUE then local programming is enabled on the door lock. If it is set to 0x00 or FALSE then local programming is disabled on the door lock. Local programming is enabled by default.
// @DH	enableOneTouchLocking			Enable/disable the ability to lock the door lock with a single touch on the door lock
// @DH	enableInsideStatusLED			Enable/disable an inside LED that allows the user to see at a glance if the door is locked.
// @DH	enablePrivacyModeButton			Enable/disable a button inside the door that is used to put the lock into privacy mode. When the lock is in privacy mode it cannot be manipulated from the outside
// @DH	wrongCodeEntryLimit			The number of incorrect codes or RFID presentment attempts a user is allowed to enter before the door will enter a lockout state. The lockout state will be for the duration of UserCodeTemporaryDisableTime
// @DH	userCodeTemporaryDisableTime		The number of seconds that the lock shuts down following wrong code entry. 1-255 seconds. Device can shutdown to lock user out for specified amount of time. (Makes it difficult to try and guess a PIN for the device.)
// @DH	sendPINOverTheAir			Boolean set to True if it is ok for the door lock server to send PINs over the air. This attribute determines the behavior of the server’s TX operation. If it is false, then it is not ok for the device to send PIN in any messages over the air. The PIN field within any door lock cluster message SHALL keep the first octet unchanged and masks the actual code by replacing with 0xFF. For example (PIN "1234" ): If the attribute value is True, 0x04 0x31 0x32 0x33 0x34 SHALL be used in the PIN field in any door lock cluster message payload. If the attribute value is False, 0x04 0xFF 0xFF 0xFF 0xFF SHALL be used
// @DH	requirePINforRFOperation		Boolean set to True if the door lock server requires that an optional PINs be included in the payload of RF lock operation events like Lock, Unlock and Toggle in order to function.
// @DH	zigBeeSecurityLevel			The Security Level attribute allows the door lock manufacturer to indicate what level of security the door lock requires.
//				0 - Network Security (default)
//				1 - APS Security
// @DH	alarmMask	The alarm mask is used to turn on/off alarms for particular functions
//				0th bit - 0x00 (Deadbolt Jammed)
//				1st bit - 0x01 (Lock Reset to Factory Defaults)
//				2nd bit - 0x02 (Reserved)
//				3rd bit - 0x03 (RF Module Power Cycled)
//				4th bit - 0x04 (Tamper Alarm – wrong code entry limit)
//				5th bit - 0x05 (Tamper Alarm - front escutcheon removed from main)
//				6th bit - 0x06 (Forced Door Open under Door Locked Condition)
// @DH	keypadOperationEventMask		Event mask used to turn on and off the transmission of keypad operation events. This mask DOES NOT apply to the storing of events in the report table.
//              0th bit - 0x00 (Event Source) 0x00 (Operation Event Code) Unknown or manufacturer-specific keypad operation event
//              1st bit - 0x00 (Event Source) 0x01 (Operation Event Code) Lock, source: keypad
//              2nd bit - 0x00 (Event Source) 0x02 (Operation Event Code) Unlock, source: keypad
//              3rd bit - 0x00 (Event Source) 0x03 (Operation Event Code) Lock, source: keypad, error: invalid PIN
//              4th bit - 0x00 (Event Source) 0x04 (Operation Event Code) Lock, source: keypad, error: invalid schedule
//              5th bit - 0x00 (Event Source) 0x05 (Operation Event Code) Unlock, source: keypad, error: invalid code
//              6th bit - 0x00 (Event Source) 0x06 (Operation Event Code) Unlock, source: keypad, error: invalid schedule
//              7th bit - 0x00 (Event Source) 0x0F (Operation Event Code) Non-Access User operation event, source keypad.
// @DH	rfOperationEventMask			Event mask used to turn on and off the transmission of RF operation events. This mask DOES NOT apply to the storing of events in the report table.
//              0th bit - 0x01 (Event Source) 0x00 (Operation Event Code)  Unknown or manufacturer-specific RF operation event
//              1st bit - 0x01 (Event Source) 0x01 (Operation Event Code)  Lock, source: RF
//              2nd bit - 0x01 (Event Source) 0x02 (Operation Event Code)  Unlock, source: RF
//              3rd bit - 0x01 (Event Source) 0x03 (Operation Event Code)  Lock, source: RF, error: invalid code
//              4th bit - 0x01 (Event Source) 0x04 (Operation Event Code)  Lock, source: RF, error: invalid schedule
//              5th bit - 0x01 (Event Source) 0x05 (Operation Event Code)  Unlock, source: RF, error: invalid code
//              6th bit - 0x01 (Event Source) 0x06 (Operation Event Code)  Unlock, source: RF, error: invalid schedule
// @DH	manualOperationEventMask		Event mask used to turn on and off manual operation events. This mask DOES NOT apply to the storing of events in the report table.
//              0th bit - 0x02 (Event Source) 0x00 (Operation Event Code)  Unknown or manufacturer-specific manual operation event
//              1st bit - 0x02 (Event Source) 0x01 (Operation Event Code)  Thumbturn Lock
//              2nd bit - 0x02 (Event Source) 0x02 (Operation Event Code)  Thumbturn Unlock
//              3rd bit - 0x02 (Event Source) 0x07 (Operation Event Code)  One touch lock
//              4th bit - 0x02 (Event Source) 0x08 (Operation Event Code)  Key Lock
//              5th bit - 0x02 (Event Source) 0x09 (Operation Event Code)  Key Unlock
//              6th bit - 0x02 (Event Source) 0x0A (Operation Event Code)  Auto lock
//              7th bit - 0x02 (Event Source) 0x0B (Operation Event Code)  Schedule Lock
//              8th bit - 0x02 (Event Source) 0x0C (Operation Event Code)  Schedule Unlock
//              9th bit - 0x02 (Event Source) 0x0D (Operation Event Code)  Manual Lock (Key or Thumbturn)
//              1st bit - 0x02 (Event Source) 0x0E (Operation Event Code) ) Manual Unlock (Key or Thumbturn)
// @DH	rfidOperandonEventMask			Event mask used to turn on and off RFID operation events. This mask DOES NOT apply to the storing of events in the report table
//              0rd bit - 0x03 (Event Source) 0x00 (Operation Event Code)  Unknown or manufacturer-specific keypad operation even
//              1th bit - 0x03 (Event Source) 0x01 (Operation Event Code)  Lock, source: RFID
//              2th bit - 0x03 (Event Source) 0x02 (Operation Event Code)  Unlock, source: RFID
//              3th bit - 0x03 (Event Source) 0x03 (Operation Event Code)  Lock, source: RFID, error: invalid RFID ID
//              4th bit - 0x03 (Event Source) 0x04 (Operation Event Code)  Lock, source: RFID, error: invalid schedule
//              5th bit - 0x03 (Event Source) 0x05 (Operation Event Code)  Unlock, source: RFID, error: invalid RFID ID
//              6th bit - 0x03 (Event Source) 0x06 (Operation Event Code)  Unlock, source: RFID, error: invalid schedule
// @DH	keypadProgrammingEventMask		Event mask used to turn on and off keypad programming events. This mask DOES NOT apply to the storing of events in the report table.
//              0th bit - 0x00 (Event Source) 0x00 (Operation Event Code)  Unknown or manufacturer-specific keypad programming event
//              1th bit - 0x00 (Event Source) 0x01 (Operation Event Code)  Master code changed, source: keypad User ID: master user ID. PIN: default or master code if codes can be sent over the air per attribute. User type: default User Status: default
//              2th bit - 0x00 (Event Source) 0x02 (Operation Event Code)  PIN added, source: keypad User ID: user ID that was added. PIN: code that was added (if codes can be sent over the air per attribute.) User type: default or type added. User Status: default or status added.
//              3th bit - 0x00 (Event Source) 0x03 (Operation Event Code)  PIN deleted, source: keypad User ID: user ID that was deleted. PIN: code that was deleted (if codes can be sent over the air per attribute.) User type: default or type deleted. User Status: default or status deleted.
//              4th bit - 0x00 (Event Source) 0x04 (Operation Event Code)  PIN changed Source: keypad User ID: user ID that was changed PIN: code that was changed (if codes can be sent over the air per attribute.) User type: default or type changed. User Status: default or status changed.
// @DH	rfProgrammingEventMask			Event mask used to turn on and off RF programming events. This mask DOES NOT apply to the storing of events in the report table.
//              0th bit - 0x01 (Event Source) 0x00 (Operation Event Code)  Unknown or manufacturer-specific RF programming event.
//              2th bit - 0x01 (Event Source) 0x02 (Operation Event Code)  PIN added, source RF Same as keypad source above
//              3th bit - 0x01 (Event Source) 0x03 (Operation Event Code)  PIN deleted, source RF Same as keypad source above.
//              4th bit - 0x01 (Event Source) 0x04 (Operation Event Code)  PIN changed Source RF Same as keypad source above
//              5th bit - 0x01 (Event Source) 0x05 (Operation Event Code)  RFID code added, Source RF
//              6th bit - 0x01 (Event Source) 0x06 (Operation Event Code)  RFID code deleted, Source RF
// @DH	rfidProgrammingEventMask		Event mask used to turn on and off RFID programming events. This mask DOES NOT apply to the storing of events in the report table.
//              0th bit - 0x03 (Event Source) 0x00 (Operation Event Code)  Unknown or manufacturer-specific keypad programming event
//              5th bit - 0x03 (Event Source) 0x05 (Operation Event Code)  ID Added, Source: RFID User ID: user ID that was added. ID: ID that was added (if codes can be sent over the air per at- tribute.) User Type: default or type added. User Status: default or status added.
//              6th bit - 0x03 (Event Source) 0x06 (Operation Event Code)  ID Deleted, Source: RFID User ID: user ID that was deleted. ID: ID that was deleted (if codes can be sent over the air per attribute.) User Type: default or type deleted. User Status: default or status deleted.

// Lock Door (0x00/0)
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: size
// Pin Code Size
//
// @param: code
// Pin Code
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_door_lock_lock_door(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBYTE size, const ZWBYTE *code, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Unlock Door (0x01/1)
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: size
// Pin Code Size
//
// @param: code
// Pin Code
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_door_lock_unlock_door(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBYTE size, const ZWBYTE *code, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Toggle (0x02/2)
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: size
// Pin Code Size
//
// @param: code
// Pin Code
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_door_lock_toggle(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBYTE size, const ZWBYTE *code, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Unlock with Timeout (0x03/3)
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: timeout
// Relock Timeout
//
// @param: size
// Pin Code Size
//
// @param: code
// Pin Code
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_door_lock_unlock_with_timeout(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWWORD timeout, ZWBYTE size, const ZWBYTE *code, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Get Log Record (0x04/4)
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: log_index
// Log Entry Index
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_door_lock_get_log_record(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWWORD log_index, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Set PIN Code (0x05/5)
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: user_id
// User Identifier
//
// @param: user_status
// User Status
//
// @param: user_type
// User Type
//
// @param: size
// Pin Code Size
//
// @param: pin
// Pin Code
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_door_lock_set_pin_code(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWWORD user_id, ZWBYTE user_status, ZWBYTE user_type, ZWBYTE size, const ZWBYTE *pin, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Get PIN Code (0x06/6)
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: user_id
// User Identifier
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_door_lock_get_pin_code(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWWORD user_id, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Clear PIN Code (0x07/7)
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: user_id
// User Identifier
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_door_lock_clear_pin_code(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWWORD user_id, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Clear All PIN Codes (0x08/8)
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_door_lock_clear_all_pin_codes(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Set User Status (0x09/9)
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: user_id
// User Identifier
//
// @param: user_type
// User Type
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_door_lock_set_user_status(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWWORD user_id, ZWBYTE user_type, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Get User Status (0x0A/10)
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: user_id
// User Identifier
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_door_lock_get_user_status(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWWORD user_id, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Set Weekday Schedule (0x0B/11)
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: id
// Schedule Identifier
//
// @param: user_id
// User Type
//
// @param: days
// Days Mask
//
// @param: start_hour
// Start Hour
//
// @param: start_minute
// Start Minute
//
// @param: end_hour
// End Hour
//
// @param: end_minute
// End Minute
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_door_lock_set_weekday_schedule(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBYTE id, ZWWORD user_id, ZWBYTE days, ZWBYTE start_hour, ZWBYTE start_minute, ZWBYTE end_hour, ZWBYTE end_minute, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Get Weekday Schedule (0x0C/12)
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: id
// Schedule Identifier
//
// @param: user_id
// User Identifier
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_door_lock_get_weekday_schedule(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBYTE id, ZWWORD user_id, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Clear Weekday Schedule (0x0D/13)
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: id
// Schedule Identifier
//
// @param: user_id
// User Identifier
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_door_lock_clear_weekday_schedule(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBYTE id, ZWWORD user_id, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Set Year Day Schedule (0x0E/14)
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: id
// Schedule Identifier
//
// @param: user_id
// User Type
//
// @param: start_time
// Start Time
//
// @param: end_time
// Start Minute
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_door_lock_set_year_day_schedule(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBYTE id, ZWWORD user_id, unsigned int start_time, unsigned int end_time, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Get Year Day Schedule (0x0F/15)
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: id
// Schedule Identifier
//
// @param: user_id
// User Identifier
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_door_lock_get_year_day_schedule(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBYTE id, ZWWORD user_id, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Clear Year Day Schedule (0x10/16)
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: id
// Schedule Identifier
//
// @param: user_id
// User Identifier
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_door_lock_clear_year_day_schedule(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBYTE id, ZWWORD user_id, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Set Holiday Schedule (0x11/17)
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: id
// Schedule Identifier
//
// @param: start_time
// Start Time
//
// @param: end_time
// Start Minute
//
// @param: mode
// Operating Mode During Holiday
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_door_lock_set_holiday_schedule(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBYTE id, unsigned int start_time, unsigned int end_time, ZWBYTE mode, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Get Holiday Schedule (0x12/18)
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: id
// Schedule Identifier
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_door_lock_get_holiday_schedule(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBYTE id, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Clear Holiday Schedule (0x13/19)
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: id
// Schedule Identifier
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_door_lock_clear_holiday_schedule(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBYTE id, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Set User Type (0x14/20)
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: user_id
// User Identifier
//
// @param: user_type
// User Type
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_door_lock_set_user_type(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWWORD user_id, ZWBYTE user_type, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Get User Status (0x15/21)
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: user_id
// User Identifier
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_door_lock_get_user_type(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWWORD user_id, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Set RFID Code (0x16/22)
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: user_id
// User Identifier
//
// @param: user_status
// User Status
//
// @param: user_type
// User Type
//
// @param: size
// RFID Code Size
//
// @param: code
// RFID Code
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_door_lock_set_rfid_code(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWWORD user_id, ZWBYTE user_status, ZWBYTE user_type, ZWBYTE size, const ZWBYTE *code, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Get RFID Code (0x17/23)
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: user_id
// User Identifier
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_door_lock_get_rfid_code(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWWORD user_id, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Clear RFID Code (0x18/24)
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: user_id
// User Identifier
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_door_lock_clear_rfid_code(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWWORD user_id, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Clear All RFID Code (0x19/25)
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_door_lock_clear_all_rfid_codes(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Color Control Get State
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_door_lock_get_state(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Color Control Configuration Get
//
// @param: zbee 
// ZBee object instance
//
// @param: node_id 
// Destination Node Id
//
// @param: endpoint_id 
// Destination EndPoint Id
//
// @param: successCallback 
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback 
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg 
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_door_lock_configuration_get(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);


// Cluster Class IlluminanceMeasurement (0x0400/1024) //
// Profile Zigbee Device Profile (0x0104/260) //
// The cluster provides an interface to temperature measurement functionality, including configuration and provision of notifications of temperature measurements
//
// @DH	measuredValue		It represents the temperature in degrees Celsius as follows: MeasuredValue = 100 x temperature in degrees Celsius. Where -273.15°C <= temperature <= 327.67 ºC, corresponding to a MeasuredValue in the range 0x954d to 0x7fff. The maximum resolution this format allows is 0.01 ºC. A MeasuredValue of 0x8000 indicates that the temperature measurement is invalid. MeasuredValue is updated continuously as new measurements are made	
// @DH	minMeasuredValue	It indicates the minimum value of MeasuredValue that is capable of being measured. A MinMeasuredValue of 0x8000 indicates that the minimum value is unknown		
// @DH	maxMeasuredValue	It indicates the maximum value of MeasuredValue that is capable of being measured. MaxMeasuredValue shall be greater than MinMeasuredValue. MinMeasuredValue and MaxMeasuredValue define the range of the sensor. A MaxMeasuredValue of 0x8000 indicates that the maximum value is unknown.	
// @DH	tolerance			It indicates the magnitude of the possible error that is associated with MeasuredValue. The true value is located in the range (MeasuredValue – Tolerance) to (MeasuredValue + Tolerance).
// @DH	lightSensorType		It  specifies the electronic type of the light sensor. This attribute shall be set to one of the non-reserved values listed in Table
//              0x00 Photodiode
//              0x01 CMOS
//              0x40 – 0xfe Reserved for manufacturer specific light sensor types
//              0xff Unknown

// Set Measured Value Reporting
//
// @param: zbee
// ZBee object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: direction
// TRUE  - receiver will be configured to receiver reports
// FALSE - receiver will be configured to send reports
//
// @param: min_interval
// Minimal Reporting Interval
//
// @param: max_interval
// Maximum Reporting Interval
//
// @param: timeout
// Timeout
//
// @param: threshold
// Threshold
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_illuminance_measurement_set_measured_value_reporting(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBOOL direction, ZWWORD min_interval, ZWWORD max_interval, ZWWORD timeout, ZWWORD threshold, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Reset Measured Value Reporting
//
// @param: zbee
// ZBee object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: direction
// TRUE  - receiver will be configured to receiver reports
// FALSE - receiver will be configured to send reports
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_illuminance_measurement_reset_measured_value_reporting(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBOOL direction, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);


// Cluster Class TemperatureMeasurement (0x0402/1026) //
// Profile Zigbee Device Profile (0x0104/260) //
// The cluster provides an interface to temperature measurement functionality, including configuration and provision of notifications of temperature measurements
//
// @DH	measuredValue		It represents the temperature in degrees Celsius as follows: MeasuredValue = 100 x temperature in degrees Celsius. Where -273.15°C <= temperature <= 327.67 ºC, corresponding to a MeasuredValue in the range 0x954d to 0x7fff. The maximum resolution this format allows is 0.01 ºC. A MeasuredValue of 0x8000 indicates that the temperature measurement is invalid. MeasuredValue is updated continuously as new measurements are made.
// @DH	minMeasuredValue	It indicates the minimum value of MeasuredValue that is capable of being measured. A MinMeasuredValue of 0x8000 indicates that the minimum value is unknown.
// @DH	maxMeasuredValue	It indicates the maximum value of MeasuredValue that is capable of being measured. MaxMeasuredValue shall be greater than MinMeasuredValue. MinMeasuredValue and MaxMeasuredValue define the range of the sensor. A MaxMeasuredValue of 0x8000 indicates that the maximum value is unknown.
// @DH	tolerance		It indicates the magnitude of the possible error that is associated with MeasuredValue. The true value is located in the range (MeasuredValue – Tolerance) to (MeasuredValue + Tolerance).

// Set Measured Value Reporting
//
// @param: zbee
// ZBee object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: direction
// TRUE  - receiver will be configured to receiver reports
// FALSE - receiver will be configured to send reports
//
// @param: min_interval
// Minimal Reporting Interval
//
// @param: max_interval
// Maximum Reporting Interval
//
// @param: timeout
// Timeout
//
// @param: threshold
// Threshold
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_temperature_measurement_set_measured_value_reporting(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBOOL direction, ZWWORD min_interval, ZWWORD max_interval, ZWWORD timeout, ZWWORD threshold, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Reset Measured Value Reporting
//
// @param: zbee
// ZBee object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: direction
// TRUE  - receiver will be configured to receiver reports
// FALSE - receiver will be configured to send reports
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_temperature_measurement_reset_measured_value_reporting(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBOOL direction, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Set Tolerance Reporting
//
// @param: zbee
// ZBee object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: direction
// TRUE  - receiver will be configured to receiver reports
// FALSE - receiver will be configured to send reports
//
// @param: min_interval
// Minimal Reporting Interval
//
// @param: max_interval
// Maximum Reporting Interval
//
// @param: timeout
// Timeout
//
// @param: threshold
// Threshold
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_temperature_measurement_set_tolerance_reporting(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBOOL direction, ZWWORD min_interval, ZWWORD max_interval, ZWWORD timeout, ZWWORD threshold, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Reset Tolerance Reporting
//
// @param: zbee
// ZBee object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: direction
// TRUE  - receiver will be configured to receiver reports
// FALSE - receiver will be configured to send reports
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_temperature_measurement_reset_tolerance_reporting(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBOOL direction, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);


// Cluster Class PressureMeasurement (0x0403/1027) //
// Profile Zigbee Device Profile (0x0104/260) //
// The cluster provides an interface to pressure measurement functionality, including configuration and provision of notifications of pressure measurements
//
// @DH	measuredValue		It represents the pressure in kPa as follows: MeasuredValue = 10 x Pressure Where -3276.7 kPa <= Pressure <= 3276.7 kPa, corresponding to a MeasuredValue in the range 0x8001 to 0x7fff. A MeasuredValue of 0x8000 indicates that the pressure measurement is invalid. MeasuredValue is updated continuously as new measurements are made.
// @DH	minMeasuredValue	It indicates the minimum value of MeasuredValue that is capable of being measured. A MinMeasuredValue of 0x8000 indicates that the minimum value is unknown.
// @DH	maxMeasuredValue	It indicates the maximum value of MeasuredValue that is capable of being measured. MaxMeasuredValue shall be greater than MinMeasuredValue. MinMeasuredValue and MaxMeasuredValue define the range of the sensor. A MaxMeasuredValue of 0x8000 indicates that the maximum value is unknown.
// @DH	tolerance		It indicates the magnitude of the possible error that is associated with MeasuredValue. The true value is located in the range (MeasuredValue – Tolerance) to (MeasuredValue + Tolerance).
// @DH	scaledValue		It represents the pressure in Pascals as follows: ScaledValue = 10^Scale x Pressure in Pa Where -3276.7x10^Scale Pa <= Pressure <= 3276.7x10^Scale Pa corresponding to a ScaledValue in the range 0x8001 to 0x7fff. A ScaledValue of 0x8000 indicates that the pressure measurement is invalid. ScaledValue is updated continuously as new measurements are made.
// @DH	minScaledValue		It indicates the magnitude of the possible error that is associated with MeasuredValue. The true value is located in the range (MeasuredValue – Tolerance) to (MeasuredValue + Tolerance).
// @DH	maxScaledValue		It indicates the magnitude of the possible error that is associated with MeasuredValue. The true value is located in the range (MeasuredValue – Tolerance) to (MeasuredValue + Tolerance).
// @DH	scaledTolerance		It indicates the magnitude of the possible error that is associated with ScaledValue. The true value is located in the range (ScaledValue – ScaledTolerance) to (ScaledValue + ScaledTolerance).
// @DH	scale			It indicates  indicates the base 10 exponent used to obtain ScaledValue.

// Set Measured Value Reporting
//
// @param: zbee
// ZBee object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: direction
// TRUE  - receiver will be configured to receiver reports
// FALSE - receiver will be configured to send reports
//
// @param: min_interval
// Minimal Reporting Interval
//
// @param: max_interval
// Maximum Reporting Interval
//
// @param: timeout
// Timeout
//
// @param: threshold
// Threshold
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_pressure_measurement_set_measured_value_reporting(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBOOL direction, ZWWORD min_interval, ZWWORD max_interval, ZWWORD timeout, ZWWORD threshold, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Reset Measured Value Reporting
//
// @param: zbee
// ZBee object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: direction
// TRUE  - receiver will be configured to receiver reports
// FALSE - receiver will be configured to send reports
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_pressure_measurement_reset_measured_value_reporting(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBOOL direction, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Set Tolerance Reporting
//
// @param: zbee
// ZBee object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: direction
// TRUE  - receiver will be configured to receiver reports
// FALSE - receiver will be configured to send reports
//
// @param: min_interval
// Minimal Reporting Interval
//
// @param: max_interval
// Maximum Reporting Interval
//
// @param: timeout
// Timeout
//
// @param: threshold
// Threshold
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_pressure_measurement_set_tolerance_reporting(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBOOL direction, ZWWORD min_interval, ZWWORD max_interval, ZWWORD timeout, ZWWORD threshold, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Reset Tolerance Reporting
//
// @param: zbee
// ZBee object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: direction
// TRUE  - receiver will be configured to receiver reports
// FALSE - receiver will be configured to send reports
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_pressure_measurement_reset_tolerance_reporting(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBOOL direction, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Set Scaled Value Reporting
//
// @param: zbee
// ZBee object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: direction
// TRUE  - receiver will be configured to receiver reports
// FALSE - receiver will be configured to send reports
//
// @param: min_interval
// Minimal Reporting Interval
//
// @param: max_interval
// Maximum Reporting Interval
//
// @param: timeout
// Timeout
//
// @param: threshold
// Threshold
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_pressure_measurement_set_scaled_value_reporting(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBOOL direction, ZWWORD min_interval, ZWWORD max_interval, ZWWORD timeout, ZWWORD threshold, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Reset Scaled Value Reporting
//
// @param: zbee
// ZBee object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: direction
// TRUE  - receiver will be configured to receiver reports
// FALSE - receiver will be configured to send reports
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_pressure_measurement_reset_scaled_value_reporting(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBOOL direction, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Set Scaled Tolerance Reporting
//
// @param: zbee
// ZBee object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: direction
// TRUE  - receiver will be configured to receiver reports
// FALSE - receiver will be configured to send reports
//
// @param: min_interval
// Minimal Reporting Interval
//
// @param: max_interval
// Maximum Reporting Interval
//
// @param: timeout
// Timeout
//
// @param: threshold
// Threshold
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_pressure_measurement_set_scaled_tolerance_reporting(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBOOL direction, ZWWORD min_interval, ZWWORD max_interval, ZWWORD timeout, ZWWORD threshold, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Reset Scaled Tolerance Reporting
//
// @param: zbee
// ZBee object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: direction
// TRUE  - receiver will be configured to receiver reports
// FALSE - receiver will be configured to send reports
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_pressure_measurement_reset_scaled_tolerance_reporting(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBOOL direction, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);


// Cluster Class RelativeHumidityMeasurement (0x0405/1029) //
// Profile Zigbee Device Profile (0x0104/260) //
// The cluster provides an interface to relative humidity measurement functionality, including configuration and provision of notifications of temperature measurements
//
// @DH	measuredValue		It represents the relative humidity in % as follows: MeasuredValue = 100 x Relative humidity Where 0% <= Relative humidity <= 100%, corresponding to a MeasuredValue in the range 0 to 0x2710. The maximum resolution this format allows is 0.01%. A MeasuredValue of 0xffff indicates that the measurement is invalid. MeasuredValue is updated continuously as new measurements are made.
// @DH	minMeasuredValue	It indicates the minimum value of MeasuredValue that is capable of being measured. A MinMeasuredValue of 0x8000 indicates that the minimum value is unknown.
// @DH	maxMeasuredValue	It indicates the maximum value of MeasuredValue that is capable of being measured. MaxMeasuredValue shall be greater than MinMeasuredValue. MinMeasuredValue and MaxMeasuredValue define the range of the sensor. A MaxMeasuredValue of 0x8000 indicates that the maximum value is unknown.
// @DH	tolerance		It indicates the magnitude of the possible error that is associated with MeasuredValue. The true value is located in the range (MeasuredValue – Tolerance) to (MeasuredValue + Tolerance).

// Set Measured Value Reporting
//
// @param: zbee
// ZBee object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: direction
// TRUE  - receiver will be configured to receiver reports
// FALSE - receiver will be configured to send reports
//
// @param: min_interval
// Minimal Reporting Interval
//
// @param: max_interval
// Maximum Reporting Interval
//
// @param: timeout
// Timeout
//
// @param: threshold
// Threshold
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_relative_humidity_measurement_set_measured_value_reporting(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBOOL direction, ZWWORD min_interval, ZWWORD max_interval, ZWWORD timeout, ZWWORD threshold, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Reset Measured Value Reporting
//
// @param: zbee
// ZBee object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: direction
// TRUE  - receiver will be configured to receiver reports
// FALSE - receiver will be configured to send reports
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_relative_humidity_measurement_reset_measured_value_reporting(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBOOL direction, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Set Tolerance Reporting
//
// @param: zbee
// ZBee object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: direction
// TRUE  - receiver will be configured to receiver reports
// FALSE - receiver will be configured to send reports
//
// @param: min_interval
// Minimal Reporting Interval
//
// @param: max_interval
// Maximum Reporting Interval
//
// @param: timeout
// Timeout
//
// @param: threshold
// Threshold
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_relative_humidity_measurement_set_tolerance_reporting(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBOOL direction, ZWWORD min_interval, ZWWORD max_interval, ZWWORD timeout, ZWWORD threshold, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Reset Tolerance Reporting
//
// @param: zbee
// ZBee object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: direction
// TRUE  - receiver will be configured to receiver reports
// FALSE - receiver will be configured to send reports
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_relative_humidity_measurement_reset_tolerance_reporting(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBOOL direction, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);


// Cluster Class OccupancySensing (0x0406/1030) //
// Profile Zigbee Device Profile (0x0104/260) //
// The server cluster provides an interface to occupancy sensing functionality, including configuration and provision of notifications of occupancy status.
//
// @DH	occupancy								Occupied State
//				0 - Unoccupied
//				1 - Occupied
// @DH	occupancySensorType						Specifies the type of the occupancy sensor
// @DH	pirOccupiedToUnoccupiedDelay			Specifies the time delay, in seconds, before the PIR sensor changes to its unoccupied state after the last detection of movement in the sensed area.
// @DH	pirUnoccupiedToOccupiedDelay			Specifies the time delay, in seconds, before the PIR sensor changes to its occupied state after the detection of movement in the sensed area
// @DH	pirOccupiedToUnoccupiedThreshold		Specifies the number of movement detection events that must occur in the period PIRUnoccupiedToOccupiedDelay, before the PIR sensor changes to its occupied state
// @DH	ultrasonicOccupiedToUnoccupiedDelay		Specifies the time delay, in seconds, before the Ultrasonic sensor changes to its unoccupied state after the last detection of movement in the sensed area.
// @DH	ultrasonicUnoccupiedToOccupiedDelay		Specifies the time delay, in seconds, before the Ultrasonic sensor changes to its occupied state after the detection of movement in the sensed area.
// @DH	ultrasonicOccupiedToUnoccupiedThreshold	Specifies the number of movement detection events that must occur in the period UltrasonicUnoccupiedToOccupiedDelay, before the Ultrasonic sensor changes to its occupied state

// Set Occupancy Reporting
//
// @param: zbee
// ZBee object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: direction
// TRUE  - receiver will be configured to receiver reports
// FALSE - receiver will be configured to send reports
//
// @param: min_interval
// Minimal Reporting Interval
//
// @param: max_interval
// Maximum Reporting Interval
//
// @param: timeout
// Timeout
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_occupancy_sensing_set_occupancy_reporting(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBOOL direction, ZWWORD min_interval, ZWWORD max_interval, ZWWORD timeout, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Reset Occupancy Reporting
//
// @param: zbee
// ZBee object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: direction
// TRUE  - receiver will be configured to receiver reports
// FALSE - receiver will be configured to send reports
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWError zbee_cc_occupancy_sensing_reset_occupancy_reporting(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBOOL direction, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);


// Cluster Class Metering (0x0702/1794) //
// Profile ZigBee Device Profile (0x0104/260) //
// The Metering Cluster provides a mechanism to retrieve usage information from Electric, Gas, Water, and potentially Thermal metering devices. These devices can operate on either battery or mains power, and can have a wide variety of sophistication. The Metering Cluster is designed to provide flexibility while limiting capabilities to a set number of metered information types. More advanced forms or data sets from metering devices will be supported in the Smart Energy Tunneling Cluster. The following figures identify three configurations as examples utilizing the Metering Cluster.
//
// @DH	currentSummationDelivered								(Id 0x0000) CurrentSummationDelivered represents the most recent summed value of Energy, Gas, or Water delivered and consumed in the premises. CurrentSummationDelivered is mandatory and must be provided as part of the minimum data set to be provided by the metering device. CurrentSummationDelivered is updated continuously as new measurements are made.
// @DH	currentSummationReceived								(Id 0x0001) CurrentSummationReceived represents the most recent summed value of Energy, Gas, or Water generated and delivered from the premises. If optionally provided, CurrentSummationReceived is updated continuously as new measurements are made.
// @DH	currentMaxDemandDelivered								(Id 0x0002) CurrentMaxDemandDelivered represents the maximum demand or rate of delivered value of Energy, Gas, or Water being utilized at the premises. If optionally provided, CurrentMaxDemandDelivered is updated continuously as new measurements are made.
// @DH	currentMaxDemandReceived								(Id 0x0003) CurrentMaxDemandReceived represents the maximum demand or rate of received value of Energy, Gas, or Water being utilized by the utility. If optionally provided, CurrentMaxDemandReceived is updated continuously as new measurements are made.
// @DH	dFtSummation											(Id 0x0004) DFTSummation represents a snapshot of attribute CurrentSummationDelivered captured at the time indicated by attribute DailyFreezeTime. If optionally provided, DFTSummation is updated once every 24 hours and captured 
// @DH	dailyFreezeTime											(Id 0x0005) DailyFreezeTime represents the time of day when DFTSummation is captured. DailyFreezeTime is an unsigned 16-bit value representing the hour and minutes for DFT. The byte usages are: Bits 0 to 7: Range of 0 to 0x3C representing the number of minutes past the top of the hour. Bits 8 to 15: Range of 0 to 0x17 representing the hour of the day (in 24-hour format).
// @DH	powerFactor												(Id 0x0006) PowerFactor contains the Average Power Factor ratio in 1/100ths. Valid values are 0 to 99.
// @DH	readingSnapShotTime										(Id 0x0007) The ReadingSnapShotTime attribute represents the last time all of the CurrentSummationDelivered, CurrentSummationReceived, CurrentMaxDemandDelivered, and CurrentMaxDemandReceived attributes that are supported by the device were updated.
// @DH	currentMaxDemandDeliveredTime							(Id 0x0008) The CurrentMaxDemandDeliveredTime attribute represents the time when CurrentMaxDemandDelivered reading was captured
// @DH	currentMaxDemandReceivedTime							(Id 0x0009) The CurrentMaxDemandReceivedTime attribute represents the time when CurrentMaxDemandReceived reading was captured.
// @DH	defaultUpdatePeriod										(Id 0x000A) The DefaultUpdatePeriod attribute represents the interval (seconds) at which the InstantaneousDemand attribute is updated when not in fast poll mode. InstantaneousDemand may be continuously updated as new measurements are acquired, but at a minimum InstantaneousDemand must be updated at the DefaultUpdatePeriod. The DefaultUpdatePeriod may apply to other attributes as defined by the device manufacturer.
// @DH	fastPollUpdatePeriod									(Id 0x000B) The FastPollUpdatePeriod attribute represents the interval (seconds) at which the InstantaneousDemand attribute is updated when in fast poll mode. InstantaneousDemand may be continuously updated as new measurements are acquired, but at a minimum, InstantaneousDemand must be updated at the FastPollUpdatePeriod. The FastPollUpdatePeriod may apply to other attributes as defined by the device manufacturer.
// @DH	currentBlockPeriodConsumptionDelivered					(Id 0x000C) The CurrentBlockPeriodConsumptionDelivered attribute represents the most recent summed value of Energy, Gas or Water delivered and consumed in the premises during the Block Tariff Period. The CurrentBlockPeriodConsumptionDelivered is reset at the start of each Block Tariff Period.
// @DH	dailyConsumptionTarget									(Id 0x000D) The DailyConsumptionTarget attribute is a daily target consumption amount that can be displayed to the consumer on a HAN device, with the intent that it can be used to compare to actual daily consumption (e.g. compare to the CurrentDayConsumptionDelivered). This may be sent from the utility to the ESI, or it may be derived. Although intended to be based on Block Thresholds, it can be used for other targets not related to blocks. The formatting will be based on the HistoricalConsumptionFormatting attribute. Example: If based on a Block Threshold, the DailyConsumptionTarget could be calculated based on the number of days specified in the Block Tariff Period and a given Block Threshold as follows: DailyConsumptionTarget = BlockNThreshold / ((BlockPeriodDuration /60) / 24). Example: If the target is based on a Block1Threshold of 675kWh and where 43200 BlockThresholdPeriod is the number of minutes in the billing period (30 days), the ConsumptionDailyTarget would be 675 / ((43200 / 60) / 24) = 22.5 kWh per day.
// @DH	currentBlock											(Id 0x000E) When Block Tariffs are enabled, CurrentBlock is an 8-bit Enumeration which indicates the currently active block. If blocks are active then the current active block is based on the CurrentBlockPeriodConsumptionDelivered and the block thresholds. Block 1 is active when the value of CurrentBlockPeriodConsumptionDelivered is less than Block1Threshold value; Block 2 is active when CurrentBlockPeriodConsumptionDelivered is greater than Block1Threshold value and less than Block2Threshold value, and so on. Block 16 is active when the value of CurrentBlockPeriodConsumptionDelivered is greater than Block15Threshold value
//				0x00 No Blocks in use
//				0x01 Block1
//				0x02 Block2
//				0x03 Block3
//				0x04 Block4
//				0x05 Block5
//				0x06 Block6
//				0x07 Block7
//				0x08 Block8
//				0x09 Block9
//				0x0A Block10
//				0x0B Block11
//				0x0C Block12
//				0x0D Block13
//				0x0E Block14
//				0x0F Block15
//				0x10 Block16
// @DH	profileIntervalPeriod									(Id 0x000F) The ProfileIntervalPeriod attribute is currently included in the Get Profile Response command payload, but does not appear in an attribute set. This represents the duration of each interval. ProfileIntervalPeriod represents the interval or time frame used to capture metered Energy, Gas, and Water consumption for profiling purposes. The enumeration for this field shall match one of the ProfileIntervalPeriod values
// @DH	intervalReadReportingPeriod								(Id 0x0010) The IntervalReadReportingPeriod attribute represents how often (in minutes) the water or gas meter is to wake up and provide interval data. E.g.: If IntervalReadReportingPeriod is set to 360, then every 6 hours the water or gas mete r is to wake up and provide 6 hours of interval data in a Get Profile Response command. If it is set to 5760, then every 4 days it will wake up and provide 4 days of interval data in a Get Profile Response command. In some cases data may overlap data sent in previous Get Profile Response comm a nd
// @DH	presetReadingTime										(Id 0x0011) The PresetReadingTime attribute represents the time of day (in quarter hour increments) at which the meter will wake up and report a register reading even if there has been no consumption for the previous 24 hours. PresetReadingTime is an unsigned 16-bit value representing the hour and minutes. The byte usages are: Bits 0 to 7: Range of 0 to 0x3B representing the number of minutes past the top of the hour Bits 8 to 15: Range of 0 to 0x17 representing the hour of the day (in 24-hour format). E.g.: A setting of 0x172D would represent 23:45 hours or 11:45 pm; a setting of 0x071E would represent 07:30 hours or 7:30 am. A setting of 0xFFFF indicates this feature is disabled. The use of Attribute Reporting Configuration is optional.
// @DH	volumePerReport											(Id 0x0012) The VolumePerReport attribute represents the volume per report increment from the water or gas meter. For example a gas meter might be set to report its register reading for every time 1 cubic meter of gas is used. For a water meter it might report the register value every 10 liters of water usage.
// @DH	flowRestriction											(Id 0x0013) The FlowRestriction attribute represents the volume per minute limit set in the flow restrictor. This applies to water but not for gas. A setting of 0xFF indicates this feature is disabled.
// @DH	supply													(Id 0x0014) The SupplyStatus attribute represents the state of the supply at the customer's premises.
//				0x00 Supply OFF
//				0x01 Supply OFF/ARMED
//				0x02 Supply ON
// @DH	currentInletEnergyCarrierSummation						(Id 0x0015) CurrentInletEnergyCarrierSummation is the current integrated volume of a given energy carrier measured on the inlet. The formatting and unit of measure for this value is specified in the EnergyCarrierUnitOfMeasure and EnergyCarrierSummationFormatting attributes. The Energy consumption registered in CurrentSummationDelivered is not necessarily a direct function of this value. The quality of the energy carrier may vary from day to day, e.g. Gas may have different quality. For heat and cooling meters the energy carrier is water at high or low temperature, the energy withdrawn from such a system is a function of the flow and the inlet and outlet temperature.
// @DH	currentOutletEnergyCarrierSummation						(Id 0x0016) CurrentOutletEnergyCarrierSummation is the current integrated volume of a given energy carrier measured on the outlet. The formatting and unit of measure for this value is specified in the EnergyCarrierUnitOfMeasure and EnergyCarrierSummationFormatting attributes
// @DH	inletTemperature										(Id 0x0017) InletTemperature is the temperature measured on the energy carrier inlet. The formatting and unit of measure for this value is specified in the TemperatureUnitOfMeasure and TemperatureFormatting attributes
// @DH	outletTemperature										(Id 0x0018) OutletTemperature is the temperature measured on the energy carrier outlet. The formatting and unit of measure for this value is specified in the TemperatureUnitOfMeasure and TemperatureFormatting attributes
// @DH	controlTemperature										(Id 0x0019) ControlTemperature is a reference temperature measured on the meter used to validate the Inlet/Outlet temperatures. The formatting and unit of measure for this value is specified in the TemperatureUnitOfMeasure and TemperatureFormatting attributes
// @DH	currentInletEnergyCarrierDemand							(Id 0x001A) CurrentInletEnergyCarrierDemand is the current absolute demand on the energy carrier inlet. The formatting and unit of measure for this value is specified in the EnergyCarrierUnitOfMeasure and EnergyCarrierDemandFormatting attributes. For a heat or cooling meter this will be the current absolute flow rate measured on the inlet.
// @DH	currentOutletEnergyCarrierDemand						(Id 0x001B) CurrentInletEnergyCarrierDemand is the current absolute demand on the energy carrier inlet. The formatting and unit of measure for this value is specified in the EnergyCarrierUnitOfMeasure and EnergyCarrierDemandFormatting attributes. For a heat or cooling meter this will be the current absolute flow rate measured on the inlet.
// @DH	previousBlockPeriodConsumptionDelivere					(Id 0x001C) The PreviousBlockPeriodConsumptionDelivered attribute represents the total value of Energy, Gas or Water delivered and consumed in the premises at the end of the previous Block Tariff Period. If supported, the PreviousBlockPeriodConsumptionDelivered attribute is updated at the end of each Block Tariff Period.
// @DH	currentTierXSummationDelivered							(Id 0x010N) (N - 0 .. 15. Decimal in name and hex in Id) Attributes CurrentTier1SummationDelivered through CurrentTierNSummationDelivered represent the most recent summed value of Energy, Gas, or Water delivered to the premises (i.e. delivered to the customer from the utility) at a specific price tier as defined by a TOU schedule or a real time pricing period. If optionally provided, attributes CurrentTier1SummationDelivered through CurrentTierNSummationDelivered are up- dated continuously as new measurements are made. 
// @DH	currentTierXSummationReceived							(Id 0x010N) (N - 0 .. 15. Decimal in name and hex in Id) Attributes CurrentTier1SummationReceived through CurrentTierNSummationReceived represent the most recent summed value of Energy, Gas, or Water provided by the premises (i.e. received by the utility from the customer) at a specific price tier as defined by a TOU schedule or a real time pricing period. If optionally provided, attributes CurrentTier1SummationReceived through CurrentTierNSummationReceived are updated continuously as new measurements are made.
// @DH	status													(Id 0x0200) The Status attribute provides indicators reflecting the current error conditions found by the metering device. This attribute is an 8-bit field where when an individual bit is set, an error or warning condition exists. The behavior causing the setting or resetting each bit is device specific. In other words, the application within the metering device will determine and control when these settings are either set or cleared. Depending on the commodity type, the bits of this attribute will take on different meaning. Table show the bit mappings for the Status attribute for Electricity, Gas, Water and Heating/Cooling, respectively. A battery-operated meter will report any change in state of the Status when it wakes up via a ZCL report attributes command. The ESI is expected to make alarms available to upstream systems together with consumption data collected from the battery operated meter
//			Electricity
//				0th bit Check Meter: Set to true when a non fatal problem has been detected on the meter such as a measurement error, memory error, and self check error.
//				1st bit Low Battery: Set to true when the battery needs maintenance.
//				2nd bit Tamper Detect: Set to true if a tamper event has been detected.
//				3rd bit Power Failure: Set to true during a power outage.
//				4th bit Power Quality: Set to true if a power quality event have been detected such as a low voltage, high voltage.
//				5th bit Leak Detect: Set to true when a leak have been detected.
//				6th bit Service Disconnect Open: Set to true when the service have been disconnected to this premises.
//				7th bit Reserved
//			Gas
//				0th bit Check Meter: Set to true when a non fatal problem has been detected on the meter such as a measurement error, memory error, or self check error.
//				1st bit Low Battery: Set to true when the battery needs maintenance.
//				2nd bit Tamper Detect: Set to true if a tamper event has been detected.
//				3rd bit Not Defines
//				4th bit Low Pressure: Set to true when the pressure at the meter is below the meter's low pressure threshold value.
//				5th bit Leak Detect: Set to true when a leak has been detected.
//				6th bit Service Disconnect: Set to true when the service has been disconnected to this premises. Ex. The valve is in the closed position preventing delivery of gas.
//				7th bit Reverse Flow: Set to true if flow detected in the opposite direction to normal (from consumer to supplier).
//			Water
//				0th bit Check Meter: Set to true when a non fatal problem has been detected on the meter such as a measurement error, memory error, or self check error.
//				1st bit Low Battery: Set to true when the battery needs maintenance.
//				2nd bit Tamper Detect: Set to true if a tamper event has been detected.
//				3rd bit Pipe Empty: Set to true when the service pipe at the meter is empty and there is no flow in either direction.
//				4th bit Low Pressure: Set to true when the pressure at the meter is below the meter's low pressure threshold value.
//				5th bit Leak Detect: Set to true when a leak has been detected.
//				6th bit Service Disconnect: Set to true when the service has been disconnected to this premises. Ex. The valve is in the closed position preventing delivery of water.
//				7th bit Reverse Flow: Set to true if flow detected in the opposite direction to normal (from consumer to supplier).
// @DH	remainingBatteryLife									(Id 0x0201) RemainingBatteryLife represents the estimated remaining life of the battery in % of capacity. A setting of 0xFF indicates this feature is disabled. The range 0 - 100 where 100 = 100%, 0xFF = Unknown
// @DH	hoursInOperation										(Id 0x0202) HoursInOperation is a counter that increments once every hour during operation. This may be used as a check for tampering. Note: For meters that are not electricity meters turning off the meter does not necessarily prevent delivery of energy—but the meter might not be able to measure it
// @DH	hoursInFault											(Id 0x0203) HoursInFault is a counter that increments once every hour when the device is in operation with a fault detected. This may be used as a check for tampering. Note: For meters that are not electricity meters turning off the meter does not necessarily prevent delivery of energy—but the meter might not be able to measure it.
// @DH	unitofMeasure											(Id 0x0300) UnitofMeasure provides a label for the Energy, Gas, or Water being measured by the metering device. The units of measure apply to all summations, consumptions/ profile interval and demand/rate supported by this cluster. Other measurements such as the power factor are self describing. This attribute is an 8-bit enu- merated field. The bit descriptions for this Attribute are listed in Table. Note: When using BCD for meter reads, the values A to F are special values or indicators denoting “Opens”, “Shorts”, and etc. conditions when reading meter register hardware. Any SE device displaying the BCD based values to end users should use a non-decimal value to replace the A to F. In other words, a device could use an “*” in place of the special values or indicators.
//				0x00 kWh (Kilowatt Hours) & kW (Kilowatts) in pure binary format
//				0x01 m3 (Cubic Meter) & m3/h (Cubic Meter per Hour) in pure binary format
//				0x02 ft3 (Cubic Feet) & ft3/h (Cubic Feet per Hour) in pure binary format
//				0x03 ccf ((100 or Centum) Cubic Feet) & ccf/h ((100 or Centum) Cubic Feet per Hour) in pure binary format
//				0x04 US gl (US Gallons) & US gl/h (US Gallons per Hour) in pure binary format.
//				0x05 IMP gl (Imperial Gallons) & IMP gl/h (Imperial Gallons per Hour) in pure binary format
//				0x06 BTUs & BTU/h in pure binary format
//				0x07 Liters & l/h (Liters per Hour) in pure binary format
//				0x08 kPA (gauge) in pure binary format
//				0x09 kPA (absolute) in pure binary format
//				0x0A mcf (1000 Cubic Feet) & mcf/h (1000 Cubic feet per hour) in pure binary format
//				0x0B Unitless in pure binary format
//				0x0C MJ (Mega Joule) and MJ/s (Mega Joule per second (MW)) in pure binary format
//				0x80 kWh (Kilowatt Hours) & kW (Kilowatts) in BCD format
//				0x81 m3 (Cubic Meter) & m3/h (Cubic Meter per Hour) in BCD format
//				0x82 ft3 (Cubic Feet) & ft3/h (Cubic Feet per Hour) in BCD format
//				0x83 ccf ((100 or Centum) Cubic Feet) & ccf/h ((100 or Centum) Cubic Feet per Hour) in BCD format
//				0x84 US gl (US Gallons) & US gl/h (US Gallons per Hour) in BCD format
//				0x85 IMP gl (Imperial Gallons) & IMP gl/h (Imperial Gallons per Hour) in BCD format
//				0x86 BTUs & BTU/h in BCD format
//				0x87 Liters & l/h (Liters per Hour) in BCD format
//				0x88 kPA (gauge) in BCD format
//				0x89 kPA (absolute) in BCD format
//				0x8A mcf (1000 Cubic Feet) & mcf/h (1000 Cubic Feet per Hour) in BCD format
//				0x8B unitless in BCD format
// @DH	multiplier												(Id 0x0301) Multiplier provides a value to be multiplied against a raw or uncompensated sensor count of Energy, Gas, or Water being measured by the metering device. If present, this attribute must be applied against all summation, consumption and demand values to derive the delivered and received values expressed in the unit of measure specified. This attribute must be used in conjunction with the Divisor attribute.
// @DH	divisor													(Id 0x0302) Divisor provides a value to divide the results of applying the Multiplier Attribute against a raw or uncompensated sensor count of Energy, Gas, or Water being measured by the metering device. If present, this attribute must be applied against all summation, consumption and demand values to derive the delivered and received values expressed in the unit of measure specified. This attribute must be used in conjunction with the Multiplier attribute.
// @DH	summationFormatting										(Id 0x0303) SummationFormatting provides a method to properly decipher the number of digits and the decimal location of the values found in the Summation Information Set of attributes. This attribute is to be decoded as follows: Bits 0 to 2: Number of Digits to the right of the Decimal Point. Bits 3 to 6: Number of Digits to the left of the Decimal Point. Bit 7: If set, suppress leading zeros. This attribute shall be used against the following attributes: 1) CurrentSummationDelivered; 2) CurrentSummationReceived; 3) TOU Information attributes; 4) DFTSummation; 5) Block Information attributes
// @DH	demandFormatting										(Id 0x0304) DemandFormatting provides a method to properly decipher the number of digits and the decimal location of the values found in the Demand-related attributes. This attribute is to be decoded as follows: Bits 0 to 2: Number of Digits to the right of the Decimal Point. Bits 3 to 6: Number of Digits to the left of the Decimal Point. Bit 7: If set, suppress leading zeros. This attribute shall be used against the following attributes: 1) CurrentMaxDemandDelivered; 2) CurrentMaxDemandReceived; 3) InstantaneousDemand
// @DH	historicalConsumptionFormatting							(Id 0x0305) HistoricalConsumptionFormatting provides a method to properly decipher the number of digits and the decimal location of the values found in the Historical Consumption Set of attributes. This attribute is to be decoded as follows: Bits 0 to 2: Number of Digits to the right of the Decimal Point. Bits 3 to 6: Number of Digits to the left of the Decimal Point. Bit 7: If set, suppress leading zeros. This attribute shall be used against the following attributes: 1) CurrentDayConsumptionDelivered; 2) CurrentDayConsumptionReceived; 3) PreviousDayConsumptionDelivered; 4) PreviousDayConsumptionReceived; 5) CurrentPartialProfileIntervalValue; 6) Intervals; 7) DailyConsumptionTarget
// @DH	meteringDeviceType										(Id 0x0306) MeteringDeviceType provides a label for identifying the type of metering device present. The attribute are values representing Energy, Gas, Water, Thermal, Heat, Cooling, and mirrored metering devices. The defined values. (Note that these values represent an Enumeration, and not an 8-bit bitmap as indicated in the attribute description. For backwards compatibility reasons, the data type has not been changed, though the data itself should be treated like an enum.) Where a mirror is provided for a battery-powered metering device, the mirror shall assume the relevant 'Mirrored Metering' device type (128-133) whilst the meter itself shall utilize the 'Metering' device type (1 to 6). It shall be the responsibility of the device providing the mirror to modify the Device Type shown on the mirror to that of a 'Mirrored Metering' device. Note: Heat and cooling meters are used for measurement and billing of heat (and cooling) delivered through liquid (water) based central heating systems. The consumers are typically billed by the kWh, calculated from the flow and the temperatures in and out.
//				0 Electric Metering
//				1 Gas Metering
//				2 Water Metering
//				3 Thermal Metering (deprecated)
//				4 Pressure Metering
//				5 Heat Metering
//				6 Cooling Metering
//				128 Mirrored Gas Metering
//				129 Mirrored Water Metering
//				130 Mirrored Thermal Metering (deprecated)
//				131 Mirrored Pressure Metering
//				132 Mirrored Heat Metering
//				133 Mirrored Cooling Metering
// @DH	siteId													(Id 0x0307) The SiteID is a ZCL Octet String field capable of storing a 32 character string (the first octet indicates length) encoded in UTF-8 format. The SiteID is a text string, known in the UK as the M-PAN number for electricity, MPRN for gas and 'Stand Point' in South Africa. These numbers specify the meter point location in a standardized way. The field is defined to accommodate the number of characters typically found in the UK and Europe (16 digits). Generally speaking the field is numeric but is defined for the possibility of an al- pha-numeric format by specifying an octet string
// @DH	meterSerialNumber										(Id 0x0308) The MeterSerialNumber is a ZCL Octet String field capable of storing a 24 character string (the first octet indicates length) encoded in UTF-8 format. It is used to provide a unique identification of the metering device.
// @DH	energyCarrierUnitOfMeasure								(Id 0x0309) The EnergyCarrierUnitOfMeasure specifies the unit of measure that the EnergyCarrier is measured in. This unit of measure is typically a unit of volume or flow and cannot be an amount of energy. The enumeration of this attribute is otherwise identical to the UnitofMeasure attribute 
// @DH	energyCarrierSummationFormatting						(Id 0x030A) EnergyCarrierSummationFormatting provides a method to properly decipher the number of digits and the decimal location of the values found in the Summation- related attributes. This attribute is to be decoded as follows: Bits 0 to 2: Number of Digits to the right of the Decimal Point. Bits 3 to 6: Number of Digits to the left of the Decimal Point. Bit 7: If set, suppress leading zeros. This attribute shall be used in relation with the following attributes: 1) CurrentInletEnergyCarrierSummation; 2) CurrentOutletEnergyCarrierSummation
// @DH	energyCarrierDemandFormatting							(Id 0x030B) EnergyCarrierDemandFormatting provides a method to properly decipher the number of digits and the decimal location of the values found in the Demand-related attributes. This attribute is to be decoded as follows: Bits 0 to 2: Number of Digits to the right of the Decimal Point. Bits 3 to 6: Number of Digits to the left of the Decimal Point. Bit 7: If set, suppress leading zeros. This attribute shall be used in relation with the following attributes: 1) CurrentInletEnergyCarrierDemand; 2) CurrentOutletEnergyCarrierDemand; 3) CurrentDayMaxEnergyCarrierDemand; 4) PreviousDayMaxEnergyCarrierDemand; 5) CurrentMonthMaxEnergyCarrierDemand; 6) CurrentMonthMinEnergyCarrierDemand; 7) CurrentYearMinEnergyCarrierDemand; 8) CurrentYearMaxEnergyCarrierDemand
// @DH	temperatureUnitOfMeasure								(Id 0x030C) The TemperatureUnitOfMeasure specifies the unit of measure that temperatures are measured in. The enumeration of this attribute is shown in Table
//				0x00 K (Degrees Kelvin) in pure Binary format.
//				0x01 °C (Degrees Celsius) in pure Binary format.
//				0x02 °F (Degrees Fahrenheit) in pure Binary format.
//				0x80 K (Degrees Kelvin) in BCD format.
//				0x81 °C (Degrees Celsius) in BCD format.
//				0x82 °F (Degrees Fahrenheit) in BCD format.
// @DH	temperatureFormatting									(Id 0x030D) TemperatureFormatting provides a method to properly decipher the number of digits and the decimal location of the values found in the Temperature-related attributes. This attribute is to be decoded as follows: Bits 0 to 2: Number of Digits to the right of the Decimal Point. Bits 3 to 6: Number of Digits to the left of the Decimal Point. Bit 7: If set, suppress leading zeros. This attribute shall be used in relation with the following attributes: 1) InletTemperature; 2) OutletTemperature; 3) ControlTemperature
// @DH	instantaneousDemand										(Id 0x0400) InstantaneousDemand represents the current Demand of Energy, Gas, or Water delivered or received at the premises. Positive values indicate demand delivered to the premises where negative values indicate demand received from the premises. InstantaneousDemand is updated continuously as new measurements are made. The frequency of updates to this field is specific to the metering device, but should be within the range of once every second to once every 5 seconds 
// @DH	currentDayConsumptionDelivered							(Id 0x0401) CurrentDayConsumptionDelivered represents the summed value of Energy, Gas, or Water generated and delivered to the premises since midnight local time. If optionally provided, CurrentDayConsumptionDelivered is updated continuously as new measurements are made.
// @DH	currentDayConsumptionReceived							(Id 0x0402) CurrentDayConsumptionReceived represents the summed value of Energy, Gas, or Water generated and received from the premises since midnight local time. If optionally provided, CurrentDayConsumptionReceived is updated continuously as new measurements are made
// @DH	previousDayConsumptionDelivered							(Id 0x0403) PreviousDayConsumptionDelivered represents the summed value of Energy, Gas, or Water generated and delivered to the premises within the previous 24 hour period starting at midnight local time. If optionally provided, CurrentDayConsumptionDelivered is updated every midnight local time
// @DH	previousDayConsumptionReceived							(Id 0x0404) PreviousDayConsumptionReceived represents the summed value of Energy, Gas, or Water generated and received from the premises within the previous 24 hour period starting at midnight local time. If optionally provided, CurrentDayConsumptionReceived is updated is updated every midnight local time.
// @DH	currentPartialProfileIntervalStartTimeDelivered			(Id 0x0405) CurrentPartialProfileIntervalStartTimeDelivered represents the start time of the current Load Profile interval being accumulated for commodity delivered.
// @DH	currentPartialProfileIntervalStartTimeReceived			(Id 0x0406) CurrentPartialProfileIntervalStartTimeReceived represents the start time of the current Load Profile interval being accumulated for commodity received
// @DH	currentPartialProfileIntervalValueDelivered				(Id 0x0407) CurrentPartialProfileIntervalValueDelivered represents the value of the current Load Profile interval being accumulated for commodity delivered
// @DH	currentPartialProfileIntervalValueReceived				(Id 0x0408) CurrentPartialProfileIntervalValueReceived represents the value of the current Load Profile interval being accumulated for commodity received.
// @DH	currentDayMaxPressure									(Id 0x0409) CurrentDayMaxPressure is the maximum pressure reported during a day from the water or gas meter.
// @DH	currentDayMinPressure									(Id 0x040A) CurrentDayMinPressure is the minimum pressure reported during a day from the water or gas meter.
// @DH	previousDayMaxPressure									(Id 0x040B) PreviousDayMaxPressure represents the maximum pressure reported during previous day from the water or gas meter.
// @DH	previousDayMinPressure									(Id 0x040C) PreviousDayMinPressure represents the minimum pressure reported during previous day from the water or gas meter
// @DH	currentDayMaxDemand										(Id 0x040D) CurrentDayMaxDemand represents the maximum demand or rate of delivered value of Energy, Gas, or Water being utilized at the premises.
// @DH	previousDayMaxDemand									(Id 0x040E) PreviousDayMaxDemand represents the maximum demand or rate of delivered value of Energy, Gas, or Water being utilized at the premises. Note: At the end of a day the metering device will transfer the CurrentDayMaxPressure into PreviousDayMaxPressure, CurrentDayMinPressure into PreviousDayMinPressure and CurrentDayMaxDemand into PreviousDayMaxDemand.
// @DH	currentMonthMaxDemand									(Id 0x040F) CurrentMonthMaxDemand is the maximum demand reported during a month from t he meter. For electricity, heat and cooling meters this is the maximum power reported in a month.
// @DH	currentYearMaxDemand									(Id 0x0410) CurrentYearMaxDemand is the maximum demand reported during a year from the meter. For electricity, heat and cooling meters this is the maximum power reported in a year.
// @DH	currentDayMaxEnergyCarrierDemand						(Id 0x0411) CurrentDayMaxEnergyCarrierDemand is the maximum energy carrier demand reported during a day from the meter. Note: At the end of a day the meter will transfer the CurrentDayMaxEnergyCarrierDemand into PreviousDayMaxEnergyCarrierDemand. For heat and cooling meters this is the maximum flow rate on the inlet reported in a day.
// @DH	previousDayMaxEnergyCarrierDemand						(Id 0x0412) PreviousDayMaxEnergyCarrierDemand is the maximum energy carrier demand reported during the previous day from the meter
// @DH	currentMonthMaxEnergyCarrierDemand						(Id 0x0413) CurrentMonthMaxEnergyCarrierDemand is the maximum energy carrier demand reported during a month from the meter. For heat and cooling meters this is the maximum flow rate on the inlet reported in a month
// @DH	currentMonthMinEnergyCarrierDemand						(Id 0x0414) CurrentMonthMinEnergyCarrierDemand is the minimum energy carrier demand reported during a month from the meter. For heat and cooling meters this is the minimum flow rate on the inlet reported in a month. Note: This attribute may be used to detect leaks if there has been no flow rate of zero in the last month
// @DH	currentYearMaxEnergyCarrierDemand						(Id 0x0415) CurrentYearMaxEnergyCarrierDemand is the maximum energy carrier demand reported during a year from the meter. For heat and cooling meters this is the maximum flow rate on the inlet reported in a year
// @DH	currentYearMinEnergyCarrierDemand						(Id 0x0416) CurrentYearMinEnergyCarrierDemand is the minimum energy carrier demand reported during a year from the heat meter. For heat and cooling meters this is the minimum flow rate on the inlet reported in a year. Note: This attribute may be used to detect leaks if there has been no flow rate of zero in the last year
// @DH	maxNumberOfPeriodsDelivered								(Id 0x0500) MaxNumberofPeriodsDelivered represents the maximum number of intervals the device is capable of returning in one Get Profile Response command. It is required MaxNumberofPeriodsDelivered fit within the default Fragmentation ASDU size of 128 bytes, or an optionally agreed upon larger Fragmentation ASDU size supported by both devices. Please refer to [Z1] for further details on Fragmentation settings.
// @DH	currentDemandDelivered									(Id 0x0600) CurrentDemandDelivered represents the current Demand of Energy, Gas, or Water delivered at the premises. CurrentDemandDelivered may be continuously updated as new measurements are acquired, but at a minimum CurrentDemandDelivered must be updated at the end of each integration sub- period, which can be obtained by dividing the DemandIntegrationPeriod by the NumberOfDemandSubintervals. This attribute shall be adjusted using the Multiplier and Divisor attributes found in the Formatting Attribute Set and can be formatted using the DemandFormatting attribute. The final result represents an engineering value in the unit defined by the UnitofMeasure attribute.
// @DH	demandLimit												(Id 0x0601) DemandLimit reflects the current supply demand limit set in the meter. This value can be compared to the CurrentDemandDelivered attribute to understand if limits are being approached or exceeded. Adjustment and formatting of this attribute follow the same rules as the CurrentDemandDelivered. A value of “0xFFFFFF” indicates “demand limiting” is switched off .
// @DH	demandIntegrationPeriod									(Id 0x0602) DemandIntegrationPeriod is the number of minutes over which the CurrentDemandDelivered attribute is calculated. Valid range is 0x01 to 0xFF. 0x00 is a reserved value
// @DH	numberOfDemandSubintervals								(Id 0x0603) NumberOfDemandSubintervals represents the number of subintervals used within the DemandIntegrationPeriod. The subinterval duration (in minutes) is obtained by dividing the DemandIntegrationPeriod by the NumberOfDemandSubintervals. The CurrentDemandDelivered attribute is updated at each subinterval. Valid range is 0x01 to 0xFF. 0x00 is a reserved value. As a Rolling Demand example, DemandIntegrationPeriod could be set at 30 (for 30 minute period) and NumberOfDemandSubintervals could be set for 6. This would provide 5 minute (30/6 = 5) subinterval periods. As a Block Demand example, DemandIntegrationPeriod could be set at 30 (for 30 minute period) and NumberOfDemandSubintervals could be set for 1. This would provide a single 30 minute subinterval p e riod.
// @DH	currentNoTierBlockXSummationDelivered					(Id 0x070X) (X - 0 .. 15) Attributes CurrentNoTierBlock1SummationDelivered through CurrentTier15Block16SummationDelivered represent the most recent summed value of Energy, Gas, or Water delivered to the premises (i.e. delivered to the customer from the utility) at a specific price tier as defined by a TOU schedule, Block Threshold or a real time pricing period. If optionally provided, attributes CurrentNoTierBlock1SummationDelivered through CurrentTier15Block16SummationDelivered are updated continuously as new measurements are made. Note: SummationFormatting shall be used against the Block Information attribute set. The expected practical limit for the number of Block attributes supported is 32. The CurrentTierNBlockNSummationDelivered attributes are reset at the start of each Block Threshold Period.
// @DH	currentTierYBlockXSummationDelivered					(Id 0x07YX) (X - 0 .. 15) (Y - 1 .. 15) Attributes CurrentNoTierBlock1SummationDelivered through CurrentTier15Block16SummationDelivered represent the most recent summed value of Energy, Gas, or Water delivered to the premises (i.e. delivered to the customer from the utility) at a specific price tier as defined by a TOU schedule, Block Threshold or a real time pricing period. If optionally provided, attributes CurrentNoTierBlock1SummationDelivered through CurrentTier15Block16SummationDelivered are updated continuously as new measurements are made. Note: SummationFormatting shall be used against the Block Information attribute set. The expected practical limit for the number of Block attributes supported is 32. The CurrentTierNBlockNSummationDelivered attributes are reset at the start of each Block Threshold Period.
// @DH	genericAlarmMask										(Id 0x0800) The AlarmMask attributes of the Alarm Attribute Set specify whether each of the alarms listed in the corresponding alarm group in Table 10-49 through Table 10-55 is enabled. When the bit number corresponding to the alarm number (minus the group offset) is set to 1, the alarm is enabled, else it is disabled. Bits not corresponding to a code in the respective table are reserved.
//				00 Check Meter
//				01 Low Battery
//				02 Tamper Detect
//				03 Electricity: Power Failure
//					Gas: Not Defined
//					Water: Pipe Empty
//					Heat/Cooling: Temperature Sensor
//				04 Electricity: Power Quality
//					Gas: Low Pressure
//					Water: Low Pressure
//					Heat/Cooling: Burst Detect
//				05 Leak Detect
//				06 Service Disconnect
//				07 Electricity: Reserved
//					Gas: Reverse Flow
//					Water: Reverse Flow
//					Heat/Cooling: Flow Sensor
//				08-0F Reserved
// @DH	electricityAlarmMask									(Id 0x0801) The alarm codes are organized in logical groups corresponding to the meter type as listed in Table. The three main alarm groups are: Generic, Electricity, and Flow/ Pressure. The Flow/Pressure Alarm Group is further divided into Generic Flow/Pressure, Water Specific, Heat and Cooling Specific, and Gas Specific. It is left for the manufacturer to select which (if any) alarm codes to support.
//				10 Low Voltage L1
//				11 High Voltage L1
//				12 Low Voltage L2
//				13 High Voltage L2
//				14 Low Voltage L3
//				15 High Voltage L3
//				16 Over Current L1
//				17 Over Current L2
//				18 Over Current L3
//				19 Frequency too Low L1
//				1A Frequency too High L1
//				1B Frequency too Low L2
//				1C Frequency too High L2
//				1D Frequency too Low L3
//				1E Frequency too High L3
//				1F Ground Fault
//				20 Electric Tamper Detect
//				21-2F Reserved
// @DH	genericFlowPressureAlarmMask							(Id 0x0802)
//				30 Burst detect
//				31 Pressure too low
//				32 Pressure too high
//				33 Flow sensor communication error
//				34 Flow sensor measurement fault
//				35 Flow sensor reverse flow
//				36 Flow sensor air detect
//				37 Pipe empty
//				38-3F Reserved
// @DH	waterSpecificAlarmMask									(Id 0x0803)
//				40-4F Reserved
// @DH	heatandCoolingSpecificAlarmMask							(Id 0x0804)
//				50 Inlet Temperature Sensor Fault
//				51 Outlet Temperature Sensor Fault
//				52-5F Reserved
// @DH	gasSpecificAlarmMask									(Id 0x0805)
//				60-6F Reserved

// Get Profile
//
// @param: zbee
// ZBee object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: interval_channel
// Interval Channel
//
// @param: end_time
// End Time
//
// @param: number_of_periods
// Number Of Periods
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_metering_get_profile(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBYTE interval_channel, int end_time, ZWBYTE number_of_periods, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Request Fast Poll Mode
//
// @param: zbee
// ZBee object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: fast_poll_update_period
// Fast Poll Update Period
//
// @param: duration
// Duration
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_metering_request_fast_poll_mode(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWBYTE fast_poll_update_period, ZWBYTE duration, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Request Mirror
//
// @param: zbee
// ZBee object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_metering_request_mirror(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Request Fast Poll Mode
//
// @param: zbee
// ZBee object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_metering_remove_mirror(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Set Attribute Reporting
//
// @param: zbee
// ZBee object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: attr_id
// Attribute Id
//
// @param: direction
// TRUE  - receiver will be configured to receiver reports
// FALSE - receiver will be configured to send reports
//
// @param: min_interval
// Minimal Reporting Interval
//
// @param: max_interval
// Maximum Reporting Interval
//
// @param: timeout
// Timeout
//
// @param: threshold
// Threshold
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_metering_set_attr_reporting(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWWORD attr_id, ZWBOOL direction, ZWWORD min_interval, ZWWORD max_interval, ZWWORD timeout, ZWDWORD64 threshold, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Set Attribute Reporting
//
// @param: zbee
// ZBee object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: attr_id
// Attribute Id
//
// @param: direction
// TRUE  - receiver will be configured to receiver reports
// FALSE - receiver will be configured to send reports
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_metering_reset_attr_reporting(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWWORD attr_id, ZWBOOL direction, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);


// Cluster Class ElectricalMeasurement (0x0B04/2820) //
// Profile Zigbee Device Profile (0x0104/260) //
// This cluster provides a mechanism for querying data about the electrical properties as measured by the device. This cluster may be implemented on any device type and be implemented on a per-endpoint basis. For example, a power strip device could represent each outlet on a different endpoint and report electrical information for each individual outlet. The only caveat is that if you implement an attribute that has an associated multiplier and divisor, then you must implement the associated multiplier and divisor attributes. For example if you implement DCVoltage, you must also implement DCVoltageMultiplier and DCVoltageDivisor.  If you are interested in reading information about the power supply or battery level on the device, please see the Power Configuration cluster.
//
// @DH	dcVoltage								This attribute indicates a device’s measurement capabilities. This will be indicated by setting the desire measurement bits to 1, as mentioned in Table and DC Measurement
//				0 Active measurement (AC)
//				1 Reactive measurement (AC)
//				2 Apparent measurement (AC)
//				3 Phase A measurement
//				4 Phase B measurement
//				5 Phase C measurement
//				6 DC measurement
//				7 Harmonics measurement
//				8 Power quality measurement
// @DH	dcVoltageMin			    			The DCVoltageMin attribute represents the lowest DC voltage value measured in Volts (V). After resetting, this attribute will return a value of 0x8000 until a measurement is made					
// @DH	dcVoltageMax							The DCVoltageMax attribute represents the highest DC voltage value measured in Volts (V). After resetting, this attribute will return a value of 0x8000 until a measurement is made.
// @DH	dcCurrent								The DCCurrent attribute represents the most recent DC current reading in Amps (A). If the current cannot be measured, a value of 0x8000 is returned
// @DH	dcCurrentMin							The DCCurrentMin attribute represents the lowest DC current value measured in Amps (A). After resetting, this attribute will return a value of 0x8000 until a measurement is made.
// @DH	dcCurrentMax							The DCCurrentMax attribute represents the highest DC current value measured in Amps (A). After resetting, this attribute will return a value of 0x8000 until a measurement is made.
// @DH	dcPower									The DCPower attribute represents the most recent DC power reading in Watts (W). If the power cannot be measured, a value of 0x8000 is returned.
// @DH	dcPowerMin								The DCPowerMin attribute represents the lowest DC power value measured in Watts (W). After resetting, this attribute will return a value of 0x8000 until a measurement is made.
// @DH	dcPowerMax								The DCPowerMax attribute represents the highest DC power value measured in Watts (W). After resetting, this attribute will return a value of 0x8000 until a measurement is made.
// @DH	dcVoltageMultiplier						The DCVoltageMultiplier provides a value to be multiplied against the DCVoltage, DCVoltageMin, and DCVoltageMax attributes. This attribute must be used in conjunction with the DCVoltageDivisor attribute. 0x0000 is an invalid value for this attribute.
// @DH	dcVoltageDivisor						The DCVoltageDivisor provides a value to be divided against the DCVoltage, DCVoltageMin, and DCVoltageMax attributes. This attribute must be used in conjunction with the DCVoltageMultiplier attribute. 0x0000 is an invalid value for this attribute.
// @DH	dcCurrentMultiplier						The DCCurrentMultiplier provides a value to be multiplied against the DCCurrent, DCCurrentMin, and DCCurrentMax attributes. This attribute must be used in conjunction with the DCCurrentDivisor attribute. 0x0000 is an invalid value for this attribute.
// @DH	dcCurrentDivisor						The DCCurrentDivisor provides a value to be divided against the DCCurrent, DCCurrentMin, and DCCurrentMax attributes. This attribute must be used in conjunction with the DCCurrentMultiplier attribute. 0x0000 is an invalid value for this attribute.
// @DH	dcPowerMultiplier						The DCPowerMultiplier provides a value to be multiplied against the DCPower, DCPowerMin, and DCPowerMax attributes. This attribute must be used in conjunction with the DCPowerDivisor attribute. 0x0000 is an invalid value for this attribute.
// @DH	dcPowerDivisor							The DCPowerDivisor provides a value to be divided against the DCPower, DCPowerMin, and DCPowerMax attributes. This attribute must be used in conjunction with the DCPowerMultiplier attribute. 0x0000 is an invalid value for this attribute.
// @DH	acFrequency								The ACFrequency attribute represents the most recent AC Frequency reading in Hertz (Hz). If the frequency cannot be measured, a value of 0xFFFF is returned.
// @DH	acFrequencyMin							The ACFrequencyMin attribute represents the lowest AC Frequency value measured in Hertz (Hz). After resetting, this attribute will return a value of 0xFFFF until a measurement is made.
// @DH	acFrequencyMax							The ACFrequencyMax attribute represents the highest AC Frequency value measured in Hertz (Hz). After resetting, this attribute will return a value of 0xFFFF until a measurement is made.
// @DH	neutralCurrent							The NeutralCurrent attribute represents the AC neutral (Line-Out) current value at the moment in time the attribute is read, in Amps (A). If the instantaneous current cannot be measured, a value of 0xFFFF is returned
// @DH	totalActivePower						Active power represents the current demand of active power delivered or received at the premises, in kW. Positive values indicate power delivered to the premises where negative values indicate power received from the premises. In case if device is capable of measuring multi elements or phases then this will be net active power value
// @DH	totalReactivePower						Reactive power represents the current demand of reactive power delivered or received at the premises, in kVAr. Positive values indicate power delivered to the premises where negative values indicate power received from the premises. In case if device is capable of measuring multi elements or phases then this will be net reactive power value.
// @DH	totalApparentPower						Represents the current demand of apparent power, in kVA. In case if device is capable of measuring multi elements or phases then this will be net apparent power value.
// @DH	measured1stHarmonicCurrent				The Measured 1st HarmonicCurrent through MeasuredNthHarmonicCurrent attributes represent the most recent Nth harmonic current reading in an AC frequency. The unit for this measurement is 10^Nth HarmonicCurrentMultiplier amperes. If Nth HarmonicCurrentMultiplier is not implemented the unit is in amperes. If the Nth harmonic current cannot be measured a value of 0x8000 is returned. A positive value indicates the measured Nth harmonic current is positive, and a negative value indicates that the measured Nth harmonic current is negative
// @DH	measured3rdHarmonicCurrent				The Measured 1st HarmonicCurrent through MeasuredNthHarmonicCurrent attributes represent the most recent Nth harmonic current reading in an AC frequency. The unit for this measurement is 10^Nth HarmonicCurrentMultiplier amperes. If Nth HarmonicCurrentMultiplier is not implemented the unit is in amperes. If the Nth harmonic current cannot be measured a value of 0x8000 is returned. A positive value indicates the measured Nth harmonic current is positive, and a negative value indicates that the measured Nth harmonic current is negative
// @DH	measured5thHarmonicCurrent				The Measured 1st HarmonicCurrent through MeasuredNthHarmonicCurrent attributes represent the most recent Nth harmonic current reading in an AC frequency. The unit for this measurement is 10^Nth HarmonicCurrentMultiplier amperes. If Nth HarmonicCurrentMultiplier is not implemented the unit is in amperes. If the Nth harmonic current cannot be measured a value of 0x8000 is returned. A positive value indicates the measured Nth harmonic current is positive, and a negative value indicates that the measured Nth harmonic current is negative
// @DH	measured7thHarmonicCurrent				The Measured 1st HarmonicCurrent through MeasuredNthHarmonicCurrent attributes represent the most recent Nth harmonic current reading in an AC frequency. The unit for this measurement is 10^Nth HarmonicCurrentMultiplier amperes. If Nth HarmonicCurrentMultiplier is not implemented the unit is in amperes. If the Nth harmonic current cannot be measured a value of 0x8000 is returned. A positive value indicates the measured Nth harmonic current is positive, and a negative value indicates that the measured Nth harmonic current is negative
// @DH	measured9thHarmonicCurrent				The Measured 1st HarmonicCurrent through MeasuredNthHarmonicCurrent attributes represent the most recent Nth harmonic current reading in an AC frequency. The unit for this measurement is 10^Nth HarmonicCurrentMultiplier amperes. If Nth HarmonicCurrentMultiplier is not implemented the unit is in amperes. If the Nth harmonic current cannot be measured a value of 0x8000 is returned. A positive value indicates the measured Nth harmonic current is positive, and a negative value indicates that the measured Nth harmonic current is negative
// @DH	measured11thHarmonicCurrent				The Measured 1st HarmonicCurrent through MeasuredNthHarmonicCurrent attributes represent the most recent Nth harmonic current reading in an AC frequency. The unit for this measurement is 10^Nth HarmonicCurrentMultiplier amperes. If Nth HarmonicCurrentMultiplier is not implemented the unit is in amperes. If the Nth harmonic current cannot be measured a value of 0x8000 is returned. A positive value indicates the measured Nth harmonic current is positive, and a negative value indicates that the measured Nth harmonic current is negative
// @DH	measuredPhase1stHarmonicCurrent			The Measured Phase 1st HarmonicCurrent through MeasuredPhaseNthHarmonicCurrent attributes represent the most recent phase of the Nth harmonic current reading in an AC frequency. The unit for this measurement is 10^Phase * Nth HarmonicCurrentMultiplier degree. If PhaseNthHarmonicCurrentMultiplier is not implemented the unit is in degree. If the phase of the Nth harmonic current cannot be measured a value of 0x8000 is returned. A positive value indicates the measured phase of the Nth harmonic current is prehurry, and a negative value indicates that the measured phase of the Nth harmonic current is lagging.	
// @DH	measuredPhase3rdHarmonicCurrent			The Measured Phase 1st HarmonicCurrent through MeasuredPhaseNthHarmonicCurrent attributes represent the most recent phase of the Nth harmonic current reading in an AC frequency. The unit for this measurement is 10^Phase * Nth HarmonicCurrentMultiplier degree. If PhaseNthHarmonicCurrentMultiplier is not implemented the unit is in degree. If the phase of the Nth harmonic current cannot be measured a value of 0x8000 is returned. A positive value indicates the measured phase of the Nth harmonic current is prehurry, and a negative value indicates that the measured phase of the Nth harmonic current is lagging.	
// @DH	measuredPhase5thHarmonicCurrent			The Measured Phase 1st HarmonicCurrent through MeasuredPhaseNthHarmonicCurrent attributes represent the most recent phase of the Nth harmonic current reading in an AC frequency. The unit for this measurement is 10^Phase * Nth HarmonicCurrentMultiplier degree. If PhaseNthHarmonicCurrentMultiplier is not implemented the unit is in degree. If the phase of the Nth harmonic current cannot be measured a value of 0x8000 is returned. A positive value indicates the measured phase of the Nth harmonic current is prehurry, and a negative value indicates that the measured phase of the Nth harmonic current is lagging.	
// @DH	measuredPhase7thHarmonicCurrent			The Measured Phase 1st HarmonicCurrent through MeasuredPhaseNthHarmonicCurrent attributes represent the most recent phase of the Nth harmonic current reading in an AC frequency. The unit for this measurement is 10^Phase * Nth HarmonicCurrentMultiplier degree. If PhaseNthHarmonicCurrentMultiplier is not implemented the unit is in degree. If the phase of the Nth harmonic current cannot be measured a value of 0x8000 is returned. A positive value indicates the measured phase of the Nth harmonic current is prehurry, and a negative value indicates that the measured phase of the Nth harmonic current is lagging.	
// @DH	measuredPhase9thHarmonicCurrent			The Measured Phase 1st HarmonicCurrent through MeasuredPhaseNthHarmonicCurrent attributes represent the most recent phase of the Nth harmonic current reading in an AC frequency. The unit for this measurement is 10^Phase * Nth HarmonicCurrentMultiplier degree. If PhaseNthHarmonicCurrentMultiplier is not implemented the unit is in degree. If the phase of the Nth harmonic current cannot be measured a value of 0x8000 is returned. A positive value indicates the measured phase of the Nth harmonic current is prehurry, and a negative value indicates that the measured phase of the Nth harmonic current is lagging.		
// @DH	measuredPhase11thHarmonicCurrent		The Measured Phase 1st HarmonicCurrent through MeasuredPhaseNthHarmonicCurrent attributes represent the most recent phase of the Nth harmonic current reading in an AC frequency. The unit for this measurement is 10^Phase * Nth HarmonicCurrentMultiplier degree. If PhaseNthHarmonicCurrentMultiplier is not implemented the unit is in degree. If the phase of the Nth harmonic current cannot be measured a value of 0x8000 is returned. A positive value indicates the measured phase of the Nth harmonic current is prehurry, and a negative value indicates that the measured phase of the Nth harmonic current is lagging.				
// @DH	acFrequencyMultiplier					Provides a value to be multiplied against the ACFrequency attribute. This attribute must be used in conjunction with the ACFrequencyDivisor attribute. 0x0000 is an invalid value for this attribute
// @DH	acFrequencyDivisor						Provides a value to be divided against the ACFrequency attribute. This attribute must be used in conjunction with the ACFrequencyMultiplier attribute. 0x0000 is an invalid value for this attribute.
// @DH	powerMultiplier							Provides a value to be multiplied against a raw or uncompensated sensor count of power being measured by the metering device. If present, this attribute must be applied against all power/demand values to derive the delivered and received values expressed in the specified units. This attribute must be used in conjunction with the PowerDivisor attribute.
// @DH	powerDivisor							Provides a value to divide against the results of applying the Multiplier attribute against a raw or uncompensated sensor count of power being measured by the metering device. If present, this attribute must be applied against all demand/power values to derive the delivered and received values expressed in the specified units. This attribute must be used in conjunction with the PowerMultiplier attribute.
// @DH	harmonicCurrentMultiplier				Represents the unit value for the MeasuredNthHarmonicCurrent attribute in the format MeasuredNthHarmonicCurrent * 10 ^ HarmonicCurrentMultiplier amperes.
// @DH	phaseHarmonicCurrentMultiplier			Represents the unit value for the MeasuredPhaseNthHarmonicCurrent attribute in the format MeasuredPhaseNthHarmonicCurrent * 10 ^ PhaseHarmonicCurrentMultiplier degrees.
// @DH	lineCurrent								Represents the single phase or Phase A, AC line current (Square root of active and reactive current) value at the moment in time the attribute is read, in Amps (A). If the instantaneous current cannot be measured, a value of 0x8000 is returned.
// @DH	activeCurrent							Represents the single phase or Phase A, AC active/resistive current value at the moment in time the attribute is read, in Amps (A). Positive values indicate power delivered to the premises where negative values indicate power received from the premises.
// @DH	reactiveCurrent							Represents the single phase or Phase A, AC reactive current value at the moment in time the attribute is read, in Amps (A). Positive values indicate power delivered to the premises where negative values indicate power received from the premises.
// @DH	rmsVoltage								Represents the most recent RMS voltage reading in Volts (V). If the RMS voltage cannot be measured, a value of 0xFFFF is returned.
// @DH	rmsVoltageMin							Represents the lowest RMS voltage value measured in Volts (V). After resetting, this attribute will return a value of 0xFFFF until a measurement is made.
// @DH	rmsVoltageMax							Represents the highest RMS voltage value measured in Volts (V). After resetting, this attribute will return a value of 0xFFFF until a measurement is made.
// @DH	rmsCurrent								Represents the most recent RMS current reading in Amps (A). If the power cannot be measured, a value of 0xFFFF is returned.
// @DH	rmsCurrentMin							Represents the lowest RMS current value measured in Amps (A). After resetting, this attribute will return a value of 0xFFFF until a measurement is made.
// @DH	rmsCurrentMax							Represents the highest RMS current value measured in Amps (A). After resetting, this attribute will return a value of 0xFFFF until a measurement is made.
// @DH	activePower								Represents the single phase or Phase A, current demand of active power delivered or received at the premises, in Watts (W). Positive values indicate power delivered to the premises where negative values indicate power received from the premises
// @DH	activePowerMin							Represents the lowest AC power value measured in Watts (W). After resetting, this attribute will return a value of 0x8000 until a measurement is made.
// @DH	activePowerMax							Represents the highest AC power value measured in Watts (W). After resetting, this attribute will return a value of 0x8000 until a measurement is made.
// @DH	reactivePower							Represents the single phase or Phase A, current demand of reactive power delivered or received at the premises, in VAr. Positive values indicate power delivered to the premises where negative values indicate power received from the premises.
// @DH	apparentPower							Represents the single phase or Phase A, current demand of apparent (Square root of active and reactive power) power, in VA.
// @DH	powerFactor								Contains the single phase or PhaseA, Power Factor ratio in 1/100ths.
// @DH	averageRmsVoltageMeasurementPeriod		The Period in seconds that the RMS voltage is averaged over.
// @DH	averageRmsOverVoltageCounter			The number of times the average RMS voltage, has been above the AverageRMS OverVoltage threshold since last reset. This counter may be reset by writing zero to the attribute.
// @DH	averageRmsUnderVoltageCounter			The number of times the average RMS voltage, has been below the AverageRMS underVoltage threshold since last reset. This counter may be reset by writing zero to the attribute.
// @DH	rmsExtremeOverVoltagePeriod				The duration in seconds used to measure an extreme over voltage condition.
// @DH	rmsExtremeUnderVoltagePeriod			The duration in seconds used to measure an extreme under voltage condition.
// @DH	rmsVoltageSagPeriod						The duration in seconds used to measure a voltage sag condition.
// @DH	rmsVoltageSwellPeriod					The duration in seconds used to measure a voltage swell condition.
// @DH	acVoltageMultiplier						Provides a value to be multiplied against the InstantaneousVoltage and RMSVoltage attributes. This attribute must be used in conjunction with the ACVoltageDivisor attribute. 0x0000 is an invalid value for this attribute.
// @DH	acVoltageDivisor						Provides a value to be divided against the InstantaneousVoltage and RMSVoltage attributes. This attribute must be used in conjunction with the ACVoltageMultiplier attribute. 0x0000 is an invalid value for this attribute.
// @DH	acCurrentMultiplier						Provides a value to be multiplied against the InstantaneousCurrent and RMSCurrent attributes. This attribute must be used in conjunction with the ACCurrentDivisor attribute. 0x0000 is an invalid value for this attribute.
// @DH	acCurrentDivisor						Provides a value to be divided against the ACCurrent, InstantaneousCurrent and RMSCurrent attributes. This attribute must be used in conjunction with the ACCurrentMultiplier attribute. 0x0000 is an invalid value for this attribute.
// @DH	acPowerMultiplier						Provides a value to be multiplied against the InstantaneousPower and ActivePower attributes. This attribute must be used in conjunction with the ACPowerDivisor attribute. 0x0000 is an invalid value for this attribute.
// @DH	acPowerDivisor							Provides a value to be divided against the InstantaneousPower and ActivePower attributes. This attribute must be used in conjunction with the ACPowerMultiplier attribute. 0x0000 is an invalid value for this attribute.
// @DH	dcOverloadAlarmsMask					Specifies which configurable alarms may be generated, as listed in Figure. A ‘1’ in each bit position enables the alarm
//				0th bit Voltage Overload
//				1st bit Current Overload
// @DH	dcVoltageOverload						Specifies the alarm threshold, set by the manufacturer, for the maximum output voltage supported by device. The value is multiplied and divided by the DCVoltageMultiplier the DCVoltageDivisor respectively.
// @DH	dcCurrentOverload						Specifies the alarm threshold, set by the manufacturer, for the maximum output current supported by device. The value is multiplied and divided by the DCCurrentMultiplier and DCCurrentDivider respectively.
// @DH	acAlarmsMask							Specifies which configurable alarms may be generated, as listed in Figure. A ‘1’ in each bit position enables the alarm.
//				0th bit Voltage Overload
//				1st bit Current Overload
//				2nd bit Active Power Overload
//				3rd bit Reactive Power Overload
//				4th bit Average RMS Over Voltage
//				5th bit Average RMS Under Voltage
//				6th bit RMS Extreme Over Voltage
//				7th bit RMS Extreme Under Voltage
//				8th bit RMS Voltage Sag
//				9th bit RMS Voltage Swell
// @DH	acVoltageOverload						Specifies the alarm threshold, set by the manufacturer, for the maximum output voltage supported by device. The value is multiplied and divided by the ACVoltageMultiplier the ACVoltageDivisor, respectively. The value is voltage RMS.
// @DH	acCurrentOverload						Specifies the alarm threshold, set by the manufacturer, for the maximum output current supported by device. The value is multiplied and divided by the ACCurrentMultiplier and ACCurrentDivider, respectively. The value is current RMS
// @DH	acActivePowerOverload					Specifies the alarm threshold, set by the manufacturer, for the maximum output active power supported by device. The value is multiplied and divided by the ACPowerMultiplier and ACPowerDivisor, respectively.
// @DH	acReactivePowerOverload					Specifies the alarm threshold, set by the manufacturer, for the maximum output reactive power supported by device. The value is multiplied and divided by the ACPowerMultiplier and ACPowerDivisor, respectively.
// @DH	averageRmsOverVoltage					The average RMS voltage above which an over voltage condition is reported. The threshold shall be configurable within the specified operating range of the electricity meter. The value is multiplied and divided by the ACVoltageMultiplier and ACVoltageDivisor, respectively
// @DH	averageRmsUnderVoltage					The average RMS voltage below which an under voltage condition is reported. The threshold shall be configurable within the specified operating range of the electricity meter. The value is multiplied and divided by the ACVoltageMultiplier and ACVoltageDivisor, respectively.
// @DH	rmsExtremeOverVoltage					The RMS voltage above which an extreme under voltage condition is reported. The threshold shall be configurable within the specified operating range of the electricity meter. The value is multiplied and divided by the ACVoltageMultiplier and ACVoltageDivisor, respectively.
// @DH	rmsExtremeUnderVoltage					The RMS voltage below which an extreme under voltage condition is reported. The threshold shall be configurable within the specified operating range of the electricity meter. The value is multiplied and divided by the ACVoltageMultiplier and ACVoltageDivisor, respectively.
// @DH	rmsVoltageSag							The RMS voltage below which a sag condition is reported. The threshold shall be configurable within the specified operating range of the electricity meter. The value is multiplied and divided by the ACVoltageMultiplier and ACVoltageDivisor, respectively.
// @DH	rmsVoltageSwell							The RMS voltage above which a swell condition is reported. The threshold shall be configurable within the specified operating range of the electricity meter. The value is multiplied and divided by the ACVoltageMultiplier and ACVoltageDivisor, respectively.
// @DH	lineCurrentPhB							Represents the Phase B, AC line current (Square root sum of active and reactive currents) value at the moment in time the attribute is read, in Amps (A). If the instantaneous current cannot be measured, a value of 0x8000 is returned.
// @DH	activeCurrentPhB						Represents the Phase B, AC active/resistive current value at the moment in time the attribute is read, in Amps (A). Positive values indicate power delivered to the premises where negative values indicate power received from the premises.
// @DH	reactiveCurrentPhB						Represents the Phase B, AC reactive current value at the moment in time the attribute is read, in Amps (A). Positive values indicate power delivered to the premises where negative values indicate power received from the premises.
// @DH	rmsVoltagePhB							Represents the most recent RMS voltage reading in Volts (V). If the RMS voltage cannot be measured, a value of 0xFFFF is returned.
// @DH	rmsVoltageMinPhB						Represents the lowest RMS voltage value measured in Volts (V). After resetting, this attribute will return a value of 0xFFFF until a measurement is made.
// @DH	rmsVoltageMaxPhB						Represents the highest RMS voltage value measured in Volts (V). After resetting, this attribute will return a value of 0xFFFF until a measurement is made
// @DH	rmsCurrentPhB							Represents the most recent RMS current reading in Amps (A). If the power cannot be measured, a value of 0xFFFF is returned.
// @DH	rmsCurrentMinPhB						Represents the lowest RMS current value measured in Amps (A). After resetting, this attribute will return a value of 0x8000 until a measurement is made.
// @DH	rmsCurrentMaxPhB						Represents the highest RMS current value measured in Amps (A). After resetting, this attribute will return a value of 0x8000 until a measurement is made
// @DH	activePowerPhB							Represents the Phase B, current demand of active power delivered or received at the premises, in Watts (W). Positive values indicate power delivered to the premises where negative values indicate power received from the premises.
// @DH	activePowerMinPhB						Represents the lowest AC power value measured in Watts (W). After resetting, this attribute will return a value of 0x8000 until a measurement is made.
// @DH	activePowerMaxPhB						Represents the highest AC power value measured in Watts (W). After resetting, this attribute will return a value of 0x8000 until a measurement is made.
// @DH	reactivePowerPhB						Represents the Phase B, current demand of reactive power delivered or received at the premises, in VAr. Positive values indicate power delivered to the premises where negative values indicate power received from the premises.
// @DH	apparentPowerPhB						Represents the Phase B, current demand of apparent (Square root of active and reactive power) power, in VA.
// @DH	powerFactorPhB							Contains the PhaseB, Power Factor ratio in 1/100ths.
// @DH	averageRmsVoltageMeasurementPeriodPh	The Period in seconds that the RMS voltage is averaged over.
// @DH	averageRmsOverVoltageCounterPhB			The number of times the average RMS voltage, has been above the AverageRMS OverVoltage threshold since last reset. This counter may be reset by writing zero to the attribute.
// @DH	averageRmsUnderVoltageCounterPhB		The number of times the average RMS voltage, has been below the AverageRMS underVoltage threshold since last reset. This counter may be reset by writing zero to the attribute.
// @DH	rmsExtremeOverVoltagePeriodPhB			The duration in seconds used to measure an extreme over voltage condition.
// @DH	rmsExtremeUnderVoltagePeriodPhB			The duration in seconds used to measure an extreme under voltage condition.
// @DH	rmsVoltageSagPeriodPhB					The duration in seconds used to measure a voltage sag condition
// @DH	rmsVoltageSwellPeriodPhB				The duration in seconds used to measure a voltage swell condition.
// @DH	lineCurrentPhC							Represents the Phase B, AC line current (Square root sum of active and reactive currents) value at the moment in time the attribute is read, in Amps (A). If the instantaneous current cannot be measured, a value of 0x8000 is returned.
// @DH	activeCurrentPhC						Represents the Phase B, AC active/resistive current value at the moment in time the attribute is read, in Amps (A). Positive values indicate power delivered to the premises where negative values indicate power received from the premises.
// @DH	reactiveCurrentPhC						Represents the Phase B, AC reactive current value at the moment in time the attribute is read, in Amps (A). Positive values indicate power delivered to the premises where negative values indicate power received from the premises.
// @DH	rmsVoltagePhC							Represents the most recent RMS voltage reading in Volts (V). If the RMS voltage cannot be measured, a value of 0xFFFF is returned.
// @DH	rmsVoltageMinPhC						Represents the lowest RMS voltage value measured in Volts (V). After resetting, this attribute will return a value of 0xFFFF until a measurement is made.
// @DH	rmsVoltageMaxPhC						Represents the highest RMS voltage value measured in Volts (V). After resetting, this attribute will return a value of 0xFFFF until a measurement is made
// @DH	rmsCurrentPhC							Represents the most recent RMS current reading in Amps (A). If the power cannot be measured, a value of 0xFFFF is returned.
// @DH	rmsCurrentMinPhC						Represents the lowest RMS current value measured in Amps (A). After resetting, this attribute will return a value of 0x8000 until a measurement is made.
// @DH	rmsCurrentMaxPhC						Represents the highest RMS current value measured in Amps (A). After resetting, this attribute will return a value of 0x8000 until a measurement is made
// @DH	activePowerPhC							Represents the Phase B, current demand of active power delivered or received at the premises, in Watts (W). Positive values indicate power delivered to the premises where negative values indicate power received from the premises.
// @DH	activePowerMinPhC						Represents the lowest AC power value measured in Watts (W). After resetting, this attribute will return a value of 0x8000 until a measurement is made.
// @DH	activePowerMaxPhC						Represents the highest AC power value measured in Watts (W). After resetting, this attribute will return a value of 0x8000 until a measurement is made.
// @DH	reactivePowerPhC						Represents the Phase B, current demand of reactive power delivered or received at the premises, in VAr. Positive values indicate power delivered to the premises where negative values indicate power received from the premises.
// @DH	apparentPowerPhC						Represents the Phase B, current demand of apparent (Square root of active and reactive power) power, in VA.
// @DH	powerFactorPhC							Contains the PhaseB, Power Factor ratio in 1/100ths.
// @DH	averageRmsVoltageMeasurementPeriodPc	The Period in seconds that the RMS voltage is averaged over.
// @DH	averageRmsOverVoltageCounterPhC			The number of times the average RMS voltage, has been above the AverageRMS OverVoltage threshold since last reset. This counter may be reset by writing zero to the attribute.
// @DH	averageRmsUnderVoltageCounterPhC		The number of times the average RMS voltage, has been below the AverageRMS underVoltage threshold since last reset. This counter may be reset by writing zero to the attribute.
// @DH	rmsExtremeOverVoltagePeriodPhC			The duration in seconds used to measure an extreme over voltage condition.
// @DH	rmsExtremeUnderVoltagePeriodPhC			The duration in seconds used to measure an extreme under voltage condition.
// @DH	rmsVoltageSagPeriodPhC					The duration in seconds used to measure a voltage sag condition
// @DH	rmsVoltageSwellPeriodPhC				The duration in seconds used to measure a voltage swell condition.

// Get Profile Info Command
//
// @param: zbee
// ZBee object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_electrical_measurement_get_profile_info_command(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Get Measurement Profile Command
//
// @param: zbee
// ZBee object instance
//
// @param: node_id
// Destination Node Id
//
// @param: endpoint_id
// Destination EndPoint Id
//
// @param: attr_id
// Attribute Id
//
// @param: start_time
// Start Time
//
// @param: num_of_intervals
// Number of Intervals
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function succes or failure
//
ZWEXPORT ZWError zbee_cc_electrical_measurement_get_measurement_profile_command(ZBee zbee, ZBNWK node_id, ZWBYTE endpoint_id, ZWWORD attr_id, ZWBYTE start_time, ZWBYTE num_of_intervals, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

#endif
